/**
 * Class and function collection for WBB
 * 
 * @author	Marcel Werk
 * @copyright 	2001-2011 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 */

/**
 * Initialize WBB namespace
 */
var WBB = {};

/**
 * Namespace for board related actions
 */
WBB.Board = {};

/**
 * Provides collapsible support for categories.
 * 
 * @see	WCF.Collapsible.Remote
 */
WBB.Board.Collapsible = WCF.Collapsible.Remote.extend({
	/**
	 * @see	WCF.Collapsible.Remote._getContainers()
	 */
	_getContainers: function() {
		return $('.wbbBoardList .wbbCollapsibleCategory');
	},
	
	/**
	 * @see	WCF.Collapsible.Remote._getTarget()
	 */
	_getTarget: function(containerID) {
		return this._containers[containerID].children('ul').first();
	},
	
	/**
	 * @see	WCF.Collapsible.Remote._getButtonContainer()
	 */
	_getButtonContainer: function(containerID) {
		return this._containers[containerID].find('header h2').first();
	},
	
	/**
	 * @see	WCF.Collapsible.Remote._getObjectID()
	 */
	_getObjectID: function(containerID) {
		return this._containers[containerID].data('boardID');
	},
	
	/**
	 * @see	WCF.Collapsible.Remote._updateContent()
	 */
	_updateContent: function(containerID, newContent, newState) {
		var $newContainer = $(newContent).attr('id', containerID);
		
		this._containers[containerID].replaceWith($newContainer);
		this._containers[containerID] = $newContainer;
		this._initContainer(containerID, $newContainer);
	},
	
	/**
	 * @see	WCF.Collapsible.Remote._getAdditionalParameters()
	 */
	_getAdditionalParameters: function(containerID) {
		return {
			depth: this._containers[containerID].data('depth')
		};
	},
});

/**
 * Marks all boards as read.
 * 
 * @param	object		callback
 */
WBB.Board.MarkAllAsRead = Class.extend({
	/**
	 * success callback function
	 * @var	object
	 */
	_callback: null,
	
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * Initializes the WBB.Board.MarkAllAsRead class.
	 * 
	 * @param	object		callback
	 */
	init: function(callback) {
		this._callback = callback;
		
		// initialize proxy
		this._proxy = new WCF.Action.Proxy({
			success: $.proxy(this._success, this)
		});
		
		// bind event listener
		$('.markAllAsReadButton').click($.proxy(this._click, this));
	},
	
	/**
	 * Handles clicks on the 'mark all as read' button.
	 */
	_click: function() {
		this._proxy.setOption('data', {
			actionName: 'markAllAsRead',
			className: 'wbb\\data\\board\\BoardAction'
		});
		
		this._proxy.sendRequest();
	},
	
	/**
	 * Marks all boards as read.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		if (this._callback && $.isFunction(this._callback)) {
			return this._callback();
		}
		
		var $boardList = $('.wbbBoardList');
		
		// remove new class
		$boardList.find('.new').removeClass('new');
		
		// remove badges
		$boardList.find('.badge.badgeUpdate').hide();
		$('#mainMenu .active .badge').hide(); // main menu badge
		
		// change icons
		$boardList.find('.wbbBoard .icon32.markAsRead').removeClass('icon-folder-close').addClass('icon-folder-close-alt');
		$boardList.find('.wbbSubBoards .icon16.markAsRead').removeClass('icon-folder-close').addClass('icon-folder-close-alt');
		
		// disable markAsRead
		$boardList.find('.markAsRead').removeClass('markAsRead').unbind('dblclick');
	}
});

/**
 * Marks a board as read.
 */
WBB.Board.MarkAsRead = Class.extend({
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * Initializes the WBB.Board.MarkAsRead class.
	 */
	init: function() {
		this._proxy = new WCF.Action.Proxy({
			success: $.proxy(this._success, this)
		});
		
		$(document).on('dblclick', '.wbbBoardList .markAsRead', $.proxy(this._dblClick, this));
	},
	
	/**
	 * Handles double-clicks on a board.
	 * 
	 * @param	object		event
	 */
	_dblClick: function(event) {
		this._proxy.setOption('data', {
			actionName: 'markAsRead',
			className: 'wbb\\data\\board\\BoardAction',
			objectIDs: [ $(event.currentTarget).parents('li:eq(0)').data('boardID') ]
		});
		this._proxy.sendRequest();
	},
	
	/**
	 * Marks a board as read.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		$('.wbbBoardList .markAsRead').each(function(index, element) {
			var $element = $(element);
			var $boardElement = $element.parents('li:eq(0)');
			if (WCF.inArray($boardElement.data('boardID'), data.objectIDs)) {
				// remove new class
				$boardElement.find('.new:eq(0)').removeClass('new');
				$element.removeClass('markAsRead');
				
				// remove badges
				$boardElement.find('.badge:eq(0)').hide();
				
				// change icons
				$element.removeClass('icon-folder-close').addClass('icon-folder-close-alt');
				
				// remove event
				$element.unbind('dblclick');
			}
		});
	}
});

/**
 * Marks all threads as read
 * 
 * @param	integer		boardID
 */
WBB.Board.MarkAllThreadsAsRead = Class.extend({
	/**
	 * board id
	 * @var	integer
	 */
	_boardID: 0,
	
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * Initializes the WBB.Board.MarkAllThreadsAsRead class.
	 * 
	 * @param	integer		boardID
	 */
	init: function(boardID) {
		this._boardID = boardID;
		this._proxy = new WCF.Action.Proxy({
			success: $.proxy(this._success, this)
		});
		
		$('.markAsReadButton').click($.proxy(this._click, this));
	},
	
	/**
	 * Handles clicks on the 'mark all threads as read' button.
	 */
	_click: function(event) {
		this._proxy.setOption('data', {
			actionName: 'markAsRead',
			className: 'wbb\\data\\board\\BoardAction',
			objectIDs: [ this._boardID ]
		});
		this._proxy.sendRequest();
	},
	
	/**
	 * Marks all threads within current board as read.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		$('.wbbThreadList .new').each(function(index, element) {
			var $element = $(element);
			
			// remove new class
			$element.removeClass('new');
			
			// hide arrows
			$element.find('.firstNewPost').parent().remove();
			
			// remove event
			$element.find('.columnAvatar').off('dblclick');
		});
	}
});

/**
 * Shows a dialog overlay to setup ignored boards.
 */
WBB.Board.IgnoreBoards = Class.extend({
	/**
	 * dialog overlay
	 * @var	jQuery
	 */
	_dialog: null,
	
	/**
	 * initialization state
	 * @var	boolean
	 */
	_didInit: false,
	
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * Initializes the WBB.Board.IgnoreBoards class.
	 */
	init: function() {
		$('.ignoreBoardsButton').click($.proxy(this._click, this));
	},
	
	/**
	 * Handles clicks on the 'ignore boards' button.
	 */
	_click: function(event) {
		if (!this._didInit) {
			this._dialog = $('<div />').hide().appendTo(document.body);
			this._proxy = new WCF.Action.Proxy({
				autoSend: true,
				data: {
					actionName: 'loadIgnoredBoards',
					className: 'wbb\\data\\board\\BoardAction'
				},
				success: $.proxy(this._success, this)
			});
			
			this._didInit = true;
		}
		else {
			this._dialog.wcfDialog('open');
		}
	},
	
	/**
	 * Initializes the ignore boards list.
	 */
	_initBoardList: function() {
		// submit event
		this._dialog.find('input[type=submit]').click($.proxy(this._submit, this));
		
		// checkbox handler
		this._dialog.find('input[type=checkbox]').click($.proxy(this._checkboxClick, this));
	},
	
	/**
	 * Handles clicks on a checkbox within the ignore board list.
	 * 
	 * @param	object		event
	 */
	_checkboxClick: function(event) {
		var $checkbox = $(event.currentTarget);
		if (!$checkbox.prop('checked')) {
			$checkbox.parents('li:eq(0)').find('ul input[type=checkbox]').each(function() {
				$(this).prop('checked', $checkbox.prop('checked')).disable();
			});
		}
		else {
			$checkbox.parents('li:eq(0)').find('ul input[type=checkbox]').each(function() {
				$(this).enable();
			});
		}
	},
	
	/**
	 * Submits the ignore boards form.
	 */
	_submit: function() {
		// get board ids
		var $boardIDs = [];
		this._dialog.find('input[type=checkbox]:not(:checked)').each(function() {
			$boardIDs.push($(this).val());
		});
		
		// disable submit button
		this._dialog.find('input[type=submit]').disable();
		
		// save board ids
		this._proxy.setOption('data', {
			actionName: 'saveIgnoredBoards',
			className: 'wbb\\data\\board\\BoardAction',
			objectIDs: $boardIDs
		});
		this._proxy.sendRequest();
	},
	
	/**
	 * Handles successful AJAX requests.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		if (data.actionName === 'loadIgnoredBoards') {
			this._dialog.html(data.returnValues.template);
			this._initBoardList();
			this._dialog.wcfDialog({
				'title': WCF.Language.get('wbb.index.ignoreBoards')
			});
		}
		else {
			// close dialog
			this._dialog.wcfDialog('close');
			
			// enable submit button
			this._dialog.find('input[type=submit]').enable();
			
			// update board list
			$('.wbbBoardList:eq(0)').html(data.returnValues.template);
		}
	}
});

/**
 * Namespace for thread related actions
 */
WBB.Thread = {};

/**
 * Marks one thread as read.
 */
WBB.Thread.MarkAsRead = Class.extend({
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * Initializes the mark as read for threads.
	 */
	init: function() {
		this._proxy = new WCF.Action.Proxy({
			success: $.proxy(this._success, this)
		});
		
		$(document).on('dblclick', '.wbbThreadList .new .columnAvatar', $.proxy(this._dblclick, this));
	},
	
	/**
	 * Handles double clicks on avatar.
	 * 
	 * @param	object		event
	 */
	_dblclick: function(event) {
		this._proxy.setOption('data', {
			actionName: 'markAsRead',
			className: 'wbb\\data\\thread\\ThreadAction',
			objectIDs: [ $(event.currentTarget).parents('tr:eq(0)').data('threadID') ]
		});
		this._proxy.sendRequest();
	},
	
	/**
	 * Handles successful AJAX requests.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		$('.wbbThreadList .new').each(function(index, element) {
			var $element = $(element);
			if (WCF.inArray($element.data('threadID'), data.objectIDs)) {
				// remove new class
				$element.removeClass('new');
				
				// hide arrows
				$element.find('.firstNewPost').parent().remove();
				
				// remove event
				$element.find('.columnAvatar').off('dblclick');
			}
		});
	}
});

/**
 * Inline editor for threads.
 * 
 * @param	string		environment
 * @param	integer		threadID
 */
WBB.Thread.Editor = {
	/**
	 * list of callbacks
	 * @var	array<object>
	 */
	_callbacks: [ ],
	
	/**
	 * editor dialog content
	 * @var	jQuery
	 */
	_dialog: null,
	
	/**
	 * editor dialog
	 * @var	jQuery
	 */
	_dialogContainer: null,
	
	/**
	 * current environment
	 * @var	string
	 */
	_environment: '',
	
	/**
	 * notification object
	 * @var	WCF.System.Notification
	 */
	_notification: null,
	
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * thread id
	 * @var	integer
	 */
	_threadID: 0,
	
	/**
	 * Fetches editor content on request.
	 * 
	 * @param	string		environment
	 * @param	integer		threadID
	 */
	beginEdit: function(environment, threadID) {
		this._environment = environment;
		this._threadID = threadID;
		
		if (this._proxy === null) {
			this._proxy = new WCF.Action.Proxy({
				success: $.proxy(this._success, this)
			});
		}
		
		this._proxy.setOption('data', {
			actionName: 'beginEdit',
			className: 'wbb\\data\\thread\\ThreadAction',
			parameters: {
				data: {
					threadID: this._threadID
				}
			}
		});
		this._proxy.sendRequest();
	},
	
	/**
	 * Registers a callback.
	 * 
	 * @param	object		callback
	 */
	registerCallback: function(callback) {
		this._callbacks.push(callback);
	},
	
	/**
	 * Handles AJAX repsonses.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		switch (data.returnValues.actionName) {
			case 'beginEdit':
				this._showEditor(data);
			break;
			
			case 'saveEdit':
				this._saveEdit(data);
			break;
		}
	},
	
	/**
	 * Displays the editor form overlay.
	 * 
	 * @param	object		data
	 */
	_showEditor: function(data) {
		if (this._dialog === null) {
			this._dialogContainer = $('<div id="threadEditor" />').data('wbbThreadEditor', this).appendTo(document.body);
			this._dialog = $('<div />').appendTo(this._dialogContainer);
			
			// add form controls
			var $formSubmit = $('<div class="formSubmit" />').appendTo(this._dialogContainer);
			var $saveButton = $('<button class="buttonPrimary">' + WCF.Language.get('wcf.global.button.save') + '</button>').appendTo($formSubmit);
			var $cancelButton = $('<button>' + WCF.Language.get('wcf.global.button.cancel') + '</button>').appendTo($formSubmit);
			
			$cancelButton.click($.proxy(this._cancel, this));
			$saveButton.click($.proxy(this._save, this));
		}
		
		this._dialogContainer.data('threadID', data.returnValues.threadID);
		this._dialog.html(data.returnValues.template);
		
		var self = this;
		this._dialog.find('input').keyup(function(event) {
			if (event.which === 13) { // Enter
				self._save();
				
				event.preventDefault();
				return false;
			}
		});
		
		// show dialog
		this._dialogContainer.wcfDialog({
			title: WCF.Language.get('wbb.thread.edit')
		});
		this._dialogContainer.wcfDialog('render');
	},
	
	/**
	 * Collects input values for save.
	 */
	_save: function() {
		var $values = { };
		
		// get all fieldsets
		var $fieldsets = this._dialog.find('fieldset');
		if (!$fieldsets.length) {
			return;
		}
		
		// search for fieldsets
		$fieldsets.each($.proxy(function(index, fieldset) {
			var $fieldset = $(fieldset);
			var $identifier = $fieldset.attr('id').replace(/^threadEditor/, '').toLowerCase();
			
			$values[$identifier] = this._getValues($fieldset);
		}, this));
		
		// execute callbacks
		for (var $i = 0, $length = this._callbacks.length; $i < $length; $i++) {
			$values = this._callbacks[$i].getValues(this._dialog, $values);
		}
		
		if ($.getLength($values)) {
			this._proxy.setOption('data', {
				actionName: 'saveEdit',
				className: 'wbb\\data\\thread\\ThreadAction',
				parameters: {
					data: {
						values: $values,
						threadID: this._threadID
					}
				}
			});
			this._proxy.sendRequest();
		}
	},
	
	/**
	 * Show success notification and optionally reload current page.
	 * 
	 * @param	object		data
	 */
	_saveEdit: function(data) {
		if (this._notification === null) {
			this._notification = new WCF.System.Notification(WCF.Language.get('wcf.global.success.edit'));
		}
		
		if (data.returnValues.forceReload) {
			this._notification.show(function() { window.location.reload(); }, undefined, WCF.Language.get('wcf.global.success.edit'));
		}
		else {
			// perform inline editing
			for (var $i in this._callbacks) {
				this._callbacks[$i].saved(this._environment, this._threadID, data);
			}
			
			this._dialogContainer.wcfDialog('close');
			this._notification.show();
		}
	},
	
	/**
	 * Returns a list of all input values within a fieldset.
	 * 
	 * @param	jQuery		fieldset
	 * @return	object
	 */
	_getValues: function(fieldset) {
		var $values = { };
		
		fieldset.find('input:not([type=radio])').each(function(index, input) {
			var $input = $(input);
			
			if ($input.attr('type') === 'checkbox') {
				$values[$input.attr('id')] = ($input.prop('checked') ? 1 : 0);
			}
			else {
				$values[$input.attr('id')] = $input.val();
			}
		});
		
		fieldset.find('input[type=radio]:checked').each(function(index, input) {
			var $input = $(input);
			$values[$input.attr('name')] = $input.val();
		});
		
		fieldset.find('select').each(function(index, select) {
			var $select = $(select);
			$values[$select.attr('id')] = $select.val();
		});
		
		return $values;
	},
	
	/**
	 * Closes the editor.
	 */
	_cancel: function() {
		this._dialogContainer.wcfDialog('close');
	}
};

/**
 * Default thread editor handler. 
 */
WBB.Thread.Editor.Default = Class.extend({
	init: function() {
		$('#threadEditor').data('wbbThreadEditor').registerCallback(this);
	},
	
	/**
	 * Does nothing.
	 */
	getValues: function(dialog, values) {
		return values;
	},
	
	/**
	 * Handles successful thread edits.
	 * 
	 * @param	string		environment
	 * @param	integer		threadID
	 * @param	object		data
	 */
	saved: function(environment, threadID, data) {
		if (data.returnValues['default'] === undefined) {
			return;
		}
		
		if (environment === 'board') {
			$('#thread' + threadID + ' > td.columnSubject > h3 > a').text(data.returnValues['default'].topic);
		}
		else {
			$('header.wbbThread[data-thread-id=' + threadID + '] > h1 > a').text(data.returnValues['default'].topic);
			
			// try to update first post (if visible on current page)
			$('#post' + data.returnValues['default'].firstPostID + ' .messageHeader > .messageHeadline > h1').text(data.returnValues['default'].topic);
		}
	}
});

/**
 * Label chooser for dialog thread editor.
 * 
 * @see	WCF.Label.Chooser
 */
WBB.Thread.Editor.LabelChooser = WCF.Label.Chooser.extend({
	/**
	 * @see	WCF.Label.Chooser.init()
	 */
	init: function(selectedLabelIDs, containerSelector, submitButtonSelector, showWithoutSelection) {
		this._super(selectedLabelIDs, containerSelector, submitButtonSelector, showWithoutSelection);
		
		$('#threadEditor').data('wbbThreadEditor').registerCallback(this);
	},
	
	/**
	 * Adds label ids to dialog data.
	 * 
	 * @param	jQuery		dialog
	 * @param	object		values
	 * @return	values
	 */
	getValues: function(dialog, values) {
		values.label = { };
		
		for (var $groupID in this._groups) {
			var $group = this._groups[$groupID];
			if ($group.data('labelID') !== undefined) {
				values.label[$groupID] = $group.data('labelID');
			}
		}
		
		return values;
	},
	
	/**
	 * Handles successful thread edits.
	 * 
	 * @param	string		environment
	 * @param	integer		threadID
	 * @param	object		data
	 */
	saved: function(environment, threadID, data) {
		if (data.returnValues.label === undefined) {
			return;
		}
		
		var $labels = data.returnValues.label.labels;
		var $count = $.getLength($labels);
		var $labelList = null;
		
		if (environment === 'board') {
			var $column = $('#thread' + threadID + ' > td.columnSubject');
			$labelList = $column.children('.labelList');
			if ($labelList.length) {
				if ($count) {
					// remove existing labels
					$labelList.empty();
				}
				else {
					// remove label list
					$labelList.remove();
				}
			}
			else if ($.getLength($labels)) {
				// create label list
				$labelList = $('<ul class="labelList" />').prependTo($column);
			}
			
			for (var $labelID in $labels) {
				var $label = $labels[$labelID];
				var $listItem = $('<li><a href="' + $label.link + '" class="badge label ' + $label.cssClassName + '">' + $label.label + '</a></li>').appendTo($labelList);
				$listItem.before(' ');
			}
		}
		else {
			var $header = $('header.wbbThread[data-thread-id=' + threadID + ']');
			$labelList = $header.children('.labelList');
			if ($labelList.length) {
				if ($count) {
					// remove existing labels
					$labelList.empty();
				}
				else {
					// remove label list
					$labelList.remove();
				}
			}
			else if ($.getLength($labels)) {
				// create label list
				$labelList = $('<ul class="labelList" />').appendTo($header);
			}
			
			for (var $labelID in $labels) {
				var $label = $labels[$labelID];
				var $listItem = $('<li class="badge label ' + $label.cssClassName + '">' + $label.label + '</li>').appendTo($labelList);
				$listItem.after(' ');
			}
		}
	}
});

/**
 * Inline editor for threads.
 * 
 * @param	string		elementSelector
 */
WBB.Thread.InlineEditor = WCF.InlineEditor.extend({
	/**
	 * board id
	 * @var	integer
	 */
	_boardID: 0,
	
	/**
	 * current editor environment
	 * @var	string
	 */
	_environment: 'thread',
	
	/**
	 * list of permissions
	 * @var	object
	 */
	_permissions: { },
	
	/**
	 * redirect URL
	 * @var	string
	 */
	_redirectURL: '',
	
	/**
	 * thread update handler
	 * @var	WBB.Thread.UpdateHandler
	 */
	_updateHandler: null,
	
	/**
	 * @see	WCF.InlineEditor._setOptions()
	 */
	_setOptions: function() {
		this._boardID = 0;
		this._environment = 'thread';
		
		this._options = [
			// isDone
			{ label: WCF.Language.get('wbb.thread.edit.done'), optionName: 'done' },
			{ label: WCF.Language.get('wbb.thread.edit.undone'), optionName: 'undone' },
			
			// isClosed
			{ label: WCF.Language.get('wbb.thread.edit.close'), optionName: 'close' },
			{ label: WCF.Language.get('wbb.thread.edit.open'), optionName: 'open' },
			
			// isSticky
			{ label: WCF.Language.get('wbb.thread.edit.sticky'), optionName: 'sticky' },
			{ label: WCF.Language.get('wbb.thread.edit.scrape'), optionName: 'scrape' },
			
			// isDisabled
			{ label: WCF.Language.get('wbb.thread.edit.enable'), optionName: 'enable' },
			{ label: WCF.Language.get('wbb.thread.edit.disable'), optionName: 'disable' },
			
			// move thread
			{ label: WCF.Language.get('wbb.thread.edit.move'), optionName: 'move' },
			{ label: WCF.Language.get('wbb.thread.edit.removeLink'), optionName: 'removeLink' },
			
			// isDeleted
			{ label: WCF.Language.get('wbb.thread.edit.trash'), optionName: 'trash' },
			{ label: WCF.Language.get('wbb.thread.edit.restore'), optionName: 'restore' },
			{ label: WCF.Language.get('wbb.thread.edit.delete'), optionName: 'delete' },
			
			// divider
			{ optionName: 'divider' },
			
			// overlay
			{ label: WCF.Language.get('wbb.thread.edit.advanced'), optionName: 'advanced', isQuickOption: true }
		];
	},
	
	/**
	 * Returns current update handler.
	 * 
	 * @return	WBB.Thread.UpdateHandler
	 */
	setUpdateHandler: function(updateHandler) {
		this._updateHandler = updateHandler;
	},
	
	/**
	 * @see	WCF.InlineEditor._getTriggerElement()
	 */
	_getTriggerElement: function(element) {
		return element.find('.jsThreadInlineEditor');
	},
	
	/**
	 * @see	WCF.InlineEditor._validate()
	 */
	_validate: function(elementID, optionName) {
		var $threadID = $('#' + elementID).data('threadID');
		
		// links for moved threads only support their removal
		if (this._updateHandler.getValue($threadID, 'isLink')) {
			if (optionName === 'removeLink' && this._getPermission('canMoveThread')) {
				return true;
			}
			
			return false;
		}
		
		switch (optionName) {
			// isOpen
			case 'close':
			case 'open':
				if (!this._getPermission('canCloseThread')) {
					return false;
				}
				
				if (optionName === 'open') {
					return (this._updateHandler.getValue($threadID, 'isClosed'));
				}
				else {
					return !(this._updateHandler.getValue($threadID, 'isClosed'));
				}
			break;
			
			//isDeleted
			case 'delete':
				if (!this._getPermission('canDeleteThreadCompletely')) {
					return false;
				}
				
				return (this._updateHandler.getValue($threadID, 'isDeleted'));
			break;
			
			case 'done':
			case 'undone':
				if (!this._updateHandler.getValue($threadID, 'canMarkAsDone')) {
					return false;
				}
				
				if (optionName === 'undone') {
					return (this._updateHandler.getValue($threadID, 'isDone'));
				}
				else {
					return !(this._updateHandler.getValue($threadID, 'isDone'));
				}
			break;
			
			case 'restore':
				if (!this._getPermission('canRestoreThread')) {
					return false;
				}
				
				return (this._updateHandler.getValue($threadID, 'isDeleted'));
			break;
			
			case 'trash':
				if (!this._getPermission('canDeleteThread')) {
					return false;
				}
				
				return !(this._updateHandler.getValue($threadID, 'isDeleted'));
			break;
			
			// isSticky
			case 'sticky':
			case 'scrape':
				if (!this._getPermission('canPinThread') || (this._updateHandler.getValue($threadID, 'isAnnouncement'))) {
					return false;
				}
				
				if (optionName === 'scrape') {
					return (this._updateHandler.getValue($threadID, 'isSticky'));
				}
				else {
					return !(this._updateHandler.getValue($threadID, 'isSticky'));
				}
			break;
			
			// isDisabled
			case 'enable':
				if (!this._getPermission('canEnableThread')) {
					return false;
				}
				
				if (this._updateHandler.getValue($threadID, 'isDeleted')) {
					return false;
				}
				
				return (this._updateHandler.getValue($threadID, 'isDisabled'));
			break;
			case 'disable':
				if (!this._getPermission('canEnableThread')) {
					return false;
				}
				
				if (this._updateHandler.getValue($threadID, 'isDeleted')) {
					return false;
				}
				
				return !(this._updateHandler.getValue($threadID, 'isDisabled'));
			break;
			
			// move
			case 'move':
				return this._getPermission('canMoveThread');
			break;
			
			case 'advanced':
				return true;
			break;
		}
		
		return false;
	},
	
	/**
	 * @see	WCF.InlineEditor._execute()
	 */
	_execute: function(elementID, optionName) {
		// abort if option is invalid or not accessible
		if (!this._validate(elementID, optionName)) {
			return false;
		}
		
		switch (optionName) {
			case 'close':
			case 'open':
				var $isClosed = (optionName === 'open') ? 0 : 1;
				this._updateThread(elementID, optionName, { isClosed: $isClosed });
			break;
			
			case 'sticky':
			case 'scrape':
				var $isSticky = (optionName === 'scrape') ? 0 : 1;
				this._updateThread(elementID, optionName, { isSticky: $isSticky });
			break;
			
			case 'done':
			case 'undone':
				var $isDone = (optionName === 'done') ? 1 : 0;
				this._updateThread(elementID, optionName, { isDone: $isDone });
			break;
			
			case 'enable':
			case 'disable':
				this._updateThread(elementID, optionName, { isDisabled: (optionName === 'enable' ? 0 : 1) });
			break;
			
			case 'move':
				var $threadID = this._elements[elementID].data('threadID');
				
				WBB.Thread.MoveHandler.prepare([ $threadID ], $.proxy(function(data) {
					this._updateHandler.update($threadID, data.returnValues.threadData[$threadID]);
				}, this));
			break;
			
			case 'removeLink':
				this._updateThread(elementID, optionName, { removeLink: 1 });
			break;
			
			case 'delete':
				var self = this;
				WCF.System.Confirmation.show(WCF.Language.get('wbb.thread.confirmDelete'), function(action) {
					if (action === 'confirm') {
						self._updateThread(elementID, optionName, { deleted: 1 });
					}
				});
			break;
			
			case 'restore':
				this._updateThread(elementID, optionName, { isDeleted: 0 });
			break;
			
			case 'trash':
				var self = this;
				WCF.System.Confirmation.show(WCF.Language.get('wbb.thread.confirmTrash'), function(action) {
					if (action === 'confirm') {
						self._updateThread(elementID, optionName, { isDeleted: 1, reason: $('#wcfSystemConfirmationContent').find('textarea').val() });
					}
				}, { }, $('<fieldset><dl><dt>' + WCF.Language.get('wbb.thread.confirmTrash.reason') + '</dt><dd><textarea cols="40" rows="4" /></dd></dl></fieldset>'));
			break;
			
			case 'advanced':
				WBB.Thread.Editor.beginEdit(this._environment, this._elements[elementID].data('threadID'));
			break;
			
			default:
				return false;
			break;
		}
		
		return true;
	},
	
	/**
	 * Updates thread properties.
	 * 
	 * @param	string		elementID
	 * @param	string		optionName
	 * @param	object		data
	 */
	_updateThread: function(elementID, optionName, data) {
		if (optionName === 'delete' || optionName === 'removeLink') {
			var self = this;
			var $threadID = this._elements[elementID].data('threadID');
			
			new WCF.Action.Proxy({
				autoSend: true,
				data: {
					actionName: optionName,
					className: 'wbb\\data\\thread\\ThreadAction',
					objectIDs: [ $threadID ]
				},
				success: function(data) {
					self._updateHandler.update($threadID, data.returnValues.threadData[$threadID]);
				}
			});
		}
		else {
			this._updateData.push({
				data: data,
				elementID: elementID,
				optionName: optionName
			});
			
			this._proxy.setOption('data', {
				actionName: optionName,
				className: 'wbb\\data\\thread\\ThreadAction',
				objectIDs: [ this._elements[elementID].data('threadID') ],
				parameters: {
					data: data
				}
			});
			this._proxy.sendRequest();
		}
	},
	
	/**
	 * @see	WCF.InlineEditor._updateState()
	 */
	_updateState: function() {
		this._notification.show();
		
		for (var $i = 0, $length = this._updateData.length; $i < $length; $i++) {
			var $data = this._updateData[$i];
			
			if (($data.data.isSticky !== undefined) && this._redirectURL) {
				window.location = this._redirectURL;
				return;
			}
			
			var $threadID = $('#' + $data.elementID).data('threadID');
			this._updateHandler.update($threadID, $data.data);
		}
	},
	
	/**
	 * Returns a specific permission.
	 * 
	 * @param	string		permission
	 * @return	integer
	 */
	_getPermission: function(permission) {
		if (this._permissions[permission]) {
			return this._permissions[permission];
		}
		
		return 0;
	},
	
	/**
	 * Sets current environment.
	 * 
	 * @param	string		environment
	 * @param	integer		boardID
	 * @param	string		redirectURL
	 */
	setEnvironment: function(environment, boardID, redirectURL) {
		if (environment !== 'board') {
			environment = 'thread';
		}
		
		this._boardID = (boardID) ? boardID : 0;
		this._environment = environment;
		this._redirectURL = redirectURL;
	},
	
	/**
	 * Sets a permission.
	 * 
	 * @param	string		permission
	 * @param	integer		value
	 */
	setPermission: function(permission, value) {
		this._permissions[permission] = value;
	},
	
	/**
	 * Sets permissions.
	 * 
	 * @param	object		permissions
	 */
	setPermissions: function(permissions) {
		for (var $permission in permissions) {
			this.setPermission($permission, permissions[$permission]);
		}
	}
});

/**
 * Provides an AJAX-based quick reply for threads.
 * 
 * @see	WCF.Message.QuickReply
 */
WBB.Thread.QuickReply = WCF.Message.QuickReply.extend({
	/**
	 * @see	WCF.Message.QuickReply.init()
	 */
	init: function(quoteManager) {
		this._super(true, quoteManager);
		
		this._successMessageNonVisible = 'wbb.post.moderation.redirect';
	},
	
	/**
	 * @see	WCF.Message.QuickReply._getClassName()
	 */
	_getClassName: function() {
		return 'wbb\\data\\post\\PostAction';
	},
	
	/**
	 * @see	WCF.Message.QuickReply._getObjectID()
	 */
	_getObjectID: function() {
		return this._container.data('threadID');
	}
});

/**
 * Provides extended actions for thread clipboard actions.
 */
WBB.Thread.Clipboard = Class.extend({
	/**
	 * board id
	 * @var	integer
	 */
	_boardID: 0,
	
	/**
	 * current environment
	 * @var	string
	 */
	_environment: 'board',
	
	/**
	 * thread update handler
	 * @var	WBB.Thread.UpdateHandler
	 */
	_updateHandler: null,
	
	/**
	 * Initializes a new WBB.Thread.Clipboard object.
	 * 
	 * @param	WBB.Thread.UpdateHandler	updateHandler
	 * @param	string				environment
	 * @param	integer				boardID
	 */
	init: function(updateHandler, environment, boardID) {
		this._updateHandler = updateHandler;
		this._environment = environment;
		this._boardID = (boardID) ? boardID : 0;
		
		// bind listener
		$('.jsClipboardEditor').each($.proxy(function(index, container) {
			var $container = $(container);
			var $types = eval($container.data('types'));
			if (WCF.inArray('com.woltlab.wbb.thread', $types)) {
				$container.on('clipboardAction', $.proxy(this._execute, this));
				$container.on('clipboardActionResponse', $.proxy(this._evaluateResponse, this));
				return false;
			}
		}, this));
	},
	
	/**
	 * Handles clipboard actions.
	 * 
	 * @param	object		event
	 * @param	string		type
	 * @param	string		actionName
	 * @param	object		parameters
	 */
	_execute: function(event, type, actionName, parameters) {
		if (type !== 'com.woltlab.wbb.thread') {
			return;
		}
		
		switch (actionName) {
			case 'com.woltlab.wbb.thread.merge':
				var $mergeHandler = new WBB.Thread.MergeHandler(parameters.objectIDs);
				$mergeHandler.load();
			break;
			
			case 'com.woltlab.wbb.thread.move':
				WBB.Thread.MoveHandler.prepare(parameters.objectIDs, $.proxy(this._move, this));
			break;
		}
	},
	
	/**
	 * Handles 'move thread' action.
	 * 
	 * @param	object		data
	 */
	_move: function(data) {
		this._evaluateResponse(null, data, 'com.woltlab.wbb.thread', 'com.woltlab.wbb.thread.move', null);
	},
	
	/**
	 * Evaluates AJAX responses.
	 * 
	 * @param	object		event
	 * @param	object		data
	 * @param	string		type
	 * @param	string		actionName
	 * @param	object		parameters
	 */
	_evaluateResponse: function(event, data, type, actionName, parameters) {
		// ignore unrelated events
		if (type !== 'com.woltlab.wbb.thread') {
			return;
		}
		
		if (!data.returnValues.threadData || !$.getLength(data.returnValues.threadData)) {
			return;
		}
		
		// loop through threads
		for (var $threadID in data.returnValues.threadData) {
			this._updateHandler.update($threadID, data.returnValues.threadData[$threadID]);
		}
	}
});

/**
 * Provides a generic update handler for threads.
 */
WBB.Thread.UpdateHandler = Class.extend({
	/**
	 * mark as done handler
	 * @var	WBB.Thread.MarkAsDone
	 */
	_markAsDoneHandler: null,
	
	/**
	 * list of threads
	 * @var	object
	 */
	_threads: { },
	
	/**
	 * Initializes the thread update handler.
	 */
	init: function() {
		var self = this;
		$('.wbbThread').each(function(index, thread) {
			var $thread = $(thread);
			
			self._threads[$thread.data('threadID')] = $thread;
		});
	},
	
	/**
	 * Sets mark as done handler.
	 * 
	 * @param	WBB.Thread.MarkAsDone	markAsDone
	 */
	setMarkAsDoneHandler: function(markAsDoneHandler) {
		this._markAsDoneHandler = markAsDoneHandler;
	},
	
	/**
	 * Updates a set of properties for given thread id.
	 * 
	 * @param	integer		threadID
	 * @param	object		data
	 */
	update: function(threadID, data) {
		if (!this._threads[threadID]) {
			console.debug("[WBB.Thread.UpdateHandler] Unknown thread id " + threadID);
			return;
		}
		
		for (var $property in data) {
			this._updateProperty(threadID, $property, data[$property]);
		}
	},
	
	/**
	 * Wrapper for property updating.
	 * 
	 * @param	integer		threadID
	 * @param	string		property
	 * @param	mixed		value
	 */
	_updateProperty: function(threadID, property, value) {
		switch (property) {
			case 'deleted':
				this._delete(threadID, value);
			break;
			
			case 'deleteNote':
				this._deleteNote(threadID, value);
			break;
			
			case 'isClosed':
				if (value) {
					this._close(threadID);
				}
				else {
					this._open(threadID);
				}
			break;
			
			case 'isDeleted':
				if (value) {
					this._trash(threadID);
				}
				else {
					this._restore(threadID);
				}
			break;
			
			case 'isDisabled':
				if (value) {
					this._disable(threadID);
				}
				else {
					this._enable(threadID);
				}
			break;
			
			case 'isDone':
				if (value) {
					this._done(threadID);
				}
				else {
					this._undone(threadID);
				}
			break;
			
			case 'isSticky':
				if (value) {
					this._sticky(threadID);
				}
				else {
					this._scrape(threadID);
				}
			break;
			
			case 'moved':
				this._moved(threadID);
			break;
			
			case 'showMoveNotice':
				this._showMoveNotice(threadID);
			break;
			
			case 'title':
				this._updateTitle(threadID, value);
			break;
			
			default:
				this._handleCustomProperty(threadID, property, value);
			break;
		}
	},
	
	/**
	 * Handles custom properties not known to _updateProperty(), override if necessary.
	 * 
	 * @param	integer		threadID
	 * @param	string		property
	 * @param	mixed		value
	 */
	_handleCustomProperty: function(threadID, property, value) {
		this._threads[threadID].trigger('threadUpdateHandlerProperty', [ threadID, property, value ]);
	},
	
	/**
	 * Closes a thread.
	 * 
	 * @param	integer		threadID
	 */
	_close: function(threadID) {
		this._threads[threadID].data('isClosed', 1);
	},
	
	/**
	 * Deletes a thread.
	 * 
	 * @param	integer		threadID
	 * @param	string		boardLink
	 */
	_delete: function(threadID, boardLink) { },
	
	/**
	 * Displays the delete notice.
	 * 
	 * @param	integer		threadID
	 * @param	string		message
	 */
	_deleteNote: function(threadID, message) { },
	
	/**
	 * Disables a thread.
	 * 
	 * @param	integer		threadID
	 */
	_disable: function(threadID) {
		this._threads[threadID].data('isDisabled', 1);
	},
	
	/**
	 * Marks a thread as done.
	 * 
	 * @param	integer		threadID
	 */
	_done: function(threadID) {
		this._threads[threadID].data('isDone', 1);
	},
	
	/**
	 * Enables a thread.
	 * 
	 * @param	integer		threadID
	 */
	_enable: function(threadID) {
		this._threads[threadID].data('isDisabled', 0);
	},
	
	
	/**
	 * Shows notice for a moved thread.
	 * 
	 * @param	integer		threadID
	 */
	_moved: function(threadID) { },
	
	/**
	 * Opens a thread.
	 * 
	 * @param	integer		threadID
	 */
	_open: function(threadID) {
		this._threads[threadID].data('isClosed', 0);
	},
	
	/**
	 * Restores a thread.
	 * 
	 * @param	integer		threadID
	 */
	_restore: function(threadID) {
		this._threads[threadID].data('isDeleted', 0);
	},
	
	/**
	 * Converts a sticky thread into a normal one.
	 * 
	 * @param	integer		threadID
	 */
	_scrape: function(threadID) {
		this._threads[threadID].data('isSticky', 0);
	},
	
	/**
	 * Sets a thread as sticky.
	 * 
	 * @param	integer		threadID
	 */
	_sticky: function(threadID) {
		this._threads[threadID].data('isSticky', 1);
	},
	
	/**
	 * Trashes a thread.
	 * 
	 * @param	integer		threadID
	 */
	_trash: function(threadID) {
		this._threads[threadID].data('isDeleted', 1);
	},
	
	/**
	 * Marks a thread as undone.
	 * 
	 * @param	integer		threadID
	 */
	_undone: function(threadID) {
		this._threads[threadID].data('isDone', 0);
	},
	
	/**
	 * Updates title of a thread.
	 * 
	 * @param	integer		threadID
	 * @param	string		title
	 */
	_updateTitle: function(threadID, title) { },
	
	/**
	 * Returns the close icon.
	 * 
	 * @return	jQuery
	 */
	_getCloseIcon: function() {
		return $('<li><span title="' + WCF.Language.get('wbb.thread.closed') + '" class="icon icon16 icon-lock jsTooltip jsIconClosed" /></li>');
	},
	
	/**
	 * Returns the done icon.
	 * 
	 * @return	jQuery
	 */
	_getDoneIcon: function() {
		return $('<li><span title="' + WCF.Language.get('wbb.thread.done') + '" class="icon icon16 icon-check jsTooltip jsIconDone wbbMarkAsDone" /></li>');
	},
	
	/**
	 * Returns the moved icon.
	 * 
	 * @return	jQuery
	 */
	_getMoveIcon: function() {
		return $('<li><span title="' + WCF.Language.get('wbb.thread.moved') + '" class="icon icon16 icon-arrow-right jsTooltip jsIconMoved" /></li>');
	},
	
	/**
	 * Returns the undone icon.
	 * 
	 * @return	jQuery
	 */
	_getUndoneIcon: function() {
		return $('<li><span title="' + WCF.Language.get('wbb.thread.undone') + '" class="icon icon16 icon-check-empty jsTooltip jsIconUndone wbbMarkAsDone" /></li>');
	},
	
	/**
	 * Returns generic property values for a thread.
	 * 
	 * @param	integer		threadID
	 * @param	string		property
	 * @return	boolean
	 */
	getValue: function(threadID, property) {
		if (!this._threads[threadID]) {
			console.debug("[WBB.Thread.UpdateHandler] Unknown thread id " + threadID);
			return;
		}
		
		switch (property) {
			/* properties */
			case 'isAnnouncement':
				return this._threads[threadID].data('isAnnouncement');
			break;
			
			case 'isClosed':
				return this._threads[threadID].data('isClosed');
			break;
			
			case 'isDeleted':
				return this._threads[threadID].data('isDeleted');
			break;
			
			case 'isDisabled':
				return this._threads[threadID].data('isDisabled');
			break;
			
			case 'isDone':
				return this._threads[threadID].data('isDone');
			break;
			
			case 'isSticky':
				return this._threads[threadID].data('isSticky');
			break;
			
			case 'isLink':
				return this._threads[threadID].data('isLink');
			break;
			
			/* permissions */
			case 'canMarkAsDone':
				return this._threads[threadID].data('canMarkAsDone');
			break;
		}
	}
});

/**
 * Thread update handler for thread list on board page.
 * 
 * @see	WBB.Thread.UpdateHandler
 */
WBB.Thread.UpdateHandler.Board = WBB.Thread.UpdateHandler.extend({
	/**
	 * @see	WBB.Thread.UpdateHandler._close()
	 */
	_close: function(threadID) {
		this._super(threadID);
		
		this._getCloseIcon().appendTo(this._threads[threadID].find('.statusIcons'));
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._delete()
	 */
	_delete: function(threadID, boardLink) {
		this._threads[threadID].remove();
		delete this._threads[threadID];
		
		WCF.Clipboard.reload();
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._deleteNote()
	 */
	_deleteNote: function(threadID, message) {
		$('<small class="wbbThreadDeleteNote">' + message + '</small>').appendTo(this._threads[threadID].find('.columnSubject'));
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._disable()
	 */
	_disable: function(threadID) {
		this._super(threadID);
		
		this._threads[threadID].addClass('messageDisabled');
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._done()
	 */
	_done: function(threadID) {
		this._super(threadID);
		
		var $undoneIcon = this._threads[threadID].find('.jsIconUndone').parent();
		var $icon = this._getDoneIcon();
		$icon.insertAfter($undoneIcon);
		$undoneIcon.remove();
		
		if (this._markAsDoneHandler !== null) {
			this._markAsDoneHandler.watch(threadID);
		}
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._enable()
	 */
	_enable: function(threadID) {
		this._super(threadID);
		
		this._threads[threadID].removeClass('messageDisabled');
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._moved()
	 */
	_moved: function(threadID) {
		window.location.reload();
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._showMoveNotice()
	 */
	_showMoveNotice: function(threadID) {
		this._getMoveIcon().appendTo(this._threads[threadID].find('.statusIcons'));
		
		WCF.Clipboard.reload();
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._open()
	 */
	_open: function(threadID) {
		this._super(threadID);
		
		this._threads[threadID].find('.jsIconClosed').parent().remove();
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._scrape()
	 */
	_scrape: function(threadID) {
		this._super(threadID);
		
		//this._threads[threadID].find('hgroup > h1 > span.jsSticky').remove();
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._sticky()
	 */
	_sticky: function(threadID) {
		this._super(threadID);
		
		//var $badge = $('<span class="badge label jsSticky">' + WCF.Language.get('wbb.thread.sticky') + '</span>').prependTo(this._threads[threadID].find('hgroup > h1'));
		//$badge.after(' ');
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._restore()
	 */
	_restore: function(threadID) {
		this._super(threadID);
		
		this._threads[threadID].removeClass('messageDeleted');
		this._threads[threadID].find('.columnSubject > .wbbThreadDeleteNote').remove();
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._trash()
	 */
	_trash: function(threadID) {
		this._super(threadID);
		
		this._threads[threadID].addClass('messageDeleted');
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._undone()
	 */
	_undone: function(threadID) {
		this._super(threadID);
		
		var $doneIcon = this._threads[threadID].find('.jsIconDone').parent();
		var $icon = this._getUndoneIcon();
		$icon.insertAfter($doneIcon);
		$doneIcon.remove();
		
		if (this._markAsDoneHandler !== null) {
			this._markAsDoneHandler.watch(threadID);
		}
	}
});

/**
 * Thread update handler for thread page.
 * 
 * @param	integer		boardID
 * @see		WBB.Thread.UpdateHandler
 */
WBB.Thread.UpdateHandler.Thread = WBB.Thread.UpdateHandler.extend({
	/**
	 * board id
	 * @var	integer
	 */
	_boardID: 0,
	
	/**
	 * post handler for the posts of the handled thread
	 * @var	WBB.Post.Handler
	 */
	_postHandler: null,
	
	/**
	 * Initializes the thread update handler for thread page.
	 * 
	 * @param	integer		boardID
	 */
	init: function(boardID) {
		this._boardID = boardID;
		
		this._super();
	},
	
	/**
	 * Sets the post handler for the posts of the handled thread.
	 * 
	 * @param	WBB.Post.Handler	postHandler
	 */
	setPostHandler: function(postHandler) {
		this._postHandler = postHandler;
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler.update()
	 */
	update: function(threadID, data) {
		if (this._threads[threadID]) {
			if (data.isDeleted !== undefined && !data.isDeleted && data.ignoreDeletedPosts !== undefined) {
				this._restore(threadID, true);
				
				delete data.isDeleted;
				delete data.ignoreDeletedPosts;
			}
			if (data.isDisabled !== undefined && !data.isDisabled && data.ignoreDisabledPosts !== undefined) {
				this._enable(threadID, true);
				
				delete data.isDisabled;
				delete data.ignoreDisabledPosts;
			}
		}
		
		this._super(threadID, data);
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._close()
	 */
	_close: function(threadID) {
		this._super(threadID);
		
		var $icon = this._getCloseIcon();
		$icon.appendTo(this._threads[threadID].find('h1 + ul.statusIcons'));
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._delete()
	 */
	_delete: function(threadID, boardLink) {
		new WCF.PeriodicalExecuter(function(pe) {
			pe.stop();
			
			window.location = boardLink;
		}, 1000);
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._disable()
	 */
	_disable: function(threadID) {
		this._super(threadID);
		
		this._postHandler.updateAllPosts('isDisabled', 1);
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._done()
	 */
	_done: function(threadID) {
		this._super(threadID);
		
		var $undoneIcon = this._threads[threadID].find('h1 + ul.statusIcons span.jsIconUndone').parent();
		var $icon = this._getDoneIcon();
		$icon.insertAfter($undoneIcon);
		$undoneIcon.remove();
		
		if (this._markAsDoneHandler !== null) {
			this._markAsDoneHandler.watch(threadID);
		}
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._enable()
	 */
	_enable: function(threadID, ignorePosts) {
		this._super(threadID);
		
		if (!ignorePosts) {
			this._postHandler.updateAllPosts('isDisabled', 0);
		}
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._moved()
	 */
	_moved: function(threadID) {
		window.location.reload();
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._showMoveNotice()
	 */
	_showMoveNotice: function(threadID) {
		window.location.reload();
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._open()
	 */
	_open: function(threadID) {
		this._super(threadID);
		
		this._threads[threadID].find('h1 + ul.statusIcons span.jsIconClosed').parent().remove();
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._scrape()
	 */
	_scrape: function(threadID) {
		this._super(threadID);
		
		this._threads[threadID].find('h1 > span.jsSticky').remove();
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._sticky()
	 */
	_sticky: function(threadID) {
		this._super(threadID);
		
		var $badge = $('<span class="badge label jsSticky">' + WCF.Language.get('wbb.thread.sticky') + '</span>').prependTo(this._threads[threadID].find('h1'));
		$badge.after(' ');
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._restore()
	 */
	_restore: function(threadID, ignorePosts) {
		this._super(threadID);
		
		if (!ignorePosts) {
			this._postHandler.updateAllPosts('isDeleted', 0);
		}
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._trash()
	 */
	_trash: function(threadID) {
		this._super(threadID);
		
		this._postHandler.updateAllPosts('isDeleted', 1);
	},
	
	/**
	 * @see	WBB.Thread.UpdateHandler._undone()
	 */
	_undone: function(threadID) {
		this._super(threadID);
		
		var $doneIcon = this._threads[threadID].find('h1 + ul.statusIcons span.jsIconDone').parent();
		var $icon = this._getUndoneIcon();
		$icon.insertAfter($doneIcon);
		$doneIcon.remove();
		
		if (this._markAsDoneHandler !== null) {
			this._markAsDoneHandler.watch(threadID);
		}
	}
});

/**
 * Merges multiple threads.
 * 
 * @param	array<integer>		objectIDs
 */
WBB.Thread.MergeHandler = Class.extend({
	/**
	 * action class name
	 * @var	string
	 */
	_className: '',
	
	/**
	 * dialog overlay
	 * @var	jQuery
	 */
	_dialog: null,
	
	/**
	 * dialog title
	 * @var	string
	 */
	_dialogTitle: '',
	
	/**
	 * list of object ids
	 * @var	array<integer>
	 */
	_objectIDs: [ ],
	
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * success message after merging
	 * @var	string
	 */
	_successMessage: '',
	
	/**
	 * Initializes the WBB.Thread.MergeHandler object.
	 * 
	 * @param	array<integer>		objectIDs
	 */
	init: function(objectIDs) {
		this._className = 'wbb\\data\\thread\\ThreadAction';
		this._dialogTitle = WCF.Language.get('wbb.thread.edit.merge');
		this._objectIDs = objectIDs;
		this._successMessage = WCF.Language.get('wbb.thread.edit.merge.success');
		
		this._proxy = new WCF.Action.Proxy({
			success: $.proxy(this._success, this)
		});
	},
	
	/**
	 * Loads the merge dialog.
	 */
	load: function() {
		this._proxy.setOption('data', {
			actionName: 'prepareMerge',
			className: this._className,
			objectIDs: this._objectIDs
		});
		this._proxy.sendRequest();
	},
	
	/**
	 * Handles successful AJAX requests.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		if (data.returnValues.redirectURL) {
			var $notification = new WCF.System.Notification(this._successMessage);
			$notification.show(function() { window.location = data.returnValues.redirectURL; });
			
			return;
		}
		
		if (this._dialog === null) {
			this._dialog = $('<div />').hide().appendTo(document.body);
			this._dialog.html(data.returnValues.template);
			this._dialog.wcfDialog({
				title: this._dialogTitle
			});
		}
		else {
			this._dialog.html(data.returnValues.template);
			this._dialog.wcfDialog('show');
		}
		
		var $button = this._dialog.find('.formSubmit > button[data-type=submit]').disable().click($.proxy(this._submit, this));
		this._dialog.find('input[type=radio]').change(function() { $button.enable(); });
	},
	
	/**
	 * Submits the merge dialog.
	 */
	_submit: function() {
		this._dialog.find('.formSubmit > button[data-type=submit]').disable();
		
		this._proxy.setOption('data', {
			actionName: 'merge',
			className: this._className,
			objectIDs: this._objectIDs,
			parameters: this._getParameters()
		});
		this._proxy.sendRequest();
	},
	
	/**
	 * Returns parameters for merging.
	 * 
	 * @return	object
	 */
	_getParameters: function() {
		return {
			threadID: this._dialog.find('input[type=radio]:checked').val()
		};
	}
});

/**
 * Generic implementation to move threads.
 */
WBB.Thread.MoveHandler = {
	/**
	 * callback object
	 * @var	object
	 */
	_callback: null,
	
	/**
	 * dialog overlay
	 * @var	jQuery
	 */
	_dialog: null,
	
	/**
	 * initialization state
	 * @var	boolean
	 */
	_didInit: false,
	
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * Initializes WBB.Thread.MoveHandler on first use.
	 */
	_init: function() {
		this._dialog = $('<div />').hide().appendTo(document.body);
		this._proxy = new WCF.Action.Proxy();
		
		this._didInit = true;
	},
	
	/**
	 * Prepares 'move thread' action.
	 * 
	 * @param	array<integer>	threadIDs
	 * @param	object		callback
	 */
	prepare: function(threadIDs, callback) {
		if (!$.isFunction(callback)) {
			console.debug("[WBB.Thread.MoveHandler] Given callback is invalid.");
			return;
		}
		this._callback = callback;
		
		// initialize handler on first use
		if (!this._didInit) {
			this._init();
		}
		
		this._proxy.setOption('data', {
			actionName: 'prepareMove',
			className: 'wbb\\data\\thread\\ThreadAction',
			objectIDs: threadIDs
		});
		this._proxy.setOption('success', $.proxy(this._success, this));
		this._proxy.sendRequest();
	},
	
	/**
	 * Handles successful AJAX calls.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		this._dialog.data('objectIDs', data.objectIDs).html(data.returnValues.template);
		this._dialog.wcfDialog({
			title: WCF.Language.get('wbb.thread.edit.moveThreads')
		});
		
		// listen for submit event
		this._dialog.find('.formSubmit > input[type=submit]').click($.proxy(this._move, this));
	},
	
	/**
	 * Handles 'move thread' action.
	 * 
	 * @param	object		event
	 */
	_move: function(event) {
		var $showMoveNotice = this._dialog.find('#showMoveNotice').prop('checked');
		var $boardID = parseInt(this._dialog.find('#boardID').prop('value'));
		
		if ($boardID) {
			this._proxy.setOption('data', {
				actionName: 'move',
				className: 'wbb\\data\\thread\\ThreadAction',
				objectIDs: this._dialog.data('objectIDs'),
				parameters: {
					boardID: $boardID,
					showMoveNotice: $showMoveNotice
				}
			});
			this._proxy.setOption('success', $.proxy(function(data) {
				this._callback(data);
			}, this));
			this._proxy.sendRequest();
		}
		
		// close dialog
		this._dialog.wcfDialog('close');
	},
};

/**
 * Thread type chooser.
 * 
 * @param	integer		type
 */
WBB.Thread.TypeChooser = Class.extend({
	/**
	 * board ids container
	 * @var	jQuery
	 */
	_boardIDsContainer: null,
	
	/**
	 * board type (0 = default, 1 = sticky, 2 = announcement)
	 * @var	integer
	 */
	_type: 0,
	
	/**
	 * Initializes the thread type chooser.
	 * 
	 * @param	integer		type
	 */
	init: function(type) {
		this._type = parseInt(type);
		this._boardIDsContainer = $('#boardIDsContainer');
		
		var $inputElements = $('input[name=type]').click($.proxy(this._click, this));
		
		// set type on init
		var self = this;
		$inputElements.each(function(index, element) {
			var $element = $(element);
			
			if ($element.prop('value') == self._type) {
				$element.prop('checked', 'checked');
				self._update();
				
				return false;
			}
		});
	},
	
	/**
	 * Handles click event.
	 * 
	 * @param	object		event
	 */
	_click: function(event) {
		this._type = parseInt($(event.currentTarget).prop('value'));
		
		this._update();
	},
	
	/**
	 * Updates board ids container state.
	 */
	_update: function() {
		switch (this._type) {
			/* default */
			case 0:
			/* sticky */
			case 1:
				this._boardIDsContainer.hide();
			break;
			
			/* announcement */
			case 2:
				this._boardIDsContainer.show();
			break;
		}
	}
});

/**
 * Shows a notice on last thread page if new posts are available.
 * 
 * @param	integer		threadID
 * @param	integer		lastPostTime
 * @param	integer		pageNo
 */
WBB.Thread.LastPageHandler = Class.extend({
	/**
	 * current action
	 * @var	string
	 */
	_action: '',
	
	/**
	 * last post time
	 * @var	integer
	 */
	_lastPostTime: 0,
	
	/**
	 * current page no
	 * @var	integer
	 */
	_pageNo: 0,
	
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * thread id
	 * @var	integer
	 */
	_threadID: 0,
	
	/**
	 * Initializes the last page handler.
	 * 
	 * @param	integer		threadID
	 * @param	integer		lastPostTime
	 * @param	integer		pageNo
	 */
	init: function(threadID, lastPostTime, pageNo) {
		this._lastPostTime = lastPostTime;
		this._pageNo = pageNo;
		this._threadID = threadID;
		
		this._action = '';
		this._proxy = new WCF.Action.Proxy({
			showLoadingOverlay: false,
			success: $.proxy(this._success, this),
			suppressErrors: true
		});
		
		new WCF.Action.Scroll(300, $.proxy(this._scroll, this), window, '.wbbThreadPostList');
	},
	
	/**
	 * Handles scroll event.
	 * 
	 * @param	WCF.Action.Scroll	scrollAPI
	 */
	_scroll: function(scrollAPI) {
		// terminate event listener
		scrollAPI.stop();
		
		this._action = 'getNewPosts';
		
		// send request
		this._proxy.setOption('data', {
			actionName: 'getNewPosts',
			className: 'wbb\\data\\thread\\ThreadAction',
			objectIDs: [ this._threadID ],
			parameters: {
				countPosts: true,
				lastPostTime: this._lastPostTime
			}
		});
		this._proxy.sendRequest();
	},
	
	/**
	 * Handles successful AJAX requests.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function (data, textStatus, jqXHR) {
		if (this._action === 'getNewPosts') {
			if (data.returnValues.newPostsCount > 0) {
				var $message = WCF.Language.get('wbb.thread.newPosts', { newPostsCount: data.returnValues.newPostsCount });
				var $newPostsLink = $('<p id="wbbNewPostsLink" class="info pointer">' + $message + '</p>').insertAfter($('.wbbThreadPostList'));
				$newPostsLink.click($.proxy(this._click, this));
			}
		}
		else {
			if (data.returnValues.redirectURL) {
				// redirect to next page
				window.location = data.returnValues.redirectURL;
			}
			else {
				// remove link
				$('#wbbNewPostsLink').remove();
				
				// insert posts
				$(data.returnValues.template).insertBefore($('#messageQuickReply'));
				
				// show notice if more posts are available on next page
				if (data.returnValues.nextPageNotice) {
					$('<p class="info">' + data.returnValues.nextPageNotice + '</p>').insertAfter($('.wbbThreadPostList'));
				}
				
				// update last post time for quick reply
				$('#messageQuickReply').data('lastPostTime', data.returnValues.lastPostTime);
			}
		}
	},
	
	/**
	 * Loads new posts.
	 */
	_click: function() {
		this._action = 'loadPosts';
		
		this._proxy.setOption('data', {
			actionName: 'getNewPosts',
			className: 'wbb\\data\\thread\\ThreadAction',
			objectIDs: [ this._threadID ],
			parameters: {
				countPosts: false,
				lastPostTime: this._lastPostTime,
				pageNo: this._pageNo
			}
		});
		this._proxy.sendRequest();
	}
});

/**
 * Provides a toggle handler for mark as done.
 * 
 * @param	WBB.Thread.UpdateHandler.Board		updateHandler
 */
WBB.Thread.MarkAsDone = Class.extend({
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * thread update handler
	 * @var	WBB.Thread.UpdateHandler.Board
	 */
	_updateHandler: null,
	
	/**
	 * list of threads
	 * @var	object
	 */
	_threads: { },
	
	/**
	 * Initializes the mark as done handler.
	 * 
	 * @param	WBB.Thread.UpdateHandler.Board		updateHandler
	 */
	init: function(updateHandler) {
		this._updateHandler = updateHandler;
		this._threads = { };
		
		this._proxy = new WCF.Action.Proxy({
			success: $.proxy(this._success, this)
		});
		
		var self = this;
		$('.wbbThread').each(function(index, thread) {
			var $thread = $(thread);
			var $threadID = $thread.data('threadID');
			
			self._threads[$threadID] = $thread;
			
			if ($thread.data('canMarkAsDone')) {
				self.watch($threadID);
			}
		});
		
		// register with update handler
		this._updateHandler.setMarkAsDoneHandler(this);
	},
	
	/**
	 * Watches for double clicks on "mark as done" icon.
	 * 
	 * @param	integer		threadID
	 */
	watch: function(threadID) {
		if (this._threads[threadID] && this._threads[threadID].data('canMarkAsDone')) {
			this._threads[threadID].find('.jsIconDone, .jsIconUndone').data('threadID', threadID).dblclick($.proxy(this._dblclick, this));
		}
	},
	
	/**
	 * Handles double click on the "mark as done" icon.
	 * 
	 * @param	object		event
	 */
	_dblclick: function(event) {
		var $img = $(event.currentTarget);
		var $isDone = ($img.hasClass('jsIconDone') ? true : false);
		
		this._proxy.setOption('data', {
			actionName: ($isDone ? 'un' : '') + 'done',
			className: 'wbb\\data\\thread\\ThreadAction',
			objectIDs: [ $img.data('threadID') ]
		});
		this._proxy.sendRequest();
	},
	
	/**
	 * Handles successful AJAX requests.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		for (var $threadID in data.returnValues.threadData) {
			this._updateHandler.update($threadID, data.returnValues.threadData[$threadID]);
		}
	}
});

/**
 * Shows similar threads when creating a new thread.
 */
WBB.Thread.SimilarThreads = Class.extend({
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * input element
	 * @var jQuery
	 */
	_element: null,
	
	/**
	 * board id
	 * @var integer
	 */
	_boardID: 0,
	
	/**
	 * Initializes the loader.
	 */
	init: function(element, boardID) {
		this._element = element;
		this._boardID = boardID;
		this._proxy = new WCF.Action.Proxy({
			showLoadingOverlay: false,
			success: $.proxy(this._success, this)
		});
		
		element.on('blur', $.proxy(this._load, this));
	},
	
	/**
	 * Loads the similar threads.
	 * 
	 * @param	object		event
	 */
	_load: function(event) {
		var $input = $(event.currentTarget);
		var $value = $input.val();
		var $languageID = null;
		
		if ($value != '' && $value.length > 3) {
			var $languageInput = $('#languageIDContainer > input[name=languageID]');
			if ($languageInput.length) $languageID = $languageInput.val();
			
			this._proxy.setOption('data', {
				actionName: 'getSimilarThreads',
				className: 'wbb\\data\\thread\\ThreadAction',
				parameters: {
					topic: $value,
					boardID: this._boardID,
					languageID: $languageID
				}
			});
			this._proxy.sendRequest();
		}
		else {
			this._element.parent().find('.wbbInlineSimilarThreadList').remove();
		}
	},
	
	/**
	 * Handles successful AJAX requests.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		if (this._element.parent().find('.wbbInlineSimilarThreadList').length > 0) {
			this._element.parent().find('.wbbInlineSimilarThreadList').replaceWith(data.returnValues.template);
		}
		else {		
			this._element.parent().append(data.returnValues.template);
		}
	}
});

/**
 * Handles unsubscribing multiple threads.
 */
WBB.Thread.WatchedThreadList = Class.extend({
	/**
	 * button to stop watching all/marked threads
	 * @var	jQuery
	 */
	_button: null,
	
	/**
	 * mark all-checkbox
	 * @var	jQuery
	 */
	_markAllCheckbox: null,
	
	/**
	 * Creates a new instance of WBB.Thread.WatchedThreadList.
	 */
	init: function() {
		this._button = $('#stopWatchingButton').click($.proxy(this._stopWatching, this));
		this._markAllCheckbox = $('.jsMarkAllWatchedThreads').change($.proxy(this._markAll, this));
		
		$('.jsWatchedThread').change($.proxy(this._mark, this));
	},
	
	/**
	 * Handles changing a watched thread checkbox.
	 */
	_mark: function(event) {
		$(event.target).parents('tr').toggleClass('jsMarked');
		
		if (this._markAllCheckbox.is(':checked')) {
			this._markAllCheckbox.prop('checked', false);
		}
		else {
			this._markAllCheckbox.prop('checked', $('.jsWatchedThread:not(:checked)').length == 0);
		}
		
		this._updateButtonLabel();
	},
	
	/**
	 * Handles changing the 'mark all' checkbox.
	 * 
	 * @param	object		event
	 */
	_markAll: function(event) {
		$('.jsWatchedThread').prop('checked', this._markAllCheckbox.prop('checked')).parents('tr').toggleClass('jsMarked', this._markAllCheckbox.prop('checked'));
		
		this._updateButtonLabel();
	},
	
	/**
	 * Handles a click on the stop watching-button.
	 * 
	 * @param	object		event
	 */
	_stopWatching: function(event) {
		var $selectedThreads = $('.jsWatchedThread:checked');
		var $threadIDs = [ ];
		var $stopWatchingAll = false;
		if ($selectedThreads.length) {
			$selectedThreads.each(function(index, element) {
				$threadIDs.push($(element).data('objectID'));
			});
		}
		else {
			$stopWatchingAll = true;
		}
		
		var $languageItem = 'wbb.thread.watchedThreads.stopWatchingMarked.confirmMessage';
		if ($stopWatchingAll) {
			$languageItem = 'wbb.thread.watchedThreads.stopWatchingAll.confirmMessage';
		}
		
		WCF.System.Confirmation.show(WCF.Language.get($languageItem), function(action) {
			if (action === 'confirm') {
				new WCF.Action.Proxy({
					autoSend: true,
					data: {
						actionName: 'stopWatching',
						className: 'wbb\\data\\thread\\ThreadAction',
						parameters: {
							stopWatchingAll: $stopWatchingAll,
							threadIDs: $threadIDs
						}
					},
					success: function() {
						window.location.reload();
					}
				});
			}
		});
	},
	
	/**
	 * Updates the label of the 'stop watching' button.
	 * 
	 * @param	boolean		showMarkedText
	 */
	_updateButtonLabel: function() {
		var $selectedThreads = $('.jsWatchedThread:checked');
		
		var $text = '';
		if ($selectedThreads.length) {
			$text = WCF.Language.get('wbb.thread.watchedThreads.stopWatchingMarked', {
				count: $selectedThreads.length
			});
		}
		else {
			$text = WCF.Language.get('wbb.thread.watchedThreads.stopWatchingAll');
		}
		
		this._button.html($text);
	}
});

/**
 * Namespace for post related actions
 */
WBB.Post = {};

/**
 * Provides extended actions for post clipboard actions.
 */
WBB.Post.Clipboard = Class.extend({
	/**
	 * post handler
	 * @var	WBB.Post.Handler
	 */
	_postHandler: null,
	
	/**
	 * thread update handler
	 * @var	WBB.Thread.UpdateHandler
	 */
	_threadUpdateHandler: null,
	
	/**
	 * Initializes a new WBB.Post.Clipboard object.
	 * 
	 * @param	WBB.Post.Handler	postHandler
	 */
	init: function(postHandler) {
		this._postHandler = postHandler;
		
		// bind listener
		$('.jsClipboardEditor').each($.proxy(function(index, container) {
			var $container = $(container);
			var $types = eval($container.data('types'));
			if (WCF.inArray('com.woltlab.wbb.post', $types)) {
				$container.on('clipboardAction', $.proxy(this._execute, this));
				$container.on('clipboardActionResponse', $.proxy(this._evaluateResponse, this));
				return false;
			}
		}, this));
	},
	
	/**
	 * Sets the thread update handler object.
	 * 
	 * @param	WBB.Thread.UpdateHandler
	 */
	setThreadUpdateHandler: function(threadUpdateHandler) {
		this._threadUpdateHandler = threadUpdateHandler;
	},
	
	/**
	 * Handles clipboard actions.
	 * 
	 * @param	object		event
	 * @param	string		type
	 * @param	string		actionName
	 * @param	object		parameters
	 */
	_execute: function(event, type, actionName, parameters) {
		if (type !== 'com.woltlab.wbb.post') {
			return;
		}
		
		switch (actionName) {
			case 'com.woltlab.wbb.post.merge':
				var $mergeHandler = new WBB.Post.MergeHandler(parameters.objectIDs);
				$mergeHandler.load();
			break;
			
			case 'com.woltlab.wbb.post.moveToNewThread':
				WBB.Post.MoveToNewThread.prepare(parameters);
			break;
		}
	},
	
	/**
	 * Evaluates AJAX responses.
	 * 
	 * @param	object		event
	 * @param	object		data
	 * @param	string		type
	 * @param	string		actionName
	 * @param	object		parameters
	 */
	_evaluateResponse: function(event, data, type, actionName, parameters) {
		// ignore unrelated events
		if (type !== 'com.woltlab.wbb.post') {
			return;
		}
		
		if (actionName === 'com.woltlab.wbb.post.moveToExistingThread') {
			var $notification = new WCF.System.Notification();
			$notification.show(function() {
				window.location = data.returnValues.redirectURL;
			});
			
			return;
		}
		
		if (data.returnValues.postData  && $.getLength(data.returnValues.postData)) {
			this._postHandler.updatePosts(data);
		}
		
		if (this._threadUpdateHandler && data.returnValues.threadData  && $.getLength(data.returnValues.threadData)) {
			for (var $threadID in data.returnValues.threadData) {
				this._threadUpdateHandler.update($threadID, data.returnValues.threadData[$threadID]);
			}
		}
	}
});

/**
 * Merges multiple posts.
 * 
 * @see	WBB.Thread.MergeHandler
 */
WBB.Post.MergeHandler = WBB.Thread.MergeHandler.extend({
	/**
	 * @see	WBB.Thread.MergeHandler.init()
	 */
	init: function(objectIDs) {
		this._super(objectIDs);
		
		this._className = 'wbb\\data\\post\\PostAction';
		this._dialogTitle = WCF.Language.get('wbb.post.edit.merge');
		this._successMessage = WCF.Language.get('wbb.post.edit.merge.success');
	},
	
	/**
	 * @see	WBB.Thread.MergeHandler._success()
	 */
	_success: function(data, textStatus, jqXHR) {
		this._super(data, textStatus, jqXHR);
		
		if (data.returnValues.redirectURL === undefined) {
			this._dialog.find('.formSubmit > button[data-type=submit]').enable();
		}
	},
	
	/**
	 * @see	WBB.Thread.MergeHandler._getParameters()
	 */
	_getParameters: function() {
		var $postOrder = [ ];
		this._dialog.find('.jsPostContainer').each(function(index, container) {
			$postOrder.push($(container).data('postID'));
		});
		
		return {
			postOrder: $postOrder
		};
	}
});

/**
 * Moves selected posts into a new thread.
 */
WBB.Post.MoveToNewThread = {
	/**
	 * board select element
	 * @var	jQuery
	 */
	_boardSelect: null,
	
	/**
	 * dialog overlay
	 * @var	jQuery
	 */
	_dialog: null,
	
	/**
	 * list of affected post ids
	 * @var	array<integer>
	 */
	_objectIDs: [ ],
	
	/**
	 * submit button
	 * @var	jQuery
	 */
	_submitButton: null,
	
	/**
	 * topic input element
	 * @var	jQuery
	 */
	_topic: null,
	
	/**
	 * Prepares moving of posts.
	 * 
	 * @param	object		parameters
	 */
	prepare: function(parameters) {
		if (this._dialog === null) {
			this._dialog = $('<div>' + parameters.template + '</div>').hide().appendTo(document.body);
			this._dialog.wcfDialog({
				title: WCF.Language.get('wbb.post.moveToNewThread')
			});
		}
		else {
			this._dialog.html($.parseHTML(parameters.template)).wcfDialog('open');
		}
		
		this._boardSelect = $('#postMoveNewThreadBoardID');
		this._objectIDs = parameters.objectIDs;
		this._submitButton = this._dialog.find('.formSubmit > button[data-type=submit]').disable().click($.proxy(this._submit, this));
		this._topic = $('#postMoveNewThreadTopic').keyup($.proxy(this._keyUp, this)).focus();
	},
	
	/**
	 * Handles the 'keyup' event to enable/disable the submit button.
	 */
	_keyUp: function() {
		if ($.trim(this._topic.val()).length > 0) {
			this._submitButton.enable();
		}
		else {
			this._submitButton.disable();
		}
	},
	
	/**
	 * Submits the form input elements.
	 */
	_submit: function() {
		this._submitButton.disable();
		
		new WCF.Action.Proxy({
			autoSend: true,
			data: {
				actionName: 'moveToNewThread',
				className: 'wbb\\data\\post\\PostAction',
				objectIDs: this._objectIDs,
				parameters: {
					boardID: parseInt(this._boardSelect.val()),
					topic: $.trim(this._topic.val())
				}
			},
			success: $.proxy(this._success, this)
		});
	},
	
	/**
	 * Handles successful AJAX requests.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		var $notification = new WCF.System.Notification();
		$notification.show(function() {
			window.location = data.returnValues.redirectURL;
		});
	}
};

/**
 * Provides a flexible post preview.
 * 
 * @see	WCF.Popover
 */
WBB.Post.Preview = WCF.Popover.extend({
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * @see	WCF.Popover.init()
	 */
	init: function() {
		this._super('.wbbTopicLink, .wbbPostLink');
		
		// init proxy
		this._proxy = new WCF.Action.Proxy({
			showLoadingOverlay: false
		});
		
		WCF.DOMNodeInsertedHandler.addCallback('WBB.Post.Preview', $.proxy(this._initContainers, this));
	},
	
	/**
	 * @see	WCF.Popover._loadContent()
	 */
	_loadContent: function() {
		var $link = $('#' + this._activeElementID);
		
		if ($link.hasClass('wbbTopicLink')) {
			this._proxy.setOption('data', {
				actionName: 'getPostPreview',
				className: 'wbb\\data\\thread\\ThreadAction',
				objectIDs: [ $link.data('threadID') ],
				parameters: {
					sortOrder: ($link.data('sortOrder') ? $link.data('sortOrder') : 'ASC')
				}
			});
		}
		else {
			this._proxy.setOption('data', {
				actionName: 'getPostPreview',
				className: 'wbb\\data\\post\\PostAction',
				objectIDs: [ $link.data('postID') ]
			});
		}
		
		var $elementID = this._activeElementID;
		var self = this;
		this._proxy.setOption('success', function(data, textStatus, jqXHR) {
			self._insertContent($elementID, data.returnValues.template, true);
		});
		this._proxy.setOption('failure', function(data, jqXHR, textStatus, errorThrown) {
			self._insertContent($elementID, data.message, true);
			
			return false;
		});
		this._proxy.sendRequest();
	}
});

WBB.Post.Add = {};

/**
 * Notification when a new post is added while writing.
 */
/*
WBB.Post.Add.Notification = Class.extend({
	_threadID: 0,
	_replies: 0,
	_proxy: null,
	init: function($threadID, $replies) {
		this._threadID = $threadID;
		this._replies = $replies;
		this._proxy = new WCF.Action.Proxy({
			success: $.proxy(this._success, this),
			showLoadingOverlay: false
		});
		new WCF.PeriodicalExecuter($.proxy(this._check, this), 60000);
	},
	_check: function() {
		this._proxy.setOption('data', {
			actionName: 'countReplies',
			className: '\\wbb\\data\\thread\\ThreadAction',
			objectIDs: [ this._threadID ]
		});
		
		this._proxy.sendRequest();
		
	},
	_success: function(data, textStatus, jqXHR) {
		if (data.returnValues == this._replies) return;
		var notification = new WCF.System.Notification(WCF.String.formatNumeric(data.returnValues - this._replies)+' new posts in the meantime');
		notification.show(null, 10000);
		
		this._replies = data.returnValues;
	}
});*/

/**
 * Post modification handler.
 * 
 * @param	string			selector
 * @param	WBB.Post.InlineEditor	postEditor
 */
WBB.Post.Handler = Class.extend({
	/**
	 * post editor
	 * @var	WBB.Post.InlineEditor
	 */
	_editor: null,
	
	/**
	 * loader for collapsed posts
	 * @var	WBB.Post.CollapsedPostLoader
	 */
	_collapsedPostLoader: null,
	
	/**
	 * list of posts
	 * @var	object
	 */
	_posts: { },
	
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * post selector
	 * @var	string
	 */
	_selector: '',
	
	/**
	 * thread update handler object
	 * @var	WBB.Thread.UpdateHandler
	 */
	_threadUpdateHandler: null,
	
	/**
	 * Initializes the WBB.Post.Handler class.
	 * 
	 * @param	string			selector
	 */
	init: function(selector) {
		this._editor = null;
		this._selector = selector;
		this._proxy = new WCF.Action.Proxy({
			success: $.proxy(this.updatePosts, this)
		});
		
		this._initPosts();
		WCF.DOMNodeInsertedHandler.addCallback('WBB.Post.Handler', $.proxy(this._initPosts, this));
	},
	
	/**
	 * Sets the loader for collapsed posts.
	 * 
	 * @param	WBB.Post.CollapsedPostLoader	collapsedPostLoader
	 */
	setCollapsedPostLoader: function(collapsedPostLoader) {
		this._collapsedPostLoader = collapsedPostLoader;
	},
	
	/**
	 * Sets post editor object
	 * 
	 * @param	WBB.Post.InlineEditor	editor
	 */
	setPostEditor: function(editor) {
		this._editor = editor;
	},
	
	/**
	 * Sets the post object of the post with the given id.
	 * 
	 * @param	integer		postID
	 * @param	jQuery		post
	 */
	setPost: function(postID, post) {
		this._posts[postID] = post;
	},
	
	/**
	 * Sets the thread update handler of the thread the handled posts belong to.
	 * 
	 * @param	WBB.Thread.UpdateHandler	threadUpdateHandler
	 */
	setThreadUpdateHandler: function(threadUpdateHandler) {
		this._threadUpdateHandler = threadUpdateHandler;
	},
	
	/**
	 * Initializes all available posts.
	 */
	_initPosts: function() {
		$(this._selector).each($.proxy(function(index, post) {
			var $post = $(post);
			var $postID = $post.data('objectID');
			
			if (this._posts[$postID] === undefined) {
				this.setPost($postID, $post);
			}
		}, this));
	},
	
	/**
	 * Updates the given data field for all posts with the given data value.
	 * 
	 * @param	string		dataField
	 * @param	string		dataValue
	 */
	updateAllPosts: function(dataField, dataValue) {
		// build object to use updatePosts
		var $postData = { };
		for (var $postID in this._posts) {
			$postData[$postID] = { };
			$postData[$postID][dataField] = dataValue;
		}
		
		this.updatePosts({
			returnValues: {
				postData: $postData
			}
		});
	},
	
	/**
	 * Updates post data.
	 * 
	 * @param	object		data
	 */
	updatePosts: function(data) {
		var $postIDs = [ ];
		for (var $postID in data.returnValues.postData) {
			if (this._posts[$postID] === undefined) {
				continue;
			}
			
			var $post = this._posts[$postID];
			var $postData = data.returnValues.postData[$postID];
			for (var $optionName in $postData) {
				$post.data($optionName, $postData[$optionName]);
				
				if ($optionName === 'isDeleted') {
					if ($postData[$optionName]) {
						$post.addClass('messageDeleted');
					}
					else {
						if ($post.hasClass('wbbPostDeleted')) {
							$postIDs.push($postID);
						}
						$post.removeClass('messageDeleted');
						$post.find('.wbbPostDeleteNote').remove();
					}
				}
				else if ($optionName === 'isDisabled') {
					if ($postData[$optionName]) {
						$post.addClass('messageDisabled');
					}
					else {
						$post.removeClass('messageDisabled');
					}
				}
				else if ($optionName === 'deleted') {
					$post.parent().remove();
				}
			}
		}
		
		if ($postIDs.length && this._collapsedPostLoader) {
			this._collapsedPostLoader.loadPosts($postIDs);
		}
		
		if (this._threadUpdateHandler && data.returnValues.threadData) {
			for (var $threadID in data.returnValues.threadData) {
				this._threadUpdateHandler.update($threadID, data.returnValues.threadData[$threadID]);
			}
		}
		
		if (this._editor !== null) {
			this._editor.rebuild();
		}
		
		WCF.Clipboard.reload();
	},
	
	/**
	 * Updates a post.
	 * 
	 * @param	integer		postID
	 * @param	string		actionName
	 * @param	object		parameters
	 */
	update: function(postID, actionName, parameters) {
		parameters = parameters || { };
		
		this._proxy.setOption('data', {
			actionName: actionName,
			className: 'wbb\\data\\post\\PostAction',
			objectIDs: [ postID ],
			parameters: parameters
		});
		this._proxy.sendRequest();
	},
	
	/**
	 * Returns a post by post id.
	 * 
	 * @param	integer		postID
	 * @return	jQuery
	 */
	getPost: function(postID) {
		if (this._posts[postID]) {
			return this._posts[postID];
		}
		
		return null;
	}
});

/**
 * Like support for posts
 * 
 * @see	WCF.Like
 */
WBB.Post.Like = WCF.Like.extend({
	/**
	 * @see	WCF.Like._getContainers()
	 */
	_getContainers: function() {
		return $('article.message:not(.messageCollapsed)');
	},

	/**
	 * @see	WCF.Like._getObjectID()
	 */
	_getObjectID: function(containerID) {
		return this._containers[containerID].data('postID');
	},

	/**
	 * @see	WCF.Like._getWidgetContainer()
	 */
	_getWidgetContainer: function(containerID) {
		return this._containers[containerID].find('.messageHeader');
	},
	
	/**
	 * @see	WCF.Like._buildWidget()
	 */
	_buildWidget: function(containerID, likeButton, dislikeButton, badge, summary) {
		var $widgetContainer = this._getWidgetContainer(containerID);
		if (this._canLike) {
			var $smallButtons = this._containers[containerID].find('.smallButtons');
			likeButton.insertBefore($smallButtons.find('.toTopLink'));
			dislikeButton.insertBefore($smallButtons.find('.toTopLink'));
			dislikeButton.find('a').addClass('button');
			likeButton.find('a').addClass('button');
		}
		
		if (summary) {
			summary.appendTo(this._containers[containerID].find('.messageBody > .messageFooter'));
			summary.addClass('messageFooterNote');
		}
		$widgetContainer.find('.permalink').after(badge);
	},
	
	/**
	 * Sets button active state.
	 * 
	 * @param 	jquery		likeButton
	 * @param 	jquery		dislikeButton
	 * @param	integer		likeStatus
	 */
	_setActiveState: function(likeButton, dislikeButton, likeStatus) {
		likeButton = likeButton.find('.button').removeClass('active');
		dislikeButton = dislikeButton.find('.button').removeClass('active');
		
		if (likeStatus == 1) {
			likeButton.addClass('active');
		}
		else if (likeStatus == -1) {
			dislikeButton.addClass('active');
		}
	},
	
	/**
	 * @see	WCF.Like._addWidget()
	 */
	_addWidget: function(containerID, widget) {}
});

/**
 * Provides an inline post editor.
 * 
 * @see	WCF.Message.InlineEditor
 */
WBB.Post.InlineEditor = WCF.Message.InlineEditor.extend({
	/**
	 * post handler object
	 * @var	WBB.Post.Handler
	 */
	_postHandler: null,
	
	/**
	 * @see	WCF.Message.InlineEditor.init()
	 */
	init: function(containerID, postHandler, quoteManager) {
		this._super(containerID, true, quoteManager);
		
		this._postHandler = postHandler;
		this._postHandler.setPostEditor(this);
	},
	
	/**
	 * @see	WCF.Message.InlineEditor._getClassName()
	 */
	_getClassName: function() {
		return 'wbb\\data\\post\\PostAction';
	},
	
	/**
	 * @see	WCF.Message.InlineEditor._initDropdownMenu()
	 */
	_initDropdownMenu: function(containerID, dropdownMenu) {
		var $container = this._container[containerID];
		var $postID = $container.data('objectID');
		
		// isDisabled
		if ($container.data('canEnable')) {
			var $enableButton = $('<li class="jsEnablePost"><span>' + WCF.Language.get('wbb.post.edit.enable') + '</span></li>').appendTo(dropdownMenu);
			$enableButton.data('postID', $postID).data('actionName', 'enable').click($.proxy(this._updatePost, this));
			
			var $disableButton = $('<li class="jsDisablePost"><span>' + WCF.Language.get('wbb.post.edit.disable') + '</span></li>').appendTo(dropdownMenu);
			$disableButton.data('postID', $postID).data('actionName', 'disable').click($.proxy(this._updatePost, this));
			
			if ($container.data('isDisabled') || $container.data('isDeleted')) {
				$disableButton.hide();
			}
			if (!$container.data('isDisabled') || $container.data('isDeleted')) {
				$enableButton.hide();
			}
		}
		
		// isClosed
		if ($container.data('canClose')) {
			var $closeButton = $('<li class="jsClosePost"><span>' + WCF.Language.get('wbb.post.edit.close') + '</span></li>').appendTo(dropdownMenu);
			var $openButton = $('<li class="jsOpenPost"><span>' + WCF.Language.get('wbb.post.edit.open') + '</span></li>').appendTo(dropdownMenu);
			
			$closeButton.data('postID', $postID).data('actionName', 'close').click($.proxy(this._updatePost, this));
			$openButton.data('postID', $postID).data('actionName', 'open').click($.proxy(this._updatePost, this));
			
			if ($container.data('isClosed')) {
				$closeButton.hide();
			}
			else {
				$openButton.hide();
			}
		}
		
		// isDeleted
		if ($container.data('canDelete')) {
			var $deleteButton = $('<li class="jsDeletePost"><span>' + WCF.Language.get('wbb.post.edit.delete') + '</span></li>').appendTo(dropdownMenu);
			$deleteButton.data('postID', $postID).data('actionName', 'trash').click($.proxy(this._updatePost, this));
			
			if ($container.data('isDeleted')) {
				$deleteButton.hide();
			}
		}
		if ($container.data('canRestore')) {
			var $restoreButton = $('<li class="jsRestorePost"><span>' + WCF.Language.get('wbb.post.edit.restore') + '</span></li>').appendTo(dropdownMenu);
			$restoreButton.data('postID', $postID).data('actionName', 'restore').click($.proxy(this._updatePost, this));
			
			if (!$container.data('isDeleted')) {
				$restoreButton.hide();
			}
		}
		if ($container.data('canDeleteCompletely')) {
			var $deleteCompletelyButton = $('<li class="jsDeletePostCompletely"><span>' + WCF.Language.get('wbb.post.edit.deleteCompletely') + '</span></li>').appendTo(dropdownMenu);
			$deleteCompletelyButton.data('postID', $postID).data('actionName', 'delete').click($.proxy(this._updatePost, this));
			
			if (!$container.data('isDeleted')) {
				$deleteCompletelyButton.hide();
			}
		}
		
		if ($container.data('canEdit')) {
			$('<li class="dropdownDivider" />').appendTo(dropdownMenu);
			
			// jump to WYSIWYG editor
			$('<li><span>' + WCF.Language.get('wbb.post.edit') + '</span></li>').click($.proxy(function() {
				this._click(null, containerID);
			}, this)).appendTo(dropdownMenu);
		}
	},
	
	/**
	 * Updates current post.
	 * 
	 * @param	object		event
	 */
	_updatePost: function(event) {
		var $item = $(event.currentTarget);
		var $container = this._postHandler.getPost($item.data('postID'));
		var $showSuccess = false;
		
		switch ($item.data('actionName')) {
			case 'trash':
				var $template = $('<fieldset><dl></dl></fieldset>');
				$('<dt><label for="postTrashReason">' + WCF.Language.get('wbb.post.edit.trash.reason') + '</label></dt>').appendTo($template.children('dl'));
				$('<dd><textarea id="postTrashReason" name="reason" cols="40" rows="3" /></dd>').appendTo($template.children('dl'));
				
				WCF.System.Confirmation.show(WCF.Language.get('wbb.post.edit.trash.confirmMessage'), $.proxy(function(action) {
					if (action === 'confirm') {
						this._postHandler.update($item.data('postID'), $item.data('actionName'), { reason: $('#postTrashReason').val() });
						
						if ($container !== null && $container.data('canEditInline') && !$container.data('canEdit')) {
							new WCF.System.Notification().show();
						}
					}
				}, this), '', $template);
			break;
			
			case 'delete':
				WCF.System.Confirmation.show(WCF.Language.get('wbb.post.edit.delete.confirmMessage'), $.proxy(function(action) {
					if (action === 'confirm') {
						this._postHandler.update($item.data('postID'), $item.data('actionName'));
						
						if ($container !== null && $container.data('canEditInline') && !$container.data('canEdit')) {
							new WCF.System.Notification().show();
						}
					}
				}, this));
			break;
			
			default:
				this._postHandler.update($item.data('postID'), $item.data('actionName'));
				$showSuccess = true;
			break;
		}
		
		if ($showSuccess && $container !== null && $container.data('canEditInline') && !$container.data('canEdit')) {
			new WCF.System.Notification().show();
		}
	},
	
	/**
	 * Rebuilds dropdown menus.
	 */
	rebuild: function() {
		for (var $containerID in this._container) {
			var $container = this._container[$containerID];
			var $postID = $container.data('objectID');
			
			if (this._dropdowns[$postID] === undefined) {
				continue;
			}
			
			var $dropdownMenu = this._dropdowns[$postID];
			
			$dropdownMenu.children('.jsEnablePost, .jsDisablePost, .jsClosePost, .jsOpenPost, .jsRestorePost, .jsDeletePostCompletely, .jsDeletePost').hide();
			
			if ($container.data('canClose')) {
				if ($container.data('isClosed')) {
					$dropdownMenu.children('.jsOpenPost').show();
				}
				else {
					$dropdownMenu.children('.jsClosePost').show();
				}
			}
			
			if ($container.data('isDeleted')) {
				if ($container.data('canRestore')) {
					$dropdownMenu.children('.jsRestorePost').show();
				}
				if ($container.data('canDeleteCompletely')) {
					$dropdownMenu.children('.jsDeletePostCompletely').show();
				}
			}
			else {
				if ($container.data('canDelete')) {
					$dropdownMenu.children('.jsDeletePost').show();
				}
			}
			
			if ($container.data('canEnable') && !$container.data('isDeleted')) {
				if ($container.data('isDisabled')) {
					$dropdownMenu.children('.jsEnablePost').show();
				}
				else {
					$dropdownMenu.children('.jsDisablePost').show();
				}
			}
		}
	},
	
	/**
	 * @see	WCF.Message.InlineEditor._getHash()
	 */
	_getHash: function(objectID) {
		return '#post' + objectID;
	}
});

/**
 * Loads collapsed posts.
 */
WBB.Post.CollapsedPostLoader = Class.extend({
	/**
	 * post handler used to update the stored posts there if a collapsed post
	 * is loaded
	 * @param	WBB.Post.Handler
	 */
	_postHandler: null,
	
	/**
	 * list of posts
	 * @var	object
	 */
	_posts: { },
	
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * Initializes the post loader.
	 */
	init: function(selector) {
		var self = this;
		$(selector).each(function(index, article) {
			var $postID = $(article).data('postID');
			var $post = $('#post' + $postID);
			
			self._posts[$postID] = $post.click($.proxy(self._loadPost, self));
		});
		
		if ($.getLength(this._posts)) {
			this._proxy = new WCF.Action.Proxy({
				success: $.proxy(this._success, this)
			});
		}
	},
	
	/**
	 * Sets the post handler.
	 * 
	 * @param	WBB.Post.Handler	postHandler
	 */
	setPostHandler: function(postHandler) {
		this._postHandler = postHandler;
		this._postHandler.setCollapsedPostLoader(this);
	},
	
	/**
	 * Loads the posts with the given ids.
	 */
	loadPosts: function(postIDs) {
		this._proxy.setOption('data', {
			actionName: 'getPosts',
			className: 'wbb\\data\\post\\PostAction',
			objectIDs: postIDs,
			parameters: {
				showCollapsedPosts: true
			}
		});
		this._proxy.sendRequest();
	},
	
	/**
	 * Loads a collapsed post.
	 * 
	 * @param	object		event
	 */
	_loadPost: function(event) {
		var $target = $(event.target);
		if ($target.getTagName() === 'a' || $target.getTagName() === 'input') {
			// ignore clicks on links or checkboxes
			return;
		}
		
		this.loadPosts([$(event.currentTarget).children('article').data('postID')]);
	},
	
	/**
	 * Handles successful AJAX requests.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		var $posts = $('' + data.returnValues.template);
		$posts.each($.proxy(function(index, element) {
			var $element = $(element);
			
			if ($element.is('li')) {
				var $post = $element.children('article');
				var $postID = $post.data('postID');
				if (this._posts[$postID]) {
					if ($posts.length === 1) {
						$element.hide();
					}
					this._posts[$postID].replaceWith($element);
					
					if ($posts.length === 1) {
						$element.wcfFadeIn(function() {
							var $scrollHandler = new WCF.Effect.Scroll();
							$scrollHandler.scrollTo($element, true);
						});
					}
					
					if (this._postHandler) {
						this._postHandler.setPost($postID, $post);
					}
				}
			}
		}, this));
		
		// reload clipboard items
		WCF.Clipboard.reload();
	}
});

/**
 * Displays an overlay for post ips.
 */
WBB.Post.IPAddressHandler = Class.extend({
	/**
	 * template cache
	 * @var	object
	 */
	_cache: { },
	
	/**
	 * dialog object
	 * @var	jQuery
	 */
	_dialog: null,
	
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * Initializes the post ip overlay.
	 */
	init: function() {
		this._cache = { };
		this._dialog = null;
		this._proxy = new WCF.Action.Proxy({
			success: $.proxy(this._success, this)
		});
		
		this._initButtons();
		
		WCF.DOMNodeInsertedHandler.addCallback('WBB.Post.IPAddressHandler', $.proxy(this._initButtons, this));
	},
	
	/**
	 * Initializes the button events.
	 */
	_initButtons: function() {
		var self = this;
		$('.jsIpAddress').each(function(index, button) {
			var $button = $(button);
			var $postID = $button.data('postID');
			
			if (self._cache[$postID] === undefined) {
				self._cache[$postID] = '';
				$button.click($.proxy(self._click, self));
			}
		});
	},
	
	/**
	 * Handles clicks on the show ip button.
	 * 
	 * @param	object		event
	 */
	_click: function(event) {
		var $postID = $(event.currentTarget).data('postID');
		
		if (this._cache[$postID]) {
			this._showDialog($postID);
		}
		else {
			this._proxy.setOption('data', {
				actionName: 'getIpLog',
				className: 'wbb\\data\\post\\PostAction',
				parameters: {
					postID: $postID
				}
			});
			this._proxy.sendRequest();
		}
	},
	
	/**
	 * Handles successful AJAX requests.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		// cache template
		this._cache[data.returnValues.postID] = data.returnValues.template;
		
		// show dialog
		this._showDialog(data.returnValues.postID);
	},
	
	/**
	 * Shows the overlay for given post id.
	 * 
	 * @param	integer		postID
	 */
	_showDialog: function(postID) {
		if (this._dialog === null) {
			this._dialog = $('<div id="wbbIpAddressLog" />').hide().appendTo(document.body);
		}
		
		this._dialog.html(this._cache[postID]);
		this._dialog.wcfDialog({
			title: WCF.Language.get('wbb.post.ipAddress.title')
		});
		this._dialog.wcfDialog('render');
	}
});

/**
 * Provides the quote manager for posts.
 * 
 * @param	WCF.Message.Quote.Manager	quoteManager
 * @see		WCF.Message.Quote.Handler
 */
WBB.Post.QuoteHandler = WCF.Message.Quote.Handler.extend({
	/**
	 * @see	WCF.Message.QuoteManager.init()
	 */
	init: function(quoteManager) {
		this._super(quoteManager, 'wbb\\data\\post\\PostAction', 'com.woltlab.wbb.post', '.message', '.messageBody', '.messageBody > div > div.messageText');
	}
});
