<?php
namespace wbb\acp\form;
use wbb\data\board\BoardCache;
use wbb\data\board\BoardNodeList;
use wbb\data\post\PostAction;
use wbb\data\post\PostList;
use wcf\form\AbstractForm;
use wcf\system\database\util\PreparedStatementConditionBuilder;
use wcf\system\exception\UserInputException;
use wcf\system\WCF;
use wcf\util\ArrayUtil;
use wcf\util\StringUtil;

/**
 * Shows the bulk processing form for posts.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	acp.form
 * @category	Burning Board
 */
class PostBulkProcessingForm extends AbstractForm {
	/**
	 * @see	wcf\page\AbstractPage::$activeMenuItem
	 */
	public $activeMenuItem = 'wbb.acp.menu.link.threadsAndPosts.postBulkProcessing';
	
	/**
	 * @see	wcf\page\AbstractPage::$neededPermissions
	 */
	public $neededPermissions = array('admin.board.canEditBoard');
	
	/**
	 * list of available actions
	 * @var	array<string>
	 */
	public $availableActions = array('trash', 'delete', 'restore', 'disable', 'enable', 'close', 'open');
	
	/**
	 * number of affected posts
	 * @var	integer
	 */
	public $affectedPosts = 0;
	
	/**
	 * time after
	 * @var	string
	 */
	public $timeAfter = '';
	
	/**
	 * time before
	 * @var	string
	 */
	public $timeBefore = '';

	/**
	 * names of users who created the posts
	 * @var	string
	 */
	public $createdBy = '';
	
	/**
	 * deleted state
	 * @var	boolean
	 */
	public $deleted = 0;

	/**
	 * not deleted state
	 * @var	boolean
	 */
	public $notDeleted = 0;
	
	/**
	 * disabled state
	 * @var	boolean
	 */
	public $disabled = 0;
	
	/**
	 * enabled state
	 * @var	boolean
	 */
	public $notDisabled = 0;
	
	/**
	 * closed state
	 * @var	boolean
	 */
	public $closed = 0;
	
	/**
	 * open state
	 * @var	boolean
	 */
	public $open = 0;
	
	/**
	 * post list object
	 * @var	wbb\data\post\PostList
	 */
	public $postList = null;
	
	/**
	 * selected board ids
	 * @var	array<integer>
	 */
	public $boardIDs = array();
	
	/**
	 * @see	wcf\form\IForm::readFormParameters()
	 */
	public function readFormParameters() {
		parent::readFormParameters();
		
		if (isset($_POST['timeAfter'])) $this->timeAfter = $_POST['timeAfter'];
		if (isset($_POST['timeBefore'])) $this->timeBefore = $_POST['timeBefore'];
		if (isset($_POST['createdBy'])) $this->createdBy = StringUtil::trim($_POST['createdBy']);
		if (isset($_POST['boardIDs']) && is_array($_POST['boardIDs'])) $this->boardIDs = ArrayUtil::toIntegerArray($_POST['boardIDs']);
		if (isset($_POST['deleted'])) $this->deleted = intval($_POST['deleted']);
		if (isset($_POST['notDeleted'])) $this->notDeleted = intval($_POST['notDeleted']);
		if (isset($_POST['disabled'])) $this->disabled = intval($_POST['disabled']);
		if (isset($_POST['notDisabled'])) $this->notDisabled = intval($_POST['notDisabled']);
		if (isset($_POST['closed'])) $this->closed = intval($_POST['closed']);
		if (isset($_POST['open'])) $this->open = intval($_POST['open']);
	}
	
	/**
	 * @see	wcf\form\IForm::validate()
	 */
	public function validate() {
		parent::validate();
		
		// action
		if (!in_array($this->action, $this->availableActions)) {
			throw new UserInputException('action');
		}
	}
	
	/**
	 * @see	wcf\form\IForm::save()
	 */
	public function save() {
		$this->postList = new PostList();
		
		parent::save();
		
		// time
		if (!empty($this->timeAfter) && ($timeAfter = @strtotime($this->timeAfter))) {
			$this->postList->getConditionBuilder()->add('time > ?', array($timeAfter));
		}
		if (!empty($this->timeBefore) && ($timeBefore = @strtotime($this->timeBefore))) {
			$this->postList->getConditionBuilder()->add('time < ?', array($timeBefore));
		}
		
		// username
		if (!empty($this->createdBy)) {
			$usernames = preg_split('/\s*,\s*/', $this->createdBy, -1, PREG_SPLIT_NO_EMPTY);
			$this->postList->getConditionBuilder()->add('username IN (?)', array($usernames));
		}
		
		// board ids
		if (!empty($this->boardIDs)) {
			$this->postList->getConditionBuilder()->add('threadID IN (SELECT threadID FROM wbb'.WCF_N.'_thread WHERE boardID IN (?))', array($this->boardIDs));
		}
		
		if ($this->deleted || $this->action == 'restore') $this->postList->getConditionBuilder()->add("isDeleted = ?", array(1));
		if ($this->notDeleted || $this->action == 'trash') $this->postList->getConditionBuilder()->add("isDeleted = ?", array(0));
		if ($this->disabled || $this->action == 'enable') $this->postList->getConditionBuilder()->add("isDisabled = ?", array(1));
		if ($this->notDisabled || $this->action == 'disable') $this->postList->getConditionBuilder()->add("isDisabled = ?", array(0));
		if ($this->closed || $this->action == 'open') $this->postList->getConditionBuilder()->add("isClosed = ?", array(1));
		if ($this->open || $this->action == 'close') $this->postList->getConditionBuilder()->add("isClosed = ?", array(0));
		
		$this->postList->readObjects();
		
		// execute action
		if (count($this->postList)) {
			$postAction = new PostAction($this->postList->getObjects(), $this->action, array('isBulkProcessing' => true));
			$postAction->executeAction();
		}
		
		$this->affectedPosts = count($this->postList);
		
		$this->saved();
		
		WCF::getTPL()->assign('affectedPosts', $this->affectedPosts);
	}
	
	/**
	 * @see	wcf\page\IPage::assignVariables()
	 */
	public function assignVariables() {
		parent::assignVariables();
		
		// board node list
		$boardNodeList = new BoardNodeList();
		$boardNodeList->readNodeTree();
		
		WCF::getTPL()->assign(array(
			'timeAfter' => $this->timeAfter,
			'timeBefore' => $this->timeBefore,
			'createdBy' => $this->createdBy,
			'deleted' => $this->deleted,
			'notDeleted' => $this->notDeleted,
			'disabled' => $this->disabled,
			'notDisabled' => $this->notDisabled,
			'closed' => $this->closed,
			'open' => $this->open,
			'boardIDs' => $this->boardIDs,
			'boardNodeList' => $boardNodeList->getNodeList()
		));
	}
}
