<?php
namespace wbb\acp\form;
use wbb\data\board\BoardCache;
use wbb\data\board\BoardNodeList;
use wbb\data\thread\ThreadAction;
use wbb\data\thread\ThreadList;
use wcf\data\language\Language;
use wcf\form\AbstractForm;
use wcf\system\database\util\PreparedStatementConditionBuilder;
use wcf\system\exception\UserInputException;
use wcf\system\language\LanguageFactory;
use wcf\system\WCF;
use wcf\util\ArrayUtil;
use wcf\util\StringUtil;

/**
 * Shows the threads bulk processing form.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	acp.form
 * @category	Burning Board
 */
class ThreadBulkProcessingForm extends AbstractForm {
	/**
	 * @see	wcf\page\AbstractPage::$activeMenuItem
	 */
	public $activeMenuItem = 'wbb.acp.menu.link.threadsAndPosts.threadBulkProcessing';
	
	/**
	 * @see	wcf\page\AbstractPage::$neededPermissions
	 */
	public $neededPermissions = array('admin.board.canEditBoard');
	
	/**
	 * list of available actions
	 * @var	array<string>
	 */
	public $availableActions = array('move', 'trash', 'delete', 'restore', 'disable', 'enable', 'close', 'open', 'removeLink', 'markAsDone');
	
	/**
	 * condition builder
	 * @var	wcf\system\database\util\PreparedStatementConditionBuilder
	 */
	public $conditionBuilder = null;
	
	/**
	 * number of affected threads
	 * @var	integer
	 */
	public $affectedThreads = 0;
	
	/**
	 * time after
	 * @var	string
	 */
	public $timeAfter = '';
	
	/**
	 * time before
	 * @var	string
	 */
	public $timeBefore = '';

	/**
	 * last post time after
	 * @var	string
	 */
	public $lastPostTimeAfter = '';
	
	/**
	 * last post time before
	 * @var	string
	 */
	public $lastPostTimeBefore = '';
	
	/**
	 * replies more than x
	 * @var	integer
	 */
	public $repliesMoreThan = '';
	
	/**
	 * replies less than x
	 * @var	integer
	 */
	public $repliesLessThan = '';
	
	/**
	 * usernames
	 * @var	string
	 */
	public $createdBy = '';
	
	/**
	 * deleted state
	 * @var	boolean
	 */
	public $deleted = 0;

	/**
	 * not deleted state
	 * @var	boolean
	 */
	public $notDeleted = 0;
	
	/**
	 * disabled state
	 * @var	boolean
	 */
	public $disabled = 0;
	
	/**
	 * enabled state
	 * @var	boolean
	 */
	public $notDisabled = 0;
	
	/**
	 * closed state
	 * @var	boolean
	 */
	public $closed = 0;
	
	/**
	 * open state
	 * @var	boolean
	 */
	public $open = 0;
	
	/**
	 * redirect state
	 * @var	boolean
	 */
	public $redirect = 0;
	
	/**
	 * no redirect state
	 * @var	boolean
	 */
	public $notRedirect = 0;
	
	/**
	 * announcement state
	 * @var	boolean
	 */
	public $announcement = 0;
	
	/**
	 * sticky state
	 * @var	boolean
	 */
	public $sticky = 0;
	
	/**
	 * normal state
	 * @var	boolean
	 */
	public $normal = 0;
	
	/**
	 * done state
	 * @var	boolean
	 */
	public $done = 0;
	
	/**
	 * undone state
	 * @var	boolean
	 */
	public $undone = 0;
	
	/**
	 * selected board ids
	 * @var array<integer>
	 */
	public $boardIDs = array();
	
	/**
	 * selected language ids
	 * @var	array<integer>
	 */
	public $languageIDs = array();
	
	/**
	 * new board
	 * @var	integer
	 */
	public $moveTo = 0;
	
	/**
	 * thread list object
	 * @var	wbb\data\thread\ThreadList
	 */
	public $threadList = null;
	
	/**
	 * @see	wcf\form\IForm::readFormParameters()
	 */
	public function readFormParameters() {
		parent::readFormParameters();
		
		if (isset($_POST['timeAfter'])) $this->timeAfter = $_POST['timeAfter'];
		if (isset($_POST['timeBefore'])) $this->timeBefore = $_POST['timeBefore'];
		if (isset($_POST['lastPostTimeAfter'])) $this->lastPostTimeAfter = $_POST['lastPostTimeAfter'];
		if (isset($_POST['lastPostTimeBefore'])) $this->lastPostTimeBefore = $_POST['lastPostTimeBefore'];
		if (!empty($_POST['repliesMoreThan'])) $this->repliesMoreThan = intval($_POST['repliesMoreThan']);
		if (!empty($_POST['repliesLessThan'])) $this->repliesLessThan = intval($_POST['repliesLessThan']);
		if (isset($_POST['createdBy'])) $this->createdBy = StringUtil::trim($_POST['createdBy']);
		if (isset($_POST['boardIDs']) && is_array($_POST['boardIDs'])) $this->boardIDs = ArrayUtil::toIntegerArray($_POST['boardIDs']);
		if (isset($_POST['deleted'])) $this->deleted = intval($_POST['deleted']);
		if (isset($_POST['notDeleted'])) $this->notDeleted = intval($_POST['notDeleted']);
		if (isset($_POST['disabled'])) $this->disabled = intval($_POST['disabled']);
		if (isset($_POST['notDisabled'])) $this->notDisabled = intval($_POST['notDisabled']);
		if (isset($_POST['closed'])) $this->closed = intval($_POST['closed']);
		if (isset($_POST['open'])) $this->open = intval($_POST['open']);
		if (isset($_POST['redirect'])) $this->redirect = intval($_POST['redirect']);
		if (isset($_POST['notRedirect'])) $this->notRedirect = intval($_POST['notRedirect']);
		if (isset($_POST['announcement'])) $this->announcement = intval($_POST['announcement']);
		if (isset($_POST['sticky'])) $this->sticky = intval($_POST['sticky']);
		if (isset($_POST['normal'])) $this->normal = intval($_POST['normal']);
		if (isset($_POST['done'])) $this->done = intval($_POST['done']);
		if (isset($_POST['undone'])) $this->undone = intval($_POST['undone']);
		if (isset($_POST['moveTo'])) $this->moveTo = intval($_POST['moveTo']);
		if (isset($_POST['languageIDs']) && is_array($_POST['languageIDs'])) $this->languageIDs = ArrayUtil::toIntegerArray($_POST['languageIDs']);
	}
	
	/**
	 * @see	wcf\form\IForm::validate()
	 */
	public function validate() {
		parent::validate();
		
		// action
		if (!in_array($this->action, $this->availableActions)) {
			throw new UserInputException('action');
		}
		
		// move to
		if ($this->action == 'move') {
			$board = BoardCache::getInstance()->getBoard($this->moveTo);
			if ($board === null) {
				throw new UserInputException('moveTo');
			}
			
			if (!$board->isBoard()) {
				throw new UserInputException('moveTo');
			}
		}
	}
	
	/**
	 * @see	wcf\form\IForm::save()
	 */
	public function save() {
		$this->threadList = new ThreadList();
		
		parent::save();
		
		// time
		if (!empty($this->timeAfter) && ($timeAfter = @strtotime($this->timeAfter))) {
			$this->threadList->getConditionBuilder()->add('time > ?', array($timeAfter));
		}
		if (!empty($this->timeBefore) && ($timeBefore = @strtotime($this->timeBefore))) {
			$this->threadList->getConditionBuilder()->add('time < ?', array($timeBefore));
		}
		
		// last post time
		if (!empty($this->lastPostTimeAfter) && ($lastPostTimeAfter = @strtotime($this->lastPostTimeAfter))) {
			$this->threadList->getConditionBuilder()->add('lastPostTime > ?', array($lastPostTimeAfter));
		}
		if (!empty($this->lastPostTimeBefore) && ($lastPostTimeBefore = @strtotime($this->lastPostTimeBefore))) {
			$this->threadList->getConditionBuilder()->add('lastPostTime < ?', array($lastPostTimeBefore));
		}
		
		// replies
		if (!empty($this->repliesMoreThan)) $this->threadList->getConditionBuilder()->add('replies > ?', array($this->repliesMoreThan));
		if (!empty($this->repliesLessThan)) $this->threadList->getConditionBuilder()->add('replies < ?', array($this->repliesLessThan));
		
		// username
		if (!empty($this->createdBy)) {
			$usernames = preg_split('/\s*,\s*/', $this->createdBy, -1, PREG_SPLIT_NO_EMPTY);
			$this->threadList->getConditionBuilder()->add('username IN (?)', array($usernames));
		}
		
		// board ids
		if (!empty($this->boardIDs)) {
			$this->threadList->getConditionBuilder()->add('boardID IN (?)', array($this->boardIDs));
		}
		
		// language ids
		if (!empty($this->languageIDs)) {
			$this->threadList->getConditionBuilder()->add("languageID IN (?)", array($this->languageIDs));
		}
		
		if ($this->deleted || $this->action == 'restore') $this->threadList->getConditionBuilder()->add("isDeleted = ?", array(1));
		if ($this->notDeleted || $this->action == 'trash') $this->threadList->getConditionBuilder()->add("isDeleted = ?", array(0));
		if ($this->disabled || $this->action == 'enable') $this->threadList->getConditionBuilder()->add("isDisabled = ?", array(1));
		if ($this->notDisabled || $this->action == 'disable') $this->threadList->getConditionBuilder()->add("isDisabled = ?", array(0));
		if ($this->closed || $this->action == 'open') $this->threadList->getConditionBuilder()->add("isClosed = ?", array(1));
		if ($this->open || $this->action == 'close') $this->threadList->getConditionBuilder()->add("isClosed = ?", array(0));
		if ($this->redirect || $this->action == 'removeLink') $this->threadList->getConditionBuilder()->add("movedThreadID IS NOT NULL");
		if ($this->notRedirect) $this->threadList->getConditionBuilder()->add("movedThreadID IS NULL");
		if ($this->announcement) $this->threadList->getConditionBuilder()->add("isAnnouncement = ?", array(1));
		if ($this->sticky) $this->threadList->getConditionBuilder()->add("isSticky = ?", array(1));
		if ($this->normal) $this->threadList->getConditionBuilder()->add("isAnnouncement = ? AND isSticky = ?", array(0, 0));
		if ($this->done) $this->threadList->getConditionBuilder()->add("isDone = ?", array(1));
		if ($this->undone || $this->action == 'markAsDone') $this->threadList->getConditionBuilder()->add("isDone = ?", array(0));
		
		// execute action
		$this->threadList->readObjects();
		
		$parameters = array('isBulkProcessing' => true);
		if ($this->action == 'move') {
			$parameters['boardID'] = $this->moveTo;
			$parameters['showMoveNotice'] = false;
		}
		
		$this->affectedThreads = count($this->threadList);
		
		if ($this->affectedThreads) {
			$threadAction = new ThreadAction($this->threadList->getObjects(), $this->action, $parameters);
			$threadAction->executeAction();
		}
		
		$this->saved();
		
		WCF::getTPL()->assign('affectedThreads', $this->affectedThreads);
	}
	
	/**
	 * @see	wcf\page\IPage::assignVariables()
	 */
	public function assignVariables() {
		parent::assignVariables();
		
		// languages
		$languages = array();
		if (LanguageFactory::getInstance()->multilingualismEnabled()) {
			$languages = LanguageFactory::getInstance()->getContentLanguages();
		}
		
		// board node list
		$boardNodeList = new BoardNodeList();
		$boardNodeList->readNodeTree();
		
		WCF::getTPL()->assign(array(
			'timeAfter' => $this->timeAfter,
			'timeBefore' => $this->timeBefore,
			'lastPostTimeAfter' => $this->lastPostTimeAfter,
			'lastPostTimeBefore' => $this->lastPostTimeBefore,
			'repliesMoreThan' => $this->repliesMoreThan,
			'repliesLessThan' => $this->repliesLessThan,
			'createdBy' => $this->createdBy,
			'deleted' => $this->deleted,
			'notDeleted' => $this->notDeleted,
			'disabled' => $this->disabled,
			'notDisabled' => $this->notDisabled,
			'closed' => $this->closed,
			'open' => $this->open,
			'redirect' => $this->redirect,
			'notRedirect' => $this->notRedirect,
			'announcement' => $this->announcement,
			'sticky' => $this->sticky,
			'normal' => $this->normal,
			'done' => $this->done,
			'undone' => $this->undone,
			'boardIDs' => $this->boardIDs,
			'moveTo' => $this->moveTo,
			'languages' => $languages,
			'languageIDs' => $this->languageIDs,
			'boardNodeList' => $boardNodeList->getNodeList()
		));
	}
}
