<?php
namespace wbb\data\board;
use wcf\data\object\type\ObjectTypeCache;
use wcf\data\package\PackageCache;
use wcf\data\AbstractDatabaseObjectAction;
use wcf\data\ILoadableContainerAction;
use wcf\data\ISortableAction;
use wcf\data\IToggleContainerAction;
use wcf\system\database\util\PreparedStatementConditionBuilder;
use wcf\system\exception\PermissionDeniedException;
use wcf\system\exception\UserInputException;
use wcf\system\user\activity\point\UserActivityPointHandler;
use wcf\system\user\collapsible\content\UserCollapsibleContentHandler;
use wcf\system\user\notification\UserNotificationHandler;
use wcf\system\user\storage\UserStorageHandler;
use wcf\system\visitTracker\VisitTracker;
use wcf\system\WCF;

/**
 * Executes board-related actions.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	data.board
 * @category	Burning Board
 */
class BoardAction extends AbstractDatabaseObjectAction implements ILoadableContainerAction, ISortableAction, IToggleContainerAction {
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::$allowGuestAccess
	 */
	protected $allowGuestAccess = array('loadContainer', 'markAsRead', 'markAllAsRead', 'toggleContainer');
	
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::$className
	 */
	protected $className = 'wbb\data\board\BoardEditor';
	
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::$permissionsDelete
	 */
	protected $permissionsDelete = array('admin.board.canDeleteBoard');
	
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::$requireACP
	 */
	protected $requireACP = array('create', 'delete', 'update', 'updatePosition');
	
	/**
	 * board object
	 * @var	wbb\data\board\Board
	 */
	public $board = null;
	
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::create()
	 */
	public function create() {
		// remove position from data
		$position = 0;
		if (isset($this->parameters['data']['position'])) {
			$position = $this->parameters['data']['position'];
			unset($this->parameters['data']['position']);
		}
		// set default values
		if (!isset($this->parameters['data']['time'])) {
			$this->parameters['data']['time'] = TIME_NOW;
		}
		
		// create board
		$board = parent::create();
		$boardEditor = new BoardEditor($board);
		
		// set position
		$boardEditor->setPosition($this->parameters['data']['parentID'], $position);
		
		return $board;
	}
	
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::update()
	 */
	public function update() {
		parent::update();
		
		// update position if required
		if (count($this->objects) == 1 && isset($this->parameters['data']['parentID']) && isset($this->parameters['data']['position'])) {
			if (($this->objects[0]->parentID != $this->parameters['data']['parentID']) || $this->objects[0]->position != $this->parameters['data']['position']) {
				$this->objects[0]->setPosition($this->parameters['data']['parentID'], $this->parameters['data']['position']);
			}
		}
	}
	
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::delete()
	 */
	public function delete() {
		// update board parents and positions
		foreach ($this->objects as $boardEditor) {
			$childBoardIDs = BoardCache::getInstance()->getChildIDs($boardEditor->boardID);
			
			$nextBoardsPositionUpdate = 0;
			if (empty($childBoardIDs)) {
				// no children: reduce the position value of each board
				// with a higher position by one
				$nextBoardsPositionUpdate = -1;
			}
			else {
				// increase the position value of each board with a higher
				// position depending on the number of children
				$nextBoardsPositionUpdate = count($childBoardIDs) - 1;
			}
			
			// if the board has just one child, the position
			// of the other boards at the same level stays the
			// same
			if ($nextBoardsPositionUpdate) {
				$conditionBuilder = new PreparedStatementConditionBuilder();
				if ($boardEditor->parentID === null) {
					$conditionBuilder->add('parentID IS NULL');
				}
				else {
					$conditionBuilder->add('parentID = ?', array($boardEditor->parentID));
				}
				$conditionBuilder->add('position >= ?', array($boardEditor->position));
				
				$sql = "UPDATE	wbb".WCF_N."_board
					SET	position = position + ?
					".$conditionBuilder;
				$statement = WCF::getDB()->prepareStatement($sql);
				$statement->execute(array_merge(array($nextBoardsPositionUpdate), $conditionBuilder->getParameters()));
			}
			
			// update the positions of the child boards
			$position = $boardEditor->position;
			foreach ($childBoardIDs as $childBoardID) {
				$childBoardEditor = new BoardEditor(BoardCache::getInstance()->getBoard($childBoardID));
				$childBoardEditor->update(array(
					'parentID' => $boardEditor->parentID,
					'position' => $position++
				));
			}
		}
		
		$count = parent::delete();
		
		BoardEditor::resetCache();
		UserStorageHandler::getInstance()->clear();
		
		return $count;
	}
	
	/**
	 * Checks the permissions for the given board ids.
	 */
	protected function checkPermissions() {
		// read data
		if (empty($this->objects)) {
			$this->readObjects();
			
			if (empty($this->objects)) {
				throw new UserInputException('objectIDs');
			}
		}
		
		// check permissions
		foreach ($this->objects as $board) {
			if (!$board->getPermission()) {
				throw new PermissionDeniedException();
			}
		}
	}
	
	/**
	 * Validates parameters for collapsible content actions.
	 */
	protected function validateCollapsibleContentAction() {
		$this->readString('containerID');
		$this->readString('newState');
		
		if (count($this->objectIDs) != 1) {
			throw new UserInputException('objectIDs');
		}
		
		$boardID = reset($this->objectIDs);
		$this->board = BoardCache::getInstance()->getBoard($boardID);
		if ($this->board === null) {
			throw new UserInputException('objectIDs');
		}
	}
	
	/**
	 * @see	wcf\data\IToggleContainerAction::validateToggleContainer()
	 */
	public function validateToggleContainer() {
		if (!WCF::getSession()->getPermission('admin.board.canEditBoard')) {
			throw new PermissionDeniedException();
		}
		
		$this->validateCollapsibleContentAction();
		if (!$this->board->isCategory()) {
			throw new UserInputException('objectIDs');
		}
	}
	
	/**
	 * @see	wcf\data\ILoadableContainerAction::validateLoadContainer()
	 */
	public function validateLoadContainer() {
		$this->validateCollapsibleContentAction();
		if (!$this->board->isCategory() || !$this->board->getPermission()) {
			throw new UserInputException('objectIDs');
		}
	}
	
	/**
	 * @see	wcf\data\IToggleContainerAction::toggleContainer()
	 */
	public function toggleContainer() {
		$objectTypeID = UserCollapsibleContentHandler::getInstance()->getObjectTypeID('com.woltlab.wbb.acpBoard');
		
		if ($this->parameters['newState'] == 'open') {
			UserCollapsibleContentHandler::getInstance()->markAsOpened($objectTypeID, $this->board->boardID);
		}
		else {
			UserCollapsibleContentHandler::getInstance()->markAsCollapsed($objectTypeID, $this->board->boardID);
		}
	}
	
	/**
	 * @see	wcf\data\ILoadableContainerAction::loadContainer()
	 */
	public function loadContainer() {
		$objectTypeID = UserCollapsibleContentHandler::getInstance()->getObjectTypeID('com.woltlab.wbb.board');
		
		if ($this->parameters['newState'] == 'open') {
			UserCollapsibleContentHandler::getInstance()->markAsOpened($objectTypeID, $this->board->boardID);
		}
		else {
			UserCollapsibleContentHandler::getInstance()->markAsCollapsed($objectTypeID, $this->board->boardID);
		}
		
		$startDepth = (!empty($this->parameters['depth']) ? intval($this->parameters['depth']) - 1 : 0);
		$nodeList = new DetailedBoardNodeList($this->board->parentID, $startDepth, array($this->board->boardID));
		$nodeList->readNodeTree();
		
		WCF::getTPL()->assign(array(
			'boardNodeList' => $nodeList->getNodeList(),
			'startDepth' => $startDepth
		));
		
		return array(
			'containerID' => $this->parameters['containerID'],
			'content' => WCF::getTPL()->fetch('boardNodeList', 'wbb'),
			'isOpen' => ($this->parameters['newState'] == 'open' ? 1 : 0)
		);
	}
	
	/**
	 * Marks a board as read.
	 */
	public function markAsRead() {
		$boardIDs = array();
		foreach ($this->objectIDs as $boardID) {
			$boardIDs[] = $boardID;
			VisitTracker::getInstance()->trackObjectVisit('com.woltlab.wbb.board', $boardID);
		}
		
		// reset storage
		if (WCF::getUser()->userID) {
			UserStorageHandler::getInstance()->reset(array(WCF::getUser()->userID), 'wbbUnreadThreads');
			UserStorageHandler::getInstance()->reset(array(WCF::getUser()->userID), 'wbbUnreadWatchedThreads');
			
			// delete obsolete notifications
			if (!empty($boardIDs)) {
				$parameters = array(ObjectTypeCache::getInstance()->getObjectTypeByName('com.woltlab.wcf.user.objectWatch', 'com.woltlab.wbb.thread')->objectTypeID, WCF::getUser()->userID);
				$parameters = array_merge($parameters, $boardIDs);
				$parameters[] = PackageCache::getInstance()->getPackageID('com.woltlab.wbb');
				$parameters[] = UserNotificationHandler::getInstance()->getEvent('com.woltlab.wbb.post', 'post')->eventID;
				$parameters[] = WCF::getUser()->userID;
				$parameters[] = TIME_NOW;
				
				$sql = "SELECT		post.postID
					FROM		wbb".WCF_N."_post post,
							wcf".WCF_N."_user_notification notification,
							wcf".WCF_N."_user_notification_to_user notification_to_user,
							(
								SELECT	threadID
								FROM	wcf".WCF_N."_user_object_watch object_watch,
									wbb".WCF_N."_thread thread
								WHERE	thread.threadID = object_watch.objectID
									AND object_watch.objectTypeID = ?
									AND object_watch.userID = ?
									AND thread.boardID IN (?".(count($boardIDs) > 1 ? str_repeat(',?', count($boardIDs) - 1) : '').")
							) AS thread
					WHERE		post.threadID = thread.threadID
							AND notification.packageID = ?
							AND notification.eventID = ?
							AND notification.objectID = post.postID
							AND notification_to_user.notificationID = notification.notificationID
							AND notification_to_user.userID = ?
							AND post.time <= ?";
				$statement = WCF::getDB()->prepareStatement($sql);
				$statement->execute($parameters);
				$postIDs = array();
				while ($row = $statement->fetchArray()) {
					$postIDs[] = $row['postID'];
				}
				
				if (!empty($postIDs)) {
					UserNotificationHandler::getInstance()->deleteNotifications('post', 'com.woltlab.wbb.post', array(WCF::getUser()->userID), $postIDs);
				}
			}
		}
	}
	
	/**
	 * Validates the mark as read action.
	 */
	public function validateMarkAsRead() {
		$this->checkPermissions();
	}
	
	/**
	 * Marks all boards as read.
	 */
	public function markAllAsRead() {
		VisitTracker::getInstance()->trackTypeVisit('com.woltlab.wbb.thread');
		VisitTracker::getInstance()->deleteObjectVisits('com.woltlab.wbb.board');
		
		// reset storage and notifications
		if (WCF::getUser()->userID) {
			UserStorageHandler::getInstance()->reset(array(WCF::getUser()->userID), 'wbbUnreadThreads');
			UserStorageHandler::getInstance()->reset(array(WCF::getUser()->userID), 'wbbUnreadWatchedThreads');
			UserNotificationHandler::getInstance()->deleteNotifications('post', 'com.woltlab.wbb.post', array(WCF::getUser()->userID));
		}
	}
	
	/**
	 * Validates the mark all as read action.
	 */
	public function validateMarkAllAsRead() {
		// does nothing
	}
	
	/**
	 * Loads the list of ignored boards.
	 */
	public function loadIgnoredBoards() {
		$boardNodeList = new RestrictedBoardNodeList();
		$boardNodeList->readNodeTree();
		
		WCF::getTPL()->assign(array(
			'boardNodeList' => $boardNodeList->getNodeList()
		));
		return array(
			'template' => WCF::getTPL()->fetch('ignoredBoards', 'wbb')
		);
	}
	
	/**
	 * Validates the load ignored boards action.
	 */
	public function validateLoadIgnoredBoards() {
		// does nothing
	}
	
	/**
	 * Saves the ignored boards.
	 */
	public function saveIgnoredBoards() {
		$objectTypeID = UserCollapsibleContentHandler::getInstance()->getObjectTypeID('com.woltlab.wbb.ignoredBoard');
		
		// delete old values
		UserCollapsibleContentHandler::getInstance()->reset($objectTypeID);
		
		// save board ids
		foreach ($this->objectIDs as $boardID) {
			UserCollapsibleContentHandler::getInstance()->markAsCollapsed($objectTypeID, $boardID);
		}
		
		// reset user storage
		UserStorageHandler::getInstance()->reset(array(WCF::getUser()->userID), 'wbbUnreadThreads');
		
		// return new board list
		$boardNodeList = new DetailedBoardNodeList();
		$boardNodeList->readNodeTree();
		
		WCF::getTPL()->assign(array(
			'boardNodeList' => $boardNodeList->getNodeList()
		));
		return array(
			'template' => WCF::getTPL()->fetch('boardNodeList', 'wbb')
		);
	}
	
	/**
	 * Validates the save ignored boards action.
	 */
	public function validateSaveIgnoredBoards() {
		if (count($this->objectIDs)) {
			$this->checkPermissions();
		}
		
		foreach ($this->objects as $board) {
			if (!$board->ignorable) throw new UserInputException('objectIDs');
		}
	}
	
	/**
	 * @see	wcf\data\ISortableAction::validateUpdatePosition()
	 */
	public function validateUpdatePosition() {
		if (!WCF::getSession()->getPermission('admin.board.canEditBoard')) {
			throw new PermissionDeniedException();
		}
		
		if (!isset($this->parameters['data']['structure'])) {
			throw new UserInputException('structure');
		}
		
		// collect board ids
		$boardIDs = array();
		foreach ($this->parameters['data']['structure'] as $parentID => $childBoardIDs) {
			if ($parentID) {
				$boardIDs[] = $parentID;
			}
			
			foreach ($childBoardIDs as $boardID) {
				if (!$boardID) {
					throw new UserInputException('structure');
				}
				
				$boardIDs[] = $boardID;
			}
		}
		
		// validate board ids against database
		$boardIDs = array_unique($boardIDs);
		$conditions = new PreparedStatementConditionBuilder();
		$conditions->add("boardID IN (?)", array($boardIDs));
		$sql = "SELECT	boardID
			FROM	wbb".WCF_N."_board
			".$conditions;
		$statement = WCF::getDB()->prepareStatement($sql);
		$statement->execute($conditions->getParameters());
		while ($row = $statement->fetchArray()) {
			$index = array_search($row['boardID'], $boardIDs);
			if ($index !== false) {
				unset($boardIDs[$index]);
			}
		}
		
		if (!empty($boardIDs)) {
			throw new UserInputException('structure');
		}
	}
	
	/**
	 * @see	wcf\data\ISortableAction::updatePosition()
	 */
	public function updatePosition() {
		// handle top boards first
		$sql = "UPDATE	wbb".WCF_N."_board
			SET	parentID = NULL,
				position = ?
			WHERE	boardID = ?";
		$statement = WCF::getDB()->prepareStatement($sql);
		WCF::getDB()->beginTransaction();
		foreach ($this->parameters['data']['structure'][0] as $position => $boardID) {
			$statement->execute(array(
				($position + 1),
				$boardID
			));
		}
		WCF::getDB()->commitTransaction();
		
		// handle non-top boards
		if (count($this->parameters['data']['structure']) > 1) {
			$sql = "UPDATE	wbb".WCF_N."_board
				SET	parentID = ?,
					position = ?
				WHERE	boardID = ?";
			$statement = WCF::getDB()->prepareStatement($sql);
			WCF::getDB()->beginTransaction();
			foreach ($this->parameters['data']['structure'] as $parentID => $boardIDs) {
				if ($parentID == 0) {
					continue;
				}
				
				foreach ($boardIDs as $position => $boardID) {
					$statement->execute(array(
						$parentID,
						($position + 1),
						$boardID
					));
				}
			}
			WCF::getDB()->commitTransaction();
		}
	}
}
