<?php
namespace wbb\data\post;
use wbb\data\board\BoardCache;
use wbb\data\thread\Thread;
use wbb\data\WBBDatabaseObject;
use wcf\data\attachment\GroupedAttachmentList;
use wcf\data\poll\Poll;
use wcf\data\IMessage;
use wcf\data\IPollObject;
use wcf\system\bbcode\AttachmentBBCode;
use wcf\system\bbcode\MessageParser;
use wcf\system\database\util\PreparedStatementConditionBuilder;
use wcf\system\exception\NamedUserException;
use wcf\system\request\LinkHandler;
use wcf\system\WCF;
use wcf\util\StringUtil;
use wcf\util\UserUtil;

/**
 * Represents a post.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	data.post
 * @category	Burning Board
 */
class Post extends WBBDatabaseObject implements IMessage, IPollObject {
	/**
	 * @see	wcf\data\DatabaseObject::$databaseTableName
	 */
	protected static $databaseTableName = 'post';
	
	/**
	 * @see	wcf\data\DatabaseObject::$databaseTableIndexName
	 */
	protected static $databaseTableIndexName = 'postID';
	
	/**
	 * poll object
	 * @var	wcf\data\poll\Poll
	 */
	protected $poll = null;
	
	/**
	 * thread object
	 * @var	wbb\data\thread\Thread
	 */
	protected $thread = null;
	
	/**
	 * @see	wcf\data\IMessage::getMessage()
	 */
	public function getMessage() {
		return $this->message;
	}
	
	/**
	 * @see	wcf\data\IMessage::getFormattedMessage()
	 */
	public function getFormattedMessage() {
		// assign embedded attachments
		AttachmentBBCode::setObjectID($this->postID);
		
		// parse and return message
		MessageParser::getInstance()->setOutputType('text/html');
		return MessageParser::getInstance()->parse($this->message, $this->enableSmilies, $this->enableHtml, $this->enableBBCodes);
	}
	
	/**
	 * Returns a simplified version of the formatted message.
	 * 
	 * @return	string
	 */
	public function getSimplifiedFormattedMessage() {
		MessageParser::getInstance()->setOutputType('text/simplified-html');
		return MessageParser::getInstance()->parse($this->message, $this->enableSmilies, $this->enableHtml, $this->enableBBCodes);
	}
	
	/**
	 * Returns and assigns embedded attachments.
	 * 
	 * @return	wcf\data\attachment\GroupedAttachmentList
	 */
	public function getAttachments() {
		if (MODULE_ATTACHMENT == 1 && $this->attachments) {
			$attachmentList = new GroupedAttachmentList('com.woltlab.wbb.post');
			$attachmentList->getConditionBuilder()->add('attachment.objectID IN (?)', array($this->postID));
			$attachmentList->readObjects();
			
			// set embedded attachments
			AttachmentBBCode::setAttachmentList($attachmentList);
			
			return $attachmentList;
		}
		
		return null;
	}
	
	/**
	 * Returns an excerpt of this post.
	 * 
	 * @param	string		$maxLength
	 * @return	string
	 */
	public function getExcerpt($maxLength = 255) {
		return StringUtil::truncateHTML($this->getSimplifiedFormattedMessage(), $maxLength);
	}
	
	/**
	 * Returns true if the active user has the permission to read this post.
	 * 
	 * @return boolean
	 */
	public function canRead() {
		$board = BoardCache::getInstance()->getBoard($this->getThread()->boardID);
		
		if (!$board->canEnter()) {
			return false;
		}
		
		if (!$board->getPermission('canReadThread') || ($this->isDeleted && !$board->getModeratorPermission('canReadDeletedPost')) || ($this->isDisabled && !$board->getModeratorPermission('canEnablePost'))) {
			return false;
		}
		
		return true;
	}
	
	/**
	 * Returns true if active user has the permission to delete this post.
	 * 
	 * @return	boolean
	 */
	public function canDelete() {
		// get thread
		$thread = $this->getThread();
		
		// get board
		$board = BoardCache::getInstance()->getBoard($thread->boardID);
		
		// check permissions
		$isModerator = $board->getModeratorPermission('canDeletePost');
		$isAuthor = $this->userID && $this->userID == WCF::getUser()->userID;
		$canDeletePost = $isModerator || ($isAuthor && $board->getPermission('canDeleteOwnPost'));
		
		if (!$canDeletePost || (!$isModerator && ($board->isClosed || $thread->isClosed || $this->isClosed))) {
			return false;
		}
		
		return true;
	}
	
	/**
	 * Returns true if active user has the permission to completely delete this post.
	 * 
	 * @return	boolean
	 */
	public function canDeleteCompletely() {
		// get thread
		$thread = $this->getThread();
		
		// get board
		$board = BoardCache::getInstance()->getBoard($thread->boardID);
		
		// check permissions
		if (!$board->getModeratorPermission('canDeletePostCompletely')) {
			return false;
		}
		
		return true;
	}
	
	/**
	 * Returns true if active user has the permission to restore this post.
	 * 
	 * @return	boolean
	 */
	public function canRestore() {
		// get thread
		$thread = $this->getThread();
		
		// get board
		$board = BoardCache::getInstance()->getBoard($thread->boardID);
		
		// check permissions
		if (!$board->getModeratorPermission('canRestorePost')) {
			return false;
		}
		
		return true;
	}
	
	/**
	 * Returns true if active user has the permission to enable this post.
	 * 
	 * @return	boolean
	 */
	public function canEnable() {
		// get thread
		$thread = $this->getThread();
		
		// get board
		$board = BoardCache::getInstance()->getBoard($thread->boardID);
		
		// check permissions
		if (!$board->getModeratorPermission('canEnableThread')) {
			return false;
		}
		
		return true;
	}
	
	/**
	 * @see	wcf\data\IMessage::isVisible()
	 */
	public function isVisible() {
		return $this->canRead();
	}
	
	/**
	 * Returns the thread of this post.
	 * 
	 * @return	wbb\data\thread\Thread
	 */
	public function getThread() {
		if ($this->thread === null) {
			$this->thread = new Thread($this->threadID);
		}
		
		return $this->thread;
	}
	
	/**
	 * Sets thread object for this post.
	 * 
	 * @param	wbb\data\thread\Thread	$thread
	 */
	public function setThread(Thread $thread) {
		if ($thread->threadID == $this->threadID) {
			$this->thread = $thread;
		}
	}
	
	/**
	 * Returns true if this post is the first post in its thread.
	 * 
	 * @return	boolean
	 */
	public function isFirstPost() {
		return ($this->getThread()->firstPostID == $this->postID);
	}
	
	/**
	 * Returns a list of ip addresses used by a user.
	 * 
	 * @param	integer		$userID
	 * @param	string		$username
	 * @param	string		$notIpAddress
	 * @return	array
	 */
	public static function getIpAddressByAuthor($userID, $username = '', $notIpAddress = '', $limit = 10) {
		$conditions = new PreparedStatementConditionBuilder();
		$conditions->add("userID = ?", array($userID));
		if (!empty($username) && !$userID) $conditions->add("username = ?", array($username));
		if (!empty($notIpAddress)) $conditions->add("ipAddress <> ?", array($notIpAddress));
		$conditions->add("ipAddress <> ''");
		
		$sql = "SELECT		DISTINCT ipAddress
			FROM		wbb".WCF_N."_post
			".$conditions."
			ORDER BY	time DESC";
		$statement = WCF::getDB()->prepareStatement($sql, $limit);
		$statement->execute($conditions->getParameters());
		
		$ipAddresses = array();
		while ($row = $statement->fetchArray()) {
			$ipAddresses[] = $row['ipAddress'];
		}
		
		return $ipAddresses;
	}
	
	/**
	 * Returns a list of users which have used the given ip address.
	 * 
	 * @param	string		$ipAddress
	 * @param	integer		$notUserID
	 * @param	string		$notUsername
	 * @return	array<array>
	 */
	public static function getAuthorByIpAddress($ipAddress, $notUserID = 0, $notUsername = '', $limit = 10) {
		$conditions = new PreparedStatementConditionBuilder();
		$conditions->add("ipAddress = ?", array($ipAddress));
		if ($notUserID) $conditions->add("userID <> ?", array($notUserID));
		if (!empty($notUsername)) $conditions->add("username <> ?", array($notUsername));
		
		$sql = "SELECT		DISTINCT username, userID
			FROM		wbb".WCF_N."_post
			".$conditions."
			ORDER BY	time DESC";
		$statement = WCF::getDB()->prepareStatement($sql, $limit);
		$statement->execute($conditions->getParameters());
		
		$users = array();
		while ($row = $statement->fetchArray()) {
			$users[] = $row;
		}
		
		return $users;
	}
	
	/**
	 * Returns post's ip address, converts into IPv4 if applicable.
	 * 
	 * @return	string
	 */
	public function getIpAddress() {
		if ($this->ipAddress) {
			return UserUtil::convertIPv6To4($this->ipAddress);
		}
		
		return '';
	}
	
	/**
	 * @see	wcf\data\IMessage::getUserID()
	 */
	public function getUserID() {
		return $this->userID;
	}
	
	/**
	 * @see	wcf\data\IMessage::getUsername()
	 */
	public function getUsername() {
		return $this->username;
	}
	
	/**
	 * @see	wcf\data\IMessage::getTime()
	 */
	public function getTime() {
		return $this->time;
	}
	
	/**
	 * @see	wcf\data\ILinkableObject::getLink()
	 */
	public function getLink() {
		return LinkHandler::getInstance()->getLink('Thread', array(
			'application' => 'wbb',
			'object' => $this->getThread(),
			'postID' => $this->postID,
			'forceFrontend' => true
		), '#post'.$this->postID);
	}
	
	/**
	 * @see	wcf\data\ITitledObject::getTitle()
	 */
	public function getTitle() {
		if ($this->subject) {
			return $this->subject;
		}
		
		return 'RE: '.$this->getThread()->topic;
	}
	
	/**
	 * @see	wcf\data\IMessage::__toString()
	 */
	public function __toString() {
		return $this->getFormattedMessage();
	}
	
	/**
	 * Returns the poll object for this post.
	 * 
	 * @return	wcf\data\poll\Poll
	 */
	public function getPoll() {
		if ($this->pollID && $this->poll === null) {
			$this->poll = new Poll($this->pollID);
			$this->poll->setRelatedObject($this);
		}
		
		return $this->poll;
	}
	
	/**
	 * Sets the poll object for this post.
	 * 
	 * @param	Poll	$poll
	 */
	public function setPoll(Poll $poll) {
		$this->poll = $poll;
		
		// assign post as related object
		$this->poll->setRelatedObject($this);
	}
	
	/**
	 * @see	wcf\data\IPollObject::canVote()
	 */
	public function canVote() {
		if (WCF::getSession()->getPermission('user.board.canVotePoll')) {
			return true;
		}
		else if ($this->getThread()->getBoard()->getPermission('canVotePoll')) {
			return true;
		}
		
		return false;
	}
	
	/**
	 * Enforces the flood control.
	 */
	public static function enforceFloodControl() {
		if (!WCF::getSession()->getPermission('user.board.floodControlTime')) {
			return;
		}
		
		$sql = "SELECT		time
			FROM		wbb".WCF_N."_post
			WHERE		".(WCF::getUser()->userID ? "userID = ?" : "ipAddress = ?")."
					AND time > ?
			ORDER BY	time DESC";
		$statement = WCF::getDB()->prepareStatement($sql, 1);
		$statement->execute(array(
			(WCF::getUser()->userID ?: WCF::getSession()->ipAddress),
			(TIME_NOW - WCF::getSession()->getPermission('user.board.floodControlTime'))
		));
		if (($row = $statement->fetchArray()) !== false) {
			throw new NamedUserException(WCF::getLanguage()->getDynamicVariable('wbb.post.error.floodControl', array('lastPostTime' => $row['time'])));
		}
	}
}
