<?php
namespace wbb\data\post;
use wbb\data\board\Board;
use wbb\data\board\BoardCache;
use wbb\data\board\BoardEditor;
use wbb\data\thread\Thread;
use wbb\data\thread\ThreadAction;
use wbb\data\thread\ThreadEditor;
use wbb\data\thread\ThreadList;
use wbb\data\thread\ViewableThread;
use wbb\system\log\modification\PostModificationLogHandler;
use wbb\system\log\modification\ThreadModificationLogHandler;
use wbb\system\user\notification\object\PostUserNotificationObject;
use wcf\data\smiley\SmileyCache;
use wcf\data\user\object\watch\UserObjectWatchAction;
use wcf\data\user\UserProfileList;
use wcf\data\AbstractDatabaseObjectAction;
use wcf\data\DatabaseObject;
use wcf\data\IClipboardAction;
use wcf\data\IExtendedMessageQuickReplyAction;
use wcf\data\IMessageInlineEditorAction;
use wcf\data\IMessageQuoteAction;
use wcf\system\attachment\AttachmentHandler;
use wcf\system\bbcode\BBCodeHandler;
use wcf\system\bbcode\BBCodeParser;
use wcf\system\bbcode\PreParser;
use wcf\system\clipboard\ClipboardHandler;
use wcf\system\database\util\PreparedStatementConditionBuilder;
use wcf\system\exception\PermissionDeniedException;
use wcf\system\exception\UserInputException;
use wcf\system\like\LikeHandler;
use wcf\system\message\censorship\Censorship;
use wcf\system\message\quote\MessageQuoteManager;
use wcf\system\message\QuickReplyManager;
use wcf\system\moderation\queue\ModerationQueueActivationManager;
use wcf\system\poll\PollManager;
use wcf\system\request\LinkHandler;
use wcf\system\search\SearchIndexManager;
use wcf\system\user\activity\event\UserActivityEventHandler;
use wcf\system\user\activity\point\UserActivityPointHandler;
use wcf\system\user\notification\UserNotificationHandler;
use wcf\system\user\object\watch\UserObjectWatchHandler;
use wcf\system\user\storage\UserStorageHandler;
use wcf\system\WCF;
use wcf\util\MessageUtil;
use wcf\util\StringUtil;
use wcf\util\UserUtil;

/**
 * Executes post-related actions.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	data.post
 * @category	Burning Board
 */
class PostAction extends AbstractDatabaseObjectAction implements IClipboardAction, IExtendedMessageQuickReplyAction, IMessageInlineEditorAction, IMessageQuoteAction {
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::$className
	 */
	protected $className = 'wbb\data\post\PostEditor';
	
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::$allowGuestAccess
	 */
	protected $allowGuestAccess = array('getPostPreview', 'jumpToExtended', 'quoteMessage', 'saveFullQuote', 'saveQuote');
	
	/**
	 * current board object
	 * @var	wbb\data\board\Board
	 */
	public $board = null;
	
	/**
	 * current post object
	 * @var	wbb\data\post\Post
	 */
	public $post = null;
	
	/**
	 * list of post data
	 * @var	array<array>
	 */
	public $postData = array();
	
	/**
	 * current post editor object
	 * @var	wbb\data\post\PostEditor
	 */
	public $postEditor = null;
	
	/**
	 * list with loaded posts
	 * @var	wbb\data\post\ViewablePostList
	 */
	public $postList = null;
	
	/**
	 * current thread object
	 * @var	wbb\data\thread\Thread
	 */
	public $thread = null;
	
	/**
	 * list of thread data
	 * @var	array<array>
	 */
	public $threadData = array();
	
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::create()
	 */
	public function create() {
		// count attachments
		if (isset($this->parameters['attachmentHandler']) && $this->parameters['attachmentHandler'] !== null) {
			$this->parameters['data']['attachments'] = count($this->parameters['attachmentHandler']);
		}
		
		if (LOG_IP_ADDRESS) {
			// add ip address
			if (!isset($this->parameters['data']['ipAddress'])) {
				$this->parameters['data']['ipAddress'] = WCF::getSession()->ipAddress;
			}
		}
		else {
			// do not track ip address
			if (isset($this->parameters['data']['ipAddress'])) {
				unset($this->parameters['data']['ipAddress']);
			}
		}
		
		// get thread
		$thread = (isset($this->parameters['thread']) ? $this->parameters['thread'] : new Thread($this->parameters['data']['threadID']));
		if ($thread->isDisabled) $this->parameters['data']['isDisabled'] = 1;
		if ($thread->isDeleted) $this->parameters['data']['isDeleted'] = 1;
		
		// create post
		$post = parent::create();
		$postEditor = new PostEditor($post);
		$post->setThread($thread);
		
		// update search index
		SearchIndexManager::getInstance()->add('com.woltlab.wbb.post', $post->postID, $post->message, $post->subject, $post->time, $post->userID, $post->username, $thread->languageID);
		
		// update attachments
		if (isset($this->parameters['attachmentHandler']) && $this->parameters['attachmentHandler'] !== null) {
			$this->parameters['attachmentHandler']->updateObjectID($post->postID);
		}
		
		// clear quotes
		if (isset($this->parameters['removeQuoteIDs']) && !empty($this->parameters['removeQuoteIDs'])) {
			MessageQuoteManager::getInstance()->markQuotesForRemoval($this->parameters['removeQuoteIDs']);
		}
		MessageQuoteManager::getInstance()->removeMarkedQuotes();
		
		// trigger publication
		if (!$post->isDisabled) {
			$action = new PostAction(array($postEditor), 'triggerPublication', array(
				'isFirstPost' => !empty($this->parameters['isFirstPost'])
			));
			$action->executeAction();
		}
		// mark post for moderated content
		else {
			ModerationQueueActivationManager::getInstance()->addModeratedContent('com.woltlab.wbb.post', $post->postID);
		}
		
		// handle thread subscription
		if (WCF::getUser()->userID && $post->userID == WCF::getUser()->userID) {
			if (!isset($this->parameters['subscribeThread']) &&  WCF::getUser()->watchThreadOnReply) {
				$this->parameters['subscribeThread'] = 1;
			}
			
			if (isset($this->parameters['subscribeThread'])) {
				if ($this->parameters['subscribeThread'] && !$thread->isSubscribed()) {
					$action = new UserObjectWatchAction(array(), 'subscribe', array(
						'data' => array(
							'objectID' => $thread->threadID,
							'objectType' => 'com.woltlab.wbb.thread'
						),
						'enableNotification' => (UserNotificationHandler::getInstance()->getEventSetting('com.woltlab.wbb.post', 'post') !== false ? 1 : 0)
					));
					$action->executeAction();
				}
				else if (!$this->parameters['subscribeThread'] && $thread->isSubscribed()) {
					$action = new UserObjectWatchAction(array(), 'unsubscribe', array(
						'data' => array(
							'objectID' => $thread->threadID,
							'objectType' => 'com.woltlab.wbb.thread'
						)
					));
					$action->executeAction();
				}
			}
		}
		
		// return new post
		return $post;
	}
	
	/**
	 * Triggers the publication of posts.
	 */
	public function triggerPublication() {
		if (empty($this->objects)) {
			$this->readObjects();
		}
		
		$isBulkProcessing = (isset($this->parameters['isBulkProcessing']) && $this->parameters['isBulkProcessing']);
		
		foreach ($this->objects as $post) {
			$threadEditor = new ThreadEditor($post->getThread());
			$boardEditor = new BoardEditor($threadEditor->getBoard());
			
			if (!$post->isFirstPost() && empty($this->parameters['isFirstPost'])) {
				if (!$isBulkProcessing) {
					// update last post
					$threadEditor->addPost($post->getDecoratedObject());
					
					// update board counters
					if (!$post->isDeleted) {
						$boardEditor->updateCounters(array(
							'posts' => 1
						));
					}
				}
				
				// fire activity event
				if ($post->userID) {
					UserActivityEventHandler::getInstance()->fireEvent('com.woltlab.wbb.recentActivityEvent.post', $post->postID, $threadEditor->languageID, $post->userID, $post->time);
					if (!$isBulkProcessing && $boardEditor->countUserPosts) UserActivityPointHandler::getInstance()->fireEvent('com.woltlab.wbb.activityPointEvent.post', $post->postID, $post->userID);
				}
				
				// update watched objects
				UserObjectWatchHandler::getInstance()->updateObject('com.woltlab.wbb.thread', $threadEditor->threadID, 'post', 'com.woltlab.wbb.post', new PostUserNotificationObject($post->getDecoratedObject()));
			}
			
			if (!$isBulkProcessing) {
				// update board last post
				$boardEditor->setLastPost($threadEditor->getDecoratedObject());
				
				// update user post counter
				if ($boardEditor->countUserPosts && $post->userID) {
					PostEditor::updatePostCounter(array($post->userID => 1));
				}
			}
			
			// check for quotes
			$usernames = MessageUtil::getQuotedUsers($post->message);
			if (!empty($usernames)) {
				// get user profiles
				$userList = new UserProfileList();
				$userList->getConditionBuilder()->add('user_table.username IN (?)', array($usernames));
				if ($post->userID) {
					// ignore self-quoting
					$userList->getConditionBuilder()->add('user_table.userID <> ?', array($post->userID));
				}
				$userList->readObjects();
				$recipientIDs = array();
				foreach ($userList as $user) {
					if ($boardEditor->checkUserPermissions($user, array('canViewBoard', 'canEnterBoard', 'canReadThread'))) {
						$recipientIDs[] = $user->userID;
					}
				}
				
				// fire event
				if (!empty($recipientIDs)) {
					UserNotificationHandler::getInstance()->fireEvent('quote', 'com.woltlab.wbb.post', new PostUserNotificationObject($post->getDecoratedObject()), $recipientIDs);
				}
			}
			
			// check for mentions
			$usernames = MessageUtil::getMentionedUsers($post->message);
			if (!empty($usernames)) {
				// get user profiles
				$userList = new UserProfileList();
				$userList->getConditionBuilder()->add('user_table.username IN (?)', array($usernames));
				if ($post->userID) {
					$userList->getConditionBuilder()->add('user_table.userID <> ?', array($post->userID));
				}
				$userList->readObjects();
				$recipientIDs = array();
				foreach ($userList as $user) {
					if ($boardEditor->checkUserPermissions($user, array('canViewBoard', 'canEnterBoard', 'canReadThread'))) {
						$recipientIDs[] = $user->userID;
					}
				}
				
				// fire event
				if (!empty($recipientIDs)) {
					UserNotificationHandler::getInstance()->fireEvent('mention', 'com.woltlab.wbb.post', new PostUserNotificationObject($post->getDecoratedObject()), $recipientIDs);
				}
			}
		}
		
		// reset storage
		UserStorageHandler::getInstance()->resetAll('wbbUnreadThreads');
		
		// reset cache
		BoardEditor::resetDataCache();
	}
	
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::update()
	 */
	public function update() {
		// count attachments
		if (isset($this->parameters['attachmentHandler']) && $this->parameters['attachmentHandler'] !== null) {
			$this->parameters['data']['attachments'] = count($this->parameters['attachmentHandler']);
		}
		
		// set delete time
		if (isset($this->parameters['data']['isDeleted'])) {
			$this->parameters['data']['deleteTime'] = ($this->parameters['data']['isDeleted'] ? TIME_NOW : 0);
		}
		
		parent::update();
		
		// handle attributes for logging
		foreach ($this->objects as $post) {
			// soft-delete of post
			if (isset($this->parameters['data']['isDeleted'])) {
				PostModificationLogHandler::getInstance()->trash($post->getDecoratedObject());
			}
			
			// moderated content
			if (isset($this->parameters['data']['isDisabled'])) {
				if ($this->parameters['data']['isDisabled']) {
					ModerationQueueActivationManager::getInstance()->addModeratedContent('com.woltlab.wbb.post', $post->postID);
				}
				else {
					ModerationQueueActivationManager::getInstance()->removeModeratedContent('com.woltlab.wbb.post', array($post->postID));
				}
			}
			
			// edit
			if (isset($this->parameters['isEdit'])) {
				$reason = (isset($this->parameters['data']['editReason'])) ? $this->parameters['data']['editReason'] : '';
				PostModificationLogHandler::getInstance()->edit($post->getDecoratedObject(), $reason);
			}
			
			// update thread attachment counter
			if (isset($this->parameters['data']['attachments']) && $this->parameters['data']['attachments'] != $post->attachments) {
				$threadEditor = new ThreadEditor(new Thread(null, array('threadID' => $post->threadID)));
				$threadEditor->updateCounters(array('attachments' => $this->parameters['data']['attachments'] - $post->attachments));
			}
			
			// handle new mentions
			if (isset($this->parameters['data']['message']) && !$post->isDisabled && !$post->isDeleted) {
				// find old matches
				$oldMatches = array();
				if (preg_match_all("~\[url='[^']+'\]@(.+?)\[/url\]~", $post->message, $matches)) {
					$oldMatches = array_map(array('\wcf\util\StringUtil', 'toLowerCase'), $matches[1]);
				}
				
				// find new matches
				$newMatches = array();
				if (preg_match_all("~\[url='[^']+'\]@(.+?)\[/url\]~", $this->parameters['data']['message'], $matches)) {
					$newMatches = array_map(array('\wcf\util\StringUtil', 'toLowerCase'), $matches[1]);
				}
				$newMatches = array_diff($newMatches, $oldMatches);
				
				if (!empty($newMatches)) {
					$board = BoardCache::getInstance()->getBoard($post->getThread()->boardID);
					
					// get user profiles
					$userList = new UserProfileList();
					$userList->getConditionBuilder()->add('user_table.username IN (?)', array($newMatches));
					if ($post->userID) {
						$userList->getConditionBuilder()->add('user_table.userID <> ?', array($post->userID));
					}
					$userList->readObjects();
					$recipientIDs = array();
					foreach ($userList as $user) {
						if ($board->checkUserPermissions($user, array('canViewBoard', 'canEnterBoard', 'canReadThread'))) {
							$recipientIDs[] = $user->userID;
						}
					}
					
					// fire event
					if (!empty($recipientIDs)) {
						UserNotificationHandler::getInstance()->fireEvent('mention', 'com.woltlab.wbb.post', new PostUserNotificationObject($post->getDecoratedObject()), $recipientIDs);
					}
				}
			}
		}
		
		if (isset($this->parameters['data']) && (isset($this->parameters['data']['subject']) || isset($this->parameters['data']['message']))) {
			// get ids
			$objectIDs = array();
			$threadIDs = array();
			foreach ($this->objects as $object) {
				$objectIDs[] = $object->postID;
				$threadIDs[] = $object->threadID;
			}
			
			// delete old entries
			if (!empty($objectIDs)) {
				SearchIndexManager::getInstance()->delete('com.woltlab.wbb.post', $objectIDs);
			}
			
			// get language ids
			$conditions = new PreparedStatementConditionBuilder();
			$conditions->add("threadID IN (?)", array($threadIDs));
			$sql = "SELECT	threadID, languageID
				FROM	wbb".WCF_N."_thread
				".$conditions;
			$statement = WCF::getDB()->prepareStatement($sql);
			$statement->execute($conditions->getParameters());
			
			$threads = array();
			while ($row = $statement->fetchArray()) {
				$threads[$row['threadID']] = $row['languageID'];
			}
			
			// save new entries
			foreach ($this->objects as $object) {
				SearchIndexManager::getInstance()->add('com.woltlab.wbb.post', $object->postID, (isset($this->parameters['data']['message']) ? $this->parameters['data']['message'] : $object->message), (isset($this->parameters['data']['subject']) ? $this->parameters['data']['subject'] : $object->subject), $object->time, $object->userID, $object->username, $threads[$object->threadID]);
			}
		}
	}
	
	/**
	 * Validates if user can enable posts.
	 */
	public function validateEnable() {
		$this->prepareObjects();
		
		// validate permissions
		foreach ($this->objects as $post) {
			if (!$post->isDisabled || $post->isDeleted) {
				throw new UserInputException('objectIDs');
			}
			
			if (!$post->canEnable()) {
				throw new PermissionDeniedException();
			}
		}
	}
	
	/**
	 * Enables given posts.
	 */
	public function enable() {
		$enableThreads = $postIDs = array();
		foreach ($this->objects as $post) {
			if (empty($this->parameters['ignoreThreads']) && $post->getThread()->isDisabled) {
				$enableThreads[] = $post->threadID;
			}
			
			$post->update(array('isDisabled' => 0));
			
			$postIDs[] = $post->postID;
			$this->addPostData($post->getDecoratedObject(), 'isDisabled', 0);
			
			PostModificationLogHandler::getInstance()->enable($post->getDecoratedObject());
		}
		
		// update user's posts
		$isBulkProcessing = (isset($this->parameters['isBulkProcessing']) && $this->parameters['isBulkProcessing']);
		$postAction = new PostAction($this->objects, 'triggerPublication', array('isBulkProcessing' => $isBulkProcessing));
		$postAction->executeAction();
		
		$this->removeModeratedContent($postIDs);
		
		if (!empty($enableThreads)) {
			$threadAction = new ThreadAction(array_unique($enableThreads), 'enable', array('ignorePosts' => true));
			$threadAction->executeAction();
			
			foreach ($enableThreads as $threadID) {
				$this->addThreadData($threadID, 'isDisabled', 0);
				$this->addThreadData($threadID, 'ignoreDisabledPosts', 1);
			}
		}
		
		$this->unmarkItems();
		
		return $this->getPostData();
	}
	
	/**
	 * Validates if user can disable posts.
	 */
	public function validateDisable() {
		$this->prepareObjects();
		
		// validate permissions
		foreach ($this->objects as $post) {
			if ($post->isDisabled || $post->isDeleted) {
				throw new UserInputException('objectIDs');
			}
			
			if (!$post->canEnable()) {
				throw new PermissionDeniedException();
			}
		}
	}
	
	/**
	 * Disables given posts.
	 */
	public function disable() {
		$isBulkProcessing = (isset($this->parameters['isBulkProcessing']) && $this->parameters['isBulkProcessing']);
		
		$checkThreads = $postIDs = $boardStats = $boardIDs = $userPosts = $userToItems = array();
		foreach ($this->objects as $post) {
			if (!$isBulkProcessing) {
				$boardIDs[] = $post->getThread()->boardID;
			}
			
			if (empty($this->parameters['ignoreThreads']) && !$post->getThread()->isDisabled) {
				$checkThreads[] = $post->threadID;
			}
			
			$post->update(array('isDisabled' => 1));
			
			$postIDs[] = $post->postID;
			$this->addPostData($post->getDecoratedObject(), 'isDisabled', 1);
			
			// add moderated content
			ModerationQueueActivationManager::getInstance()->addModeratedContent('com.woltlab.wbb.post', $post->postID);
			
			if (!$isBulkProcessing) {
				if (!isset($boardStats[$post->getThread()->boardID])) {
					$boardStats[$post->getThread()->boardID] = array(
						'posts' => 0
					);
				}
				$boardStats[$post->getThread()->boardID]['posts']--;
				
				if ($post->userID && $post->getThread()->getBoard()->countUserPosts) {
					if (!isset($userPosts[$post->userID])) {
						$userPosts[$post->userID] = 0;
					}
					
					$userPosts[$post->userID]--;
					
					if (!isset($userToItems[$post->userID])) {
						$userToItems[$post->userID] = 0;
					}
					
					$userToItems[$post->userID]++;
				}
			}
			
			PostModificationLogHandler::getInstance()->disable($post->getDecoratedObject());
		}
		
		// update counters
		if (!empty($boardStats)) {
			foreach ($boardStats as $boardID => $stats) {
				$boardEditor = new BoardEditor(BoardCache::getInstance()->getBoard($boardID));
				$boardEditor->updateCounters($stats);
			}
		}
		
		if (!$isBulkProcessing) {
			// remove activity points and activity events
			UserActivityPointHandler::getInstance()->removeEvents('com.woltlab.wbb.activityPointEvent.post', $userToItems);
			UserActivityEventHandler::getInstance()->removeEvents('com.woltlab.wbb.recentActivityEvent.post', $postIDs);
			
			// delete notifications
			UserNotificationHandler::getInstance()->deleteNotifications('post', 'com.woltlab.wbb.post', array(), $postIDs);
			UserNotificationHandler::getInstance()->deleteNotifications('quote', 'com.woltlab.wbb.post', array(), $postIDs);
			UserNotificationHandler::getInstance()->deleteNotifications('mention', 'com.woltlab.wbb.post', array(), $postIDs);
		}
		
		if (!empty($checkThreads)) {
			$checkThreads = array_unique($checkThreads);
			
			// rebuild data
			ThreadEditor::rebuildThreadData($checkThreads);
			
			// disable threads if necessary
			$ignoreThreads = array();
			$conditionBuilder = new PreparedStatementConditionBuilder();
			$conditionBuilder->add('threadID IN (?)', array($checkThreads));
			$conditionBuilder->add('isDisabled = ?', array(0));
			$conditionBuilder->add('isDeleted = ?', array(0));
			$sql = "SELECT	threadID
				FROM	wbb".WCF_N."_post
				".$conditionBuilder;
			$statement = WCF::getDB()->prepareStatement($sql);
			$statement->execute($conditionBuilder->getParameters());
			while ($row = $statement->fetchArray()) $ignoreThreads[] = $row['threadID'];
			
			$disableThreads = array_diff($checkThreads, $ignoreThreads);
			if (!empty($disableThreads)) {
				$threadAction = new ThreadAction($disableThreads, 'disable', array('ignorePosts' => true));
				$threadAction->executeAction();
			}
		}
		
		$this->updateLastPost($boardIDs);
		
		// update users' posts
		if (!empty($userPosts)) {
			PostEditor::updatePostCounter($userPosts);
		}
		
		$this->unmarkItems();
		
		return $this->getPostData();
	}
	
	/**
	 * Validates if user can move posts into trash bin.
	 */
	public function validateTrash() {
		$this->prepareObjects();
		$this->readString('reason', true);
		
		// when trashing posts using the clipboard, the reason is stored
		// in the data array
		if (isset($this->parameters['data']) && isset($this->parameters['data']['reason'])) {
			$this->parameters['reason'] = $this->parameters['data']['reason'];
		}
		
		// validate permissions
		foreach ($this->objects as $post) {
			if (!$post->canDelete()) {
				throw new PermissionDeniedException();
			}
		}
	}
	
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::validateDelete()
	 */
	public function validateDelete() {
		$this->prepareObjects();
		
		// validate permissions
		foreach ($this->objects as $post) {
			if (!$post->canDeleteCompletely()) {
				throw new PermissionDeniedException();
			}
		}
	}
	
	/**
	 * Validates if user can restore posts.
	 */
	public function validateRestore() {
		$this->prepareObjects();
		
		// validate permissions
		foreach ($this->objects as $post) {
			if (!$post->canRestore()) {
				throw new PermissionDeniedException();
			}
		}
	}
	
	/**
	 * Prepares deletion of posts.
	 */
	protected function prepareObjects() {
		if (empty($this->objects)) {
			$this->readObjects();
			
			if (empty($this->objects)) {
				throw new UserInputException('objectIDs');
			}
		}
		
		// load threads
		$threadIDs = array();
		foreach ($this->objects as $post) {
			$threadIDs[] = $post->threadID;
		}
		
		$threadList = new ThreadList();
		$threadList->getConditionBuilder()->add("thread.threadID IN (?)", array($threadIDs));
		$threadList->readObjects();
		
		// assign threads
		foreach ($threadList as $thread) {
			foreach ($this->objects as $post) {
				if ($thread->threadID == $post->threadID) {
					$post->setThread($thread);
				}
			}
		}
	}
	
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::delete()
	 */
	public function delete() {
		$this->prepareObjects();
		
		$isBulkProcessing = (isset($this->parameters['isBulkProcessing']) && $this->parameters['isBulkProcessing']);
		$ignoreThreads = (isset($this->parameters['ignoreThreads']) && $this->parameters['ignoreThreads']);
		
		$threadIDs = array();
		foreach ($this->objects as $post) {
			if (!isset($threadIDs[$post->threadID])) {
				$threadIDs[$post->threadID] = $post->threadID;
			}
		}
		
		$postStats = $this->getDetailedPostStats($threadIDs);
		
		parent::delete();
		
		// update threads
		$attachmentPostIDs = $boardIDs = $pollIDs = $pollThreads = $postIDs = $userPosts = $userToItems = $threadEditors = array();
		foreach ($this->objects as $post) {
			$threadEditors[$post->threadID] = new ThreadEditor($post->getThread());
			$postIDs[] = $post->postID;
			
			if (!$isBulkProcessing) {
				if ($post->userID && !$post->isDisabled && $post->getThread()->getBoard()->countUserPosts) {
					if (!isset($userPosts[$post->userID])) {
						$userPosts[$post->userID] = 0;
					}
					
					$userPosts[$post->userID]--;
					
					if (!isset($userToItems[$post->userID])) {
						$userToItems[$post->userID] = 0;
					}
					
					$userToItems[$post->userID]++;
				}
			}
			
			$postStats[$post->threadID]['posts']--;
			if ($post->isDisabled) {
				$postStats[$post->threadID]['disabledPosts']--;
			}
			
			// thread has no posts anymore -> delete it
			if (!$ignoreThreads && !$postStats[$post->threadID]['posts']) {
				unset($threadIDs[$post->threadID]); // deleted threads need no data update
				
				$threadEditors[$post->threadID]->delete();
				$boardIDs[] = $threadEditors[$post->threadID]->boardID;
				ThreadModificationLogHandler::getInstance()->delete($post->getThread());
				
				$this->addThreadData($post->threadID, 'deleted', $threadEditors[$post->threadID]->getBoard()->getLink());
				
				continue;
			}
			
			PostModificationLogHandler::getInstance()->delete($post->getDecoratedObject());
			
			if ($post->pollID) {
				$pollIDs[] = $post->pollID;
				if (!isset($pollThreads[$post->threadID])) {
					$pollThreads[$post->threadID] = array(
						'count' => 0,
						'object' => $threadEditors[$post->threadID]
					);
				}
				
				$pollThreads[$post->threadID]['count']++;
			}
			
			if ($post->attachments) {
				$attachmentPostIDs[] = $post->postID;
			}
			
			$this->addPostData($post->getDecoratedObject(), 'deleted', 1);
		}
		
		if (!$ignoreThreads) {
			// check if the left posts are all disabled and if the thread
			// is also disabled
			$disableThreads = array();
			foreach ($threadIDs as $threadID) {
				if ($postStats[$threadID]['posts'] == $postStats[$threadID]['disabledPosts'] && !$threadEditors[$threadID]->isDisabled) {
					$disableThreads[] = $threadEditors[$threadID];
					$this->addThreadData($threadID, 'isDisabled', 1);
					
					// after disabling the thread, the thread data
					// will be be rebuild in that method, so no need
					// to rebuild it here, too
					unset($threadIDs[$threadID]);
				}
			}
			if (!empty($disableThreads)) {
				$threadAction = new ThreadAction($disableThreads, 'disable', array(
					'ignorePosts' => true
				));
				$threadAction->executeAction();
			}
			
			if (!$isBulkProcessing && !empty($threadIDs)) {
				ThreadEditor::rebuildThreadData($threadIDs);
			}
		}
		
		if (!empty($postIDs)) {
			// update search index
			SearchIndexManager::getInstance()->delete('com.woltlab.wbb.post', $postIDs);
			
			// unmark posts
			if (isset($this->parameters['unmarkItems'])) {
				$this->unmarkItems($postIDs);
			}
			
			// delete likes
			LikeHandler::getInstance()->removeLikes('com.woltlab.wbb.likeablePost', $postIDs);
			
			// remove activity points
			UserActivityPointHandler::getInstance()->removeEvents('com.woltlab.wbb.activityPointEvent.post', $userToItems);
		}
		
		// update last post
		if (!$isBulkProcessing) {
			$this->updateLastPost($boardIDs);
		}
		
		// remove polls
		if (!empty($pollIDs)) {
			PollManager::getInstance()->removePolls($pollIDs);
			
			// update poll count
			foreach ($pollThreads as $pollThread) {
				$pollThread['object']->updateCounters(array(
					'polls' => -1 * $pollThread['count']
				));
			}
		}
		
		// remove attachments
		if (!empty($attachmentPostIDs)) {
			AttachmentHandler::removeAttachments('com.woltlab.wbb.post', $attachmentPostIDs);
		}
		
		// update users' posts
		if (!$isBulkProcessing && !empty($userPosts)) {
			PostEditor::updatePostCounter($userPosts);
		}
		
		return $this->getPostData();
	}
	
	/**
	 * Moves given posts into trash bin.
	 */
	public function trash() {
		$isBulkProcessing = (isset($this->parameters['isBulkProcessing']) && $this->parameters['isBulkProcessing']);
		$ignoreThreads = (isset($this->parameters['ignoreThreads']) && $this->parameters['ignoreThreads']);
		
		$threadIDs = array();
		foreach ($this->objects as $post) {
			if (!isset($threadIDs[$post->threadID])) {
				$threadIDs[$post->threadID] = $post->threadID;
			}
		}
		
		if (!$ignoreThreads) {
			$postStats = $this->getDetailedPostStats($threadIDs);
		}
		
		// update threads
		$boardPostUpdates = $boardIDs = $postIDs = $trashThreads = $disableThreads = array();
		foreach ($this->objects as $post) {
			// ignore posts which are already in trash bin
			if ($post->isDeleted) {
				continue;
			}
			
			$parameters = array();
			$threadEditor = new ThreadEditor($post->getThread());
			$postIDs[] = $post->postID;
			
			if (!$ignoreThreads) {
				if (!isset($boardPostUpdates[$threadEditor->boardID])) {
					$boardPostUpdates[$threadEditor->boardID] = 0;
				}
				if (!$post->isDisabled) {
					$boardPostUpdates[$threadEditor->boardID]--;
					$postStats[$post->threadID]['posts']--;
					$postStats[$post->threadID]['visiblePosts']--;
				}
				else {
					$postStats[$post->threadID]['disabledPosts']--;
				}
				
				// no more visible posts
				if (!$postStats[$post->threadID]['visiblePosts']) {
					if ($postStats[$post->threadID]['disabledPosts']) {
						// thread has disabled posts thus disable
						// thread if it isn't already disabled
						if (!$threadEditor->isDisabled) {
							$disableThreads[] = $threadEditor;
							
							$this->addThreadData($threadEditor->threadID, 'isDisabled', 1);
						}
					}
					else {
						// thread only has deleted posts anymore
						// thus trash it
						$trashThreads[] = $threadEditor;
						
						$this->addThreadData($threadEditor->threadID, 'isDeleted', 1);
					}
					$boardIDs[] = $threadEditor->boardID;
				}
			}
			
			$post->update(array(
				'deleteTime' => TIME_NOW,
				'isDeleted' => 1
			));
			PostModificationLogHandler::getInstance()->trash($post->getDecoratedObject(), (!empty($this->parameters['reason']) ? $this->parameters['reason'] : ''));
			
			$this->addPostData($post->getDecoratedObject(), 'isDeleted', 1);
			
			if (!$isBulkProcessing && !$ignoreThreads) {
				if ($post->postID == $threadEditor->lastPostID) {
					$threadEditor->updateLastPost();
				}
			}
		}
		
		if (!empty($threadIDs)) {
			ThreadEditor::rebuildThreadData($threadIDs);
		}
		
		// disable threads with no more visible posts but which still have
		// disabled posts
		if (!$ignoreThreads && !empty($disableThreads)) {
			$threadAction = new ThreadAction($disableThreads, 'disable', array(
				'ignorePosts' => true
			));
			$threadAction->executeAction();
		}
		
		// trash threads with no more visible posts
		if (!$ignoreThreads && !empty($trashThreads)) {
			$threadAction = new ThreadAction($trashThreads, 'trash', array(
				'data' => array(
					'reason' => empty($this->parameters['reason']) ? '' : $this->parameters['reason']
				),
				'ignorePosts' => true
			));
			$threadAction->executeAction();
		}
		
		// update board stats
		$updatedBoardStats = false;
		if (!$isBulkProcessing) {
			foreach ($boardPostUpdates as $boardID => $postUpdate) {
				if ($postUpdate) {
					$boardEditor = new BoardEditor(BoardCache::getInstance()->getBoard($boardID));
					$boardEditor->updateCounters(array(
						'posts' => $postUpdate
					));
					
					$updatedBoardStats = true;
				}
			}
		}
		
		if ($updatedBoardStats) {
			BoardEditor::resetDataCache();
		}
		
		$this->unmarkItems($postIDs);
		
		// update last posts
		if (!$isBulkProcessing) {
			$this->updateLastPost($boardIDs);
		}
		
		return $this->getPostData();
	}
	
	/**
	 * Restores given posts.
	 */
	public function restore() {
		$isBulkProcessing = (isset($this->parameters['isBulkProcessing']) && $this->parameters['isBulkProcessing']);
		
		// update threads
		$boardIDs = $boardStats = $postIDs = $threadIDs = array();
		foreach ($this->objects as $post) {
			// ignore posts already restored
			if (!$post->isDeleted) {
				continue;
			}
			
			$parameters = array();
			$threadEditor = new ThreadEditor($post->getThread());
			$postIDs[] = $post->postID;
			$threadIDs[] = $post->threadID;
			
			if (!$post->isDisabled) {
				if (!isset($boardStats[$threadEditor->boardID])) {
					$boardStats[$threadEditor->boardID] = array(
						'posts' => 0,
						'threads' => array()
					);
				}
				$boardStats[$threadEditor->boardID]['posts']++;
			}
			
			// restore thread if at least one post is accessible
			if ($threadEditor->isDeleted && !isset($boardStats[$threadEditor->boardID]['threads'][$threadEditor->threadID])) {
				$parameters['deleteTime'] = 0;
				$parameters['isDeleted'] = 0;
				$boardIDs[] = $threadEditor->boardID;
				
				ThreadModificationLogHandler::getInstance()->restore($post->getThread());
				
				$this->addThreadData($threadEditor->threadID, 'isDeleted', 0);
				$this->addThreadData($threadEditor->threadID, 'ignoreDeletedPosts', 1);
				
				// mark that this thread will be restored already
				$boardStats[$threadEditor->boardID]['threads'][$threadEditor->threadID] = $threadEditor->threadID;
			}
			
			// update thread
			if (!empty($parameters)) {
				$threadEditor->update($parameters);
			}
			
			// restore post
			$post->update(array(
				'deleteTime' => 0,
				'isDeleted' => 0
			));
			PostModificationLogHandler::getInstance()->restore($post->getDecoratedObject());
			
			if (isset($this->parameters['unmarkItems'])) {
				$postIDs[] = $post->postID;
			}
			
			$this->addPostData($post->getDecoratedObject(), 'isDeleted', 0);
			
			if (!$isBulkProcessing) {
				if ($post->time > $threadEditor->lastPostTime) {
					$threadEditor->updateLastPost();
				}
			}
		}
		
		if (!empty($threadIDs)) {
			$threadIDs = array_unique($threadIDs);
			ThreadEditor::rebuildThreadData($threadIDs);
		}
		
		if (!empty($postIDs)) {
			$this->unmarkItems($postIDs);
		}
		
		// update board counters
		$updatedBoardStats = false;
		if (!$isBulkProcessing) {
			foreach ($boardStats as $boardID => $update) {
				$boardEditor = new BoardEditor(BoardCache::getInstance()->getBoard($boardID));
				$boardEditor->updateCounters(array(
					'posts' => $update['posts'],
					'threads' => count($update['threads'])
				));
				
				$updatedBoardStats = true;
			}
		}
		
		if ($updatedBoardStats) {
			BoardEditor::resetDataCache();
		}
		
		// update last post
		if (!$isBulkProcessing) {
			$this->updateLastPost($boardIDs);
		}
		
		return $this->getPostData();
	}
	
	/**
	 * Validates parameters to close posts.
	 */
	public function validateClose() {
		$this->prepareObjects();
		
		foreach ($this->objects as $post) {
			if (!$post->getThread()->getBoard()->getModeratorPermission('canClosePost')) {
				throw new PermissionDeniedException();
			}
		}
	}
	
	/**
	 * Closes posts.
	 * 
	 * @return	array<array>
	 */
	public function close() {
		foreach ($this->objects as $post) {
			$post->update(array(
				'isClosed' => 1
			));
			
			PostModificationLogHandler::getInstance()->close($post->getDecoratedObject());
			$this->addPostData($post->getDecoratedObject(), 'isClosed', 1);
		}
		
		return $this->getPostData();
	}
	
	/**
	 * Validates parameters to open posts.
	 */
	public function validateOpen() {
		$this->validateClose();
	}
	
	/**
	 * Opens posts.
	 * 
	 * @return	array<array>
	 */
	public function open() {
		foreach ($this->objects as $post) {
			$post->update(array(
				'isClosed' => 0
			));
			
			PostModificationLogHandler::getInstance()->open($post->getDecoratedObject());
			$this->addPostData($post->getDecoratedObject(), 'isClosed', 0);
		}
		
		return $this->getPostData();
	}
	
	/**
	 * Validates parameters to prepare posts for merging.
	 */
	public function validatePrepareMerge() {
		// read posts
		$postList = new ViewablePostList();
		$postList->getConditionBuilder()->add("post.postID IN (?)", array($this->objectIDs));
		$postList->readObjects();
		$this->objects = $postList->getObjects();
		
		if (count($this->objects) < 2) {
			throw new UserInputException('objectIDs');
		}
		
		// read threads
		$threadIDs = array();
		foreach ($this->objects as $post) {
			$threadIDs[] = $post->threadID;
		}
		
		$threadList = new ThreadList();
		$threadList->getConditionBuilder()->add("thread.threadID IN (?)", array($threadIDs));
		$threadList->readObjects();
		$threads = $threadList->getObjects();
		foreach ($this->objects as $post) {
			$post->setThread($threads[$post->threadID]);
			
			if (!$post->getThread()->canRead() || !$post->getThread()->getBoard()->getModeratorPermission('canMergePost')) {
				throw new PermissionDeniedException();
			}
		}
	}
	
	/**
	 * Prepares posts for merging.
	 * 
	 * @return	array
	 */
	public function prepareMerge() {
		WCF::getTPL()->assign(array(
			'posts' => $this->objects
		));
		
		return array(
			'template' => WCF::getTPL()->fetch('postMerge', 'wbb')
		);
	}
	
	/**
	 * Validates parameters to merge posts.
	 */
	public function validateMerge() {
		$this->validatePrepareMerge();
		
		if (!isset($this->parameters['postOrder']) || !is_array($this->parameters['postOrder']) || count($this->parameters['postOrder']) != count($this->objects)) {
			throw new UserInputException('postOrder');
		}
		
		foreach ($this->parameters['postOrder'] as $index => $postID) {
			if (!isset($this->objects[$postID])) {
				throw new UserInputException('postOrder');
			}
			
			if ($index == 0) {
				$this->postEditor = new PostEditor($this->objects[$postID]->getDecoratedObject());
				unset($this->parameters['postOrder'][$index]);
				unset($this->objects[$postID]);
			}
		}
		
		if ($this->postEditor === null) {
			throw new UserInputException('postOrder');
		}
	}
	
	/**
	 * Merges posts.
	 * 
	 * @return	array
	 */
	public function merge() {
		$attachmentPostIDs = $removePollIDs = array();
		$attachmentsCount = $this->postEditor->attachments;
		$message = $this->postEditor->message;
		$pollID = $this->postEditor->pollID;
		$threadUpdates = array();
		$userPostUpdates = array();
		foreach ($this->parameters['postOrder'] as $postID) {
			$post = $this->objects[$postID];
			
			// merge post messages
			$message .= "\n\n";
			$message .= $post->message;
			
			// merge attachments
			if ($post->attachments) {
				$attachmentsCount += $post->attachments;
				$attachmentPostIDs[] = $post->postID;
			}
			
			if ($post->pollID) {
				if ($pollID) {
					$removePollIDs[] = $post->pollID;
				}
				else {
					$pollID = $post->pollID;
				}
			}
		}
		
		// merge attachments
		if (!empty($attachmentPostIDs)) {
			AttachmentHandler::transferAttachments('com.woltlab.wbb.post', $this->postEditor->postID, $attachmentPostIDs);
		}
		
		// update post
		$this->postEditor->update(array(
			'attachments' => $attachmentsCount,
			'message' => $message,
			'pollID' => $pollID
		));
		
		// remove likes
		LikeHandler::getInstance()->removeLikes('com.woltlab.wbb.likeablePost', array_keys($this->objects));
		
		// remove old posts
		$posts = array();
		foreach ($this->objects as $post) {
			$posts[$post->postID] = $post->getDecoratedObject();
		}
		$postAction = new PostAction($posts, 'delete');
		$postAction->executeAction();
		
		// remove unusable polls
		if (!empty($removePollIDs)) {
			PollManager::getInstance()->removePolls($removePollIDs);
		}
		
		// unmark post ids
		$postIDs = array_keys($posts);
		$postIDs[] = $this->postEditor->postID;
		$this->unmarkItems($postIDs);
		
		// redirect to merge target
		return array(
			'redirectURL' => LinkHandler::getInstance()->getLink('Thread', array(
				'application' => 'wbb',
				'object' => $this->postEditor->getThread()
			), 'postID='.$this->postEditor->postID.'#post'.$this->postEditor->postID)
		);
	}
	
	/**
	 * Validates the get post preview action.
	 */
	public function validateGetPostPreview() {
		$this->postEditor = $this->getSingleObject();
		if (!$this->postEditor->canRead()) {
			throw new PermissionDeniedException();
		}
	}
	
	/**
	 * Returns a preview of a post.
	 * 
	 * @return	array
	 */
	public function getPostPreview() {
		$postList = new ViewablePostList();
		$postList->getConditionBuilder()->add('post.postID = ?', array($this->postEditor->postID));
		$postList->sqlLimit = 1;
		$postList->readObjects();
		$posts = $postList->getObjects();
		
		WCF::getTPL()->assign(array(
			'post' => reset($posts)
		));
		return array(
			'template' => WCF::getTPL()->fetch('postPreview', 'wbb')
		);
	}
	
	/**
	 * @see	wcf\data\IMessageQuickReplyAction::validateQuickReply()
	 */
	public function validateQuickReply() {
		// check flood control
		Post::enforceFloodControl();
		
		QuickReplyManager::getInstance()->setAllowedBBCodes(explode(',', WCF::getSession()->getPermission('user.message.allowedBBCodes')));
		QuickReplyManager::getInstance()->validateParameters($this, $this->parameters, 'wbb\data\thread\Thread', 'wbb\data\thread\ViewableThread');
	}
	
	/**
	 * @see	wcf\data\IMessageQuickReplyAction::quickReply()
	 */
	public function quickReply() {
		if (!QuickReplyManager::getInstance()->getContainer()->getBoard()->getPermission('canReplyThreadWithoutModeration')) {
			$this->parameters['data']['isDisabled'] = 1;
		}
		
		return QuickReplyManager::getInstance()->createMessage(
			$this,
			$this->parameters,
			'wbb\data\thread\ThreadAction',
			QuickReplyManager::getInstance()->getContainer()->getBoard()->getPostSortOrder(),
			'threadPostList',
			'wbb'
		);
	}
	
	/**
	 * @see	wcf\data\IMessageQuickReplyAction::validateMessage()
	 */
	public function validateMessage(DatabaseObject $thread, $message) {
		$message = MessageUtil::stripCrap($message);
		
		if (WBB_POST_MIN_CHAR_LENGTH && (mb_strlen($message) < WBB_POST_MIN_CHAR_LENGTH)) {
			throw new UserInputException('message', WCF::getLanguage()->getDynamicVariable('wbb.post.message.error.minCharLength', array('minCharLength' => WBB_POST_MIN_CHAR_LENGTH)));
		}
		
		if (WBB_POST_MIN_WORD_COUNT && (count(explode(' ', $message)) < WBB_POST_MIN_WORD_COUNT)) {
			throw new UserInputException('message', WCF::getLanguage()->getDynamicVariable('wbb.post.message.error.minWordCount', array('minWordCount' => WBB_POST_MIN_WORD_COUNT)));
		}
		
		if (mb_strlen($message) > WCF::getSession()->getPermission('user.board.maxPostLength')) {
			throw new UserInputException('message', WCF::getLanguage()->getDynamicVariable('wcf.message.error.tooLong', array('maxTextLength' => WCF::getSession()->getPermission('user.board.maxPostLength'))));
		}
		
		// search for disallowed bbcodes
		$disallowedBBCodes = BBCodeParser::getInstance()->validateBBCodes($message, explode(',', WCF::getSession()->getPermission('user.message.allowedBBCodes')));
		if (!empty($disallowedBBCodes)) {
			throw new UserInputException('text', WCF::getLanguage()->getDynamicVariable('wcf.message.error.disallowedBBCodes', array('disallowedBBCodes' => $disallowedBBCodes)));
		}
		
		// search for censored words
		if (ENABLE_CENSORSHIP) {
			$result = Censorship::getInstance()->test($message);
			if ($result) {
				throw new UserInputException('message', WCF::getLanguage()->getDynamicVariable('wcf.message.error.censoredWordsFound', array('censoredWords' => $result)));
			}
		}
	}
	
	/**
	 * @see	wcf\data\IMessageQuickReplyAction::validateContainer()
	 */
	public function validateContainer(DatabaseObject $thread) {
		if (!$thread->threadID) {
			throw new UserInputException('objectID');
		}
		
		if (!$thread->canReply()) {
			throw new PermissionDeniedException();
		}
	}
	
	/**
	 * @see	wcf\data\IMessageQuickReplyAction::getMessageList()
	 */
	public function getMessageList(DatabaseObject $thread, $lastMessageTime) {
		$messageList = new ThreadPostList($thread->getDecoratedObject());
		$messageList->getConditionBuilder()->add("post.time > ?", array($lastMessageTime));
		$messageList->sqlOrderBy = "post.time ".$thread->getBoard()->getPostSortOrder();
		$messageList->readObjects();
		
		return $messageList;
	}
	
	/**
	 * @see	wcf\data\IMessageQuickReply::getPageNo()
	 */
	public function getPageNo(DatabaseObject $thread) {
		$sql = "SELECT	COUNT(*) AS count
			FROM	wbb".WCF_N."_post
			WHERE	threadID = ?";
		$statement = WCF::getDB()->prepareStatement($sql);
		$statement->execute(array($thread->threadID));
		$count = $statement->fetchArray();
		
		return array(intval(ceil($count['count'] / $thread->getBoard()->getPostsPerPage())), $count['count']);
	}
	
	/**
	 * @see	wcf\data\IMessageQuickReply::getRedirectUrl()
	 */
	public function getRedirectUrl(DatabaseObject $thread, DatabaseObject $post) {
		return LinkHandler::getInstance()->getLink('Thread', array(
			'application' => 'wbb',
			'object' => $thread,
			'postID' => $post->postID
		)).'#post'.$post->postID;
	}
	
	/**
	 * Validates the 'getPosts' action.
	 */
	public function validateGetPosts() {
		if (empty($this->objectIDs)) {
			throw new UserInputException('objectIDs');
		}
		
		$post = new Post(reset($this->objectIDs));
		$this->thread = new Thread($post->threadID);
		
		$this->postList = new ViewablePostList();
		$this->postList->getConditionBuilder()->add('post.postID IN (?)', array($this->objectIDs));
		$this->postList->setThread($this->thread);
		$this->postList->readObjects();
		
		// make sure that all posts belong to the same thread
		foreach ($this->postList as $post) {
			if (!$post->canRead()) {
				throw new PermissionDeniedException();
			}
			
			if ($post->threadID != $this->thread->threadID) {
				throw new UserInputException('objectIDs');
			}
		}
	}
	
	/**
	 * Loads a list of posts that belong to one thread.
	 * 
	 * @return	array<template>
	 */
	public function getPosts() {
		if (isset($this->parameters['showCollapsedPosts'])) {
			WCF::getTPL()->assign(array(
				'showCollapsedPosts' => true
			));
		}
		
		WCF::getTPL()->assign(array(
			'attachmentList' => $this->postList->getAttachmentList(),
			'objects' => $this->postList,
			'sortOrder' => $this->thread->getBoard()->getPostSortOrder(),
			'thread' => new ViewableThread($this->thread)
		));
		
		return array(
			'template' => WCF::getTPL()->fetch('threadPostList', 'wbb')
		);
	}
	
	/**
	 * @see	wcf\data\IExtendedMessageQuickReplyAction::validateJumpToExtended()
	 */
	public function validateJumpToExtended() {
		if (!isset($this->parameters['message'])) {
			throw new UserInputException('message');
		}
		
		$this->parameters['containerID'] = (isset($this->parameters['containerID'])) ? intval($this->parameters['containerID']) : 0;
		if (!$this->parameters['containerID']) {
			throw new UserInputException('containerID');
		}
		else {
			$this->thread = new Thread($this->parameters['containerID']);
			if (!$this->thread->threadID) {
				throw new UserInputException('containerID');
			}
		}
		
		// request if for quick reply
		if (!isset($this->parameters['messageID'])) {
			if (!$this->thread->canReply()) {
				throw new PermissionDeniedException();
			}
		}
		else {
			// editing existing post
			$this->post = new Post(intval($this->parameters['messageID']));
			if (!$this->post->postID || ($this->thread->threadID != $this->post->threadID)) {
				throw new UserInputException('messageID');
			}
			
			if (!$this->thread->canEditPost($this->post)) {
				throw new PermissionDeniedException();
			}
		}
	}
	
	/**
	 * @see	wcf\data\IExtendedMessageQuickReplyAction::jumpToExtended()
	 */
	public function jumpToExtended() {
		// quick reply
		if ($this->post === null) {
			QuickReplyManager::getInstance()->setMessage('thread', $this->thread->threadID, $this->parameters['message']);
			$url = LinkHandler::getInstance()->getLink('PostAdd', array(
				'application' => 'wbb',
				'object' => $this->thread
			));
		}
		else {
			// editing message
			QuickReplyManager::getInstance()->setMessage('threadPost', $this->post->postID, $this->parameters['message']);
			$url = LinkHandler::getInstance()->getLink('PostEdit', array(
				'application' => 'wbb',
				'id' => $this->post->postID
			));
		}
		
		// redirect
		return array(
			'url' => $url
		);
	}
	
	/**
	 * @see	wcf\data\IMessageInlineEditorAction::validateBeginEdit()
	 */
	public function validateBeginEdit() {
		$this->parameters['containerID'] = (isset($this->parameters['containerID'])) ? intval($this->parameters['containerID']) : 0;
		if (!$this->parameters['containerID']) {
			throw new UserInputException('containerID');
		}
		else {
			$this->thread = new Thread($this->parameters['containerID']);
			if (!$this->thread->threadID) {
				throw new UserInputException('containerID');
			}
		}
		
		$this->parameters['objectID'] = (isset($this->parameters['objectID'])) ? intval($this->parameters['objectID']) : 0;
		if (!$this->parameters['objectID']) {
			throw new UserInputException('objectID');
		}
		else {
			$this->post = new Post($this->parameters['objectID']);
			if (!$this->post->postID || ($this->thread->threadID != $this->post->threadID)) {
				throw new UserInputException('objectID');
			}
			
			if (!$this->thread->canEditPost($this->post)) {
				throw new PermissionDeniedException();
			}
		}
	}
	
	/**
	 * @see	wcf\data\IMessageInlineEditorAction::beginEdit()
	 */
	public function beginEdit() {
		BBCodeHandler::getInstance()->setAllowedBBCodes(explode(',', WCF::getSession()->getPermission('user.message.allowedBBCodes')));
		
		WCF::getTPL()->assign(array(
			'defaultSmilies' => SmileyCache::getInstance()->getCategorySmilies(),
			'permissionCanUseSmilies' => 'user.message.canUseSmilies',
			'post' => $this->post,
			'wysiwygSelector' => 'messageEditor'.$this->post->postID
		));
		
		return array(
			'actionName' => 'beginEdit',
			'template' => WCF::getTPL()->fetch('postInlineEditor', 'wbb')
		);
	}
	
	/**
	 * @see	wcf\data\IMessageInlineEditorAction::validateSave()
	 */
	public function validateSave() {
		if (!isset($this->parameters['data']) || !isset($this->parameters['data']['message']) || empty($this->parameters['data']['message'])) {
			throw new UserInputException('message');
		}
		
		$this->validateBeginEdit();
		
		$this->validateMessage($this->thread, $this->parameters['data']['message']);
	}
	
	/**
	 * @see	wcf\data\IMessageInlineEditorAction::save()
	 */
	public function save() {
		$postData = array(
			'message' => MessageUtil::stripCrap($this->parameters['data']['message'])
		);
		
		// pre-parse message text
		$postData['message'] = PreParser::getInstance()->parse($postData['message'], explode(',', WCF::getSession()->getPermission('user.message.allowedBBCodes')));
		
		// append edit note
		$board = BoardCache::getInstance()->getBoard($this->thread->boardID);
		if (!$board->getPermission('canHideEditNote') && (WCF::getUser()->userID != $this->post->userID || $this->post->time <= TIME_NOW - WBB_POST_EDIT_HIDE_EDIT_NOTE_PERIOD * 60)) {
			$postData['editCount'] = $this->post->editCount + 1;
			$postData['editReason'] = '';
			$postData['editor'] = WCF::getUser()->username;
			$postData['editorID'] = WCF::getUser()->userID;
			$postData['lastEditTime'] = TIME_NOW;
		}
		
		// execute update action
		$action = new PostAction(array($this->post), 'update', array('data' => $postData));
		$action->executeAction();
		
		// load new post
		$post = new Post($this->post->postID);
		$attachmentList = $post->getAttachments();
		if ($attachmentList !== null) {
			// set permisions
			$attachmentList->setPermissions(array(
				'canDownload' => $board->getPermission('canDownloadAttachment'),
				'canViewPreview' => $board->getPermission('canViewAttachmentPreview')
			));
		}
		
		return array(
			'actionName' => 'save',
			'message' => $post->getFormattedMessage()
		);
	}
	
	/**
	 * Validates parameters to return the logged ip addresses.
	 */
	public function validateGetIpLog() {
		if (!LOG_IP_ADDRESS) {
			throw new PermissionDeniedException();
		}
		
		if (isset($this->parameters['postID'])) {
			$this->post = new Post($this->parameters['postID']);
		}
		if ($this->post === null || !$this->post->postID) {
			throw new UserInputException('postID');
		}
		
		if (!$this->post->canRead()) {
			throw new PermissionDeniedException();
		}
	}
	
	/**
	 * Returns a list of the logged ip addresses.
	 * 
	 * @return	array
	 */
	public function getIpLog() {
		// get ip addresses of the author
		$authorIpAddresses = Post::getIpAddressByAuthor($this->post->userID, $this->post->username, $this->post->ipAddress);
		
		// resolve hostnames
		$newIpAddresses = array();
		foreach ($authorIpAddresses as $ipAddress) {
			$ipAddress = UserUtil::convertIPv6To4($ipAddress);
			
			$newIpAddresses[] = array(
				'hostname' => @gethostbyaddr($ipAddress),
				'ipAddress' => $ipAddress
			);
		}
		$authorIpAddresses = $newIpAddresses;
		
		// get other users of this ip address
		$otherUsers = array();
		if ($this->post->ipAddress) {
			$otherUsers = Post::getAuthorByIpAddress($this->post->ipAddress, $this->post->userID, $this->post->username);
		}
		
		$ipAddress = UserUtil::convertIPv6To4($this->post->ipAddress);
		
		if ($this->post->userID) {
			$sql = "SELECT	registrationIpAddress
				FROM	wcf".WCF_N."_user
				WHERE	userID = ?";
			$statement = WCF::getDB()->prepareStatement($sql);
			$statement->execute(array(
				$this->post->userID
			));
			$row = $statement->fetchArray();
			
			if ($row !== false && $row['registrationIpAddress']) {
				$registrationIpAddress = UserUtil::convertIPv6To4($row['registrationIpAddress']);
				WCF::getTPL()->assign(array(
					'registrationIpAddress' => array(
						'hostname' => @gethostbyaddr($registrationIpAddress),
						'ipAddress' => $registrationIpAddress
					)
				));
			}
		}
		
		WCF::getTPL()->assign(array(
			'authorIpAddresses' => $authorIpAddresses,
			'ipAddress' => array(
				'hostname' => @gethostbyaddr($ipAddress),
				'ipAddress' => $ipAddress
			),
			'otherUsers' => $otherUsers,
			'post' => $this->post
		));
		
		return array(
			'postID' => $this->post->postID,
			'template' => WCF::getTPL()->fetch('postIpAddress', 'wbb')
		);
	}
	
	/**
	 * @see	wcf\data\IMessageQuoteAction::validateSaveFullQuote()
	 */
	public function validateSaveFullQuote() {
		if (empty($this->objects)) {
			$this->readObjects();
			
			if (empty($this->objects)) {
				throw new UserInputException('objectIDs');
			}
		}
		
		// validate permissions
		$this->post = current($this->objects);
		if (!$this->post->canRead()) {
			throw new PermissionDeniedException();
		}
	}
	
	/**
	 * @see	wcf\data\IMessageQuoteAction::saveFullQuote()
	 */
	public function saveFullQuote() {
		if (!MessageQuoteManager::getInstance()->addQuote('com.woltlab.wbb.post', $this->post->threadID, $this->post->postID, $this->post->getExcerpt(), $this->post->getMessage())) {
			$quoteID = MessageQuoteManager::getInstance()->getQuoteID('com.woltlab.wbb.post', $this->post->postID, $this->post->getExcerpt(), $this->post->getMessage());
			MessageQuoteManager::getInstance()->removeQuote($quoteID);
		}
		
		return array(
			'count' => MessageQuoteManager::getInstance()->countQuotes(),
			'fullQuoteMessageIDs' => MessageQuoteManager::getInstance()->getFullQuoteObjectIDs(array('com.woltlab.wbb.post'))
		);
	}
	
	/**
	 * @see	wcf\data\IMessageQuoteAction::validateSaveQuote()
	 */
	public function validateSaveQuote() {
		$this->parameters['message'] = (isset($this->parameters['message'])) ? StringUtil::trim($this->parameters['message']) : '';
		if (empty($this->parameters['message'])) {
			throw new UserInputException('message');
		}
		
		if (empty($this->objects)) {
			$this->readObjects();
		
			if (empty($this->objects)) {
				throw new UserInputException('objectIDs');
			}
		}
		
		$this->post = current($this->objects);
		if (!$this->post->canRead()) {
			throw new PermissionDeniedException();
		}
	}
	
	/**
	 * @see	wcf\data\IMessageQuoteAction::saveQuote()
	 */
	public function saveQuote() {
		MessageQuoteManager::getInstance()->addQuote('com.woltlab.wbb.post', $this->post->threadID, $this->post->postID, $this->parameters['message']);
		
		return array(
			'count' => MessageQuoteManager::getInstance()->countQuotes(),
			'fullQuoteMessageIDs' => MessageQuoteManager::getInstance()->getFullQuoteObjectIDs(array('com.woltlab.wbb.post'))
		);
	}
	
	/**
	 * @see	wcf\data\IMessageQuoteAction::validateGetRenderedQuotes()
	 */
	public function validateGetRenderedQuotes() {
		$this->readInteger('parentObjectID');
		
		$this->thread = new Thread($this->parameters['parentObjectID']);
		if (!$this->thread->threadID) {
			throw new UserInputException('parentObjectID');
		}
	}
	
	/**
	 * @see	wcf\data\IMessageQuoteAction::getRenderedQuotes()
	 */
	public function getRenderedQuotes() {
		$quotes = MessageQuoteManager::getInstance()->getQuotesByParentObjectID('com.woltlab.wbb.post', $this->thread->threadID);
		
		return array(
			'template' => implode("\n\n", $quotes)
		);
	}
	
	/**
	 * Validates the 'unmarkAll' action.
	 */
	public function validateUnmarkAll() {
		// does nothing
	}
	
	/**
	 * Unmarks all posts.
	 */
	public function unmarkAll() {
		ClipboardHandler::getInstance()->removeItems(ClipboardHandler::getInstance()->getObjectTypeID('com.woltlab.wbb.post'));
	}
	
	/**
	 * Validates parameters to move posts into an existing thread.
	 */
	public function validateMoveToExistingThread() {
		$this->readInteger('threadID');
		
		$this->readObjects();
		if (empty($this->objects)) {
			throw new UserInputException('objectIDs');
		}
		
		// validate thread and board permissions
		$this->thread = new Thread($this->parameters['threadID']);
		if (!$this->thread->threadID) {
			throw new UserInputException('threadID');
		}
		else if (!$this->thread->canRead() || !$this->thread->getBoard()->getModeratorPermission('canMovePost')) {
			throw new PermissionDeniedException();
		}
		
		$threadIDs = array();
		foreach ($this->objects as $post) {
			$threadIDs[] = $post->threadID;
			
			// cannot move post if origin and destination are equal
			if ($post->threadID == $this->thread->threadID) {
				throw new UserInputException('threadID');
			}
		}
		
		// read threads
		$threadList = new ThreadList();
		$threadList->getConditionBuilder()->add("thread.threadID IN (?)", array($threadIDs));
		$threadList->readObjects();
		$threads = $threadList->getObjects();
		
		// assign threads and validate permissions
		foreach ($this->objects as $post) {
			$post->setThread($threads[$post->threadID]);
			if (!$post->canRead() || !$post->getThread()->getBoard()->getModeratorPermission('canMovePost')) {
				throw new PermissionDeniedException();
			}
		}
	}
	
	/**
	 * Moves posts into an existing thread.
	 * 
	 * @return	array
	 */
	public function moveToExistingThread() {
		$addEvents = $boardIDs = $removeEvents = $threadIDs = $userPosts = array();
		foreach ($this->objects as $post) {
			$boardIDs[] = $post->getThread()->boardID;
			$threadIDs[] = $post->threadID;
			
			if (!$post->isDisabled && $post->userID) {
				if (!isset($userPosts[$post->userID])) {
					$userPosts[$post->userID] = 0;
				}
			
				if ($post->getThread()->getBoard()->countUserPosts != $this->thread->getBoard()->countUserPosts) {
					if ($post->getThread()->getBoard()->countUserPosts) {
						$userPosts[$post->userID]--;
						
						if (!isset($removeEvents[$post->userID])) {
							$removeEvents[$post->userID] = 0;
						}
						$removeEvents[$post->userID]++;
					}
					
					if ($this->thread->getBoard()->countUserPosts) {
						$userPosts[$post->userID]++;
						
						if (!isset($addEvents[$post->userID])) {
							$addEvents[$post->userID] = 0;
						}
						$addEvents[$post->userID]++;
					}
				}
			}
		}
		
		// move posts
		$postAction = new PostAction($this->objects, 'update', array('data' => array('threadID' => $this->thread->threadID)));
		$postAction->executeAction();
		
		// update users post counter
		foreach ($userPosts as $userID => $count) {
			if (!$count) {
				unset($userPosts[$userID]);
			}
		}
		if (!empty($userPosts)) {
			PostEditor::updatePostCounter($userPosts);
		}
		
		// update activity points
		if (!empty($addEvents)) {
			UserActivityPointHandler::getInstance()->fireEvents('com.woltlab.wbb.activityPointEvent.post', $addEvents);
		}
		if (!empty($removeEvents)) {
			UserActivityPointHandler::getInstance()->removeEvents('com.woltlab.wbb.activityPointEvent.post', $removeEvents);
		}
		
		// update threads
		$threadIDs[] = $this->thread->threadID;
		$threadAction = new ThreadAction($threadIDs, 'rebuild');
		$threadAction->executeAction();
		
		// update board stats
		$boardIDs[] = $this->thread->boardID;
		$boardIDs = array_unique($boardIDs);
		for ($i = 0, $length = count($boardIDs); $i < $length; $i++) {
			$boardEditor = new BoardEditor(BoardCache::getInstance()->getBoard($boardIDs[$i]));
			$boardEditor->rebuildStats();
			$boardEditor->updateLastPost();
		}
		
		// reset board cache
		BoardEditor::resetCache();
		
		// unmark clipboard items
		$this->unmarkItems($this->objectIDs);
		
		return array(
			'threadID' => $this->thread->threadID,
			'redirectURL' => LinkHandler::getInstance()->getLink('Thread', array(
				'application' => 'wbb',
				'object' => $this->thread
			))
		);
	}
	
	/**
	 * Validates parameters to move posts into a new thread.
	 */
	public function validateMoveToNewThread() {
		$this->readInteger('boardID');
		$this->readString('topic');
		
		$this->readObjects();
		if (empty($this->objects)) {
			throw new UserInputException('objectIDs');
		}
		
		// validate board
		$this->board = BoardCache::getInstance()->getBoard($this->parameters['boardID']);
		if ($this->board === null) {
			throw new UserInputException('boardID');
		}
		else if (!$this->board->canStartThread()) {
			throw new PermissionDeniedException();
		}
		
		$threadIDs = array();
		foreach ($this->objects as $post) {
			$threadIDs[] = $post->threadID;
		}
		
		// read threads
		$threadList = new ThreadList();
		$threadList->getConditionBuilder()->add("thread.threadID IN (?)", array($threadIDs));
		$threadList->readObjects();
		$threads = $threadList->getObjects();
		
		// assign threads and validate permissions
		foreach ($this->objects as $post) {
			$post->setThread($threads[$post->threadID]);
			if (!$post->getThread()->getBoard()->getModeratorPermission('canMovePost')) {
				throw new PermissionDeniedException();
			}
		}
	}
	
	/**
	 * Moves posts into a new thread.
	 * 
	 * @return	array
	 */
	public function moveToNewThread() {
		// identify posts which are their associated thread's starting post
		$addEvents = $boardIDs = $firstPostThreadIDs = $removeEvents = $threadIDs = $updateThreadIDs = $userPosts = array();
		$firstPost = null;
		$threadData = array(
			'attachments' => 0,
			'polls' => 0,
			'replies' => 0
		);
		
		foreach ($this->objects as $post) {
			if ($post->postID == $post->getThread()->firstPostID) {
				$firstPostThreadIDs[] = $post->threadID;
			}
			
			$updateThreadIDs[] = $post->threadID;
			
			// identify the chronological first post
			if ($firstPost === null || $post->time < $firstPost->time) {
				$firstPost = $post;
			}
			
			$threadData['attachments'] += $post->attachments;
			$threadData['polls'] += ($post->pollID ? 1 : 0);
			$threadData['replies']++;
			
			if (!$post->isDisabled && $post->userID) {
				if (!isset($userPosts[$post->userID])) {
					$userPosts[$post->userID] = 0;
				}
				
				if ($post->getThread()->getBoard()->countUserPosts != $this->board->countUserPosts) {
					if ($post->getThread()->getBoard()->countUserPosts) {
						$userPosts[$post->userID]--;
						
						if (!isset($removeEvents[$post->userID])) {
							$removeEvents[$post->userID] = 0;
						}
						$removeEvents[$post->userID]++;
					}
					
					if ($this->board->countUserPosts) {
						$userPosts[$post->userID]++;
						
						if (!isset($addEvents[$post->userID])) {
							$addEvents[$post->userID] = 0;
						}
						$addEvents[$post->userID]++;
					}
				}
			}
			
			$boardIDs[] = $post->getThread()->boardID;
			$threadIDs[] = $post->threadID;
		}
		
		// do not count first post as reply
		$threadData['replies']--;
		
		// create new thread
		$thread = ThreadEditor::create(array_merge($threadData, array(
			'boardID' => $this->board->boardID,
			'languageID' => $firstPost->getThread()->languageID,
			'topic' => $this->parameters['topic'],
			'firstPostID' => $firstPost->postID,
			'time' => $firstPost->time,
			'userID' => ($firstPost->userID ?: null),
			'username' => $firstPost->username,
			'cumulativeLikes' => $firstPost->cumulativeLikes,
			'isDeleted' => $firstPost->isDeleted,
			'isDisabled' => $firstPost->isDisabled
		)));
		
		// move posts to new thread
		$postAction = new PostAction($this->objects, 'update', array('data' => array('threadID' => $thread->threadID)));
		$postAction->executeAction();
		
		// update last post for newly created thread
		$threadEditor = new ThreadEditor($thread);
		$threadEditor->updateLastPost();
		
		// update users post counter
		foreach ($userPosts as $userID => $count) {
			if (!$count) {
				unset($userPosts[$userID]);
			}
		}
		if (!empty($userPosts)) {
			PostEditor::updatePostCounter($userPosts);
		}
		
		// update activity points
		if (!empty($addEvents)) {
			UserActivityPointHandler::getInstance()->fireEvents('com.woltlab.wbb.activityPointEvent.post', $addEvents);
		}
		if (!empty($removeEvents)) {
			UserActivityPointHandler::getInstance()->removeEvents('com.woltlab.wbb.activityPointEvent.post', $removeEvents);
		}
		
		// update old threads
		$threadAction = new ThreadAction($threadIDs, 'rebuild');
		$threadAction->executeAction();
		
		// update board stats
		$boardIDs[] = $this->board->boardID;
		$boardIDs = array_unique($boardIDs);
		foreach ($boardIDs as $boardID) {
			$boardEditor = new BoardEditor(BoardCache::getInstance()->getBoard($boardID));
			$boardEditor->rebuildStats();
			$boardEditor->updateLastPost();
		}
		
		// reset board cache
		BoardEditor::resetCache();
		
		// unmark clipboard items
		$this->unmarkItems($this->objectIDs);
		
		return array(
			'threadID' => $thread->threadID,
			'redirectURL' => LinkHandler::getInstance()->getLink('Thread', array('application' => 'wbb', 'object' => $thread))
		);
	}
	
	/**
	 * Adds post data.
	 * 
	 * @param	wbb\data\post\Post	$post
	 * @param	string			$key
	 * @param	mixed			$value
	 */
	protected function addPostData(Post $post, $key, $value) {
		if (!isset($this->postData[$post->postID])) {
			$this->postData[$post->postID] = array();
		}
		
		$this->postData[$post->postID][$key] = $value;
	}
	
	/**
	 * Adds thread data.
	 * 
	 * @param	integer		$threadID
	 * @param	string		$key
	 * @param	mixed		$value
	 */
	protected function addThreadData($threadID, $key, $value) {
		if (!isset($this->threadData[$threadID])) {
			$this->threadData[$threadID] = array();
		}
		
		$this->threadData[$threadID][$key] = $value;
	}
	
	/**
	 * Returns post data.
	 * 
	 * @return	array<array>
	 */
	protected function getPostData() {
		return array(
			'postData' => $this->postData,
			'threadData' => $this->threadData
		);
	}
	
	/**
	 * Removes moderated content entries for given post ids.
	 * 
	 * @param	array<integer>		$postIDs
	 */
	protected function removeModeratedContent(array $postIDs) {
		ModerationQueueActivationManager::getInstance()->removeModeratedContent('com.woltlab.wbb.post', $postIDs);
	}
	
	/**
	 * Unmarks posts.
	 * 
	 * @param	array<integer>	$objectIDs
	 */
	protected function unmarkItems(array $objectIDs = array()) {
		if (empty($objectIDs)) {
			foreach ($this->objects as $post) {
				$objectIDs[] = $post->postID;
			}
		}
		
		if (!empty($objectIDs)) {
			ClipboardHandler::getInstance()->unmark($objectIDs, ClipboardHandler::getInstance()->getObjectTypeID('com.woltlab.wbb.post'));
		}
	}
	
	/**
	 * Updates last post for given board ids.
	 * 
	 * @param	array<integer>		$boardIDs
	 */
	protected function updateLastPost(array $boardIDs) {
		if (empty($boardIDs)) {
			return;
		}
		
		$boardIDs = array_unique($boardIDs);
		foreach ($boardIDs as $boardID) {
			$boardEditor = new BoardEditor(new Board($boardID));
			$boardEditor->updateLastPost();
		}
		
		BoardEditor::resetDataCache();
	}
	
	/**
	 * Returns detailed post statistics for the threads with the given ids.
	 * 
	 * @param	array<integer>		$threadIDs
	 * @return	array
	 */
	protected function getDetailedPostStats(array $threadIDs) {
		if (empty($threadIDs)) {
			return array();
		}
		
		$conditionBuilder = new PreparedStatementConditionBuilder();
		$conditionBuilder->add('threadID IN (?)', array($threadIDs));
		
		$sql = "SELECT		threadID,
					COUNT(*) AS posts,
					SUM(CASE WHEN isDeleted = 0 AND isDisabled = 0 THEN 1 ELSE 0 END) AS visiblePosts,
					SUM(CASE WHEN isDeleted = 1 THEN 1 ELSE 0 END) AS deletedPosts,
					SUM(CASE WHEN isDisabled = 1 THEN 1 ELSE 0 END) AS disabledPosts
			FROM		wbb".WCF_N."_post
			".$conditionBuilder."
			GROUP BY	threadID";
		$statement = WCF::getDB()->prepareStatement($sql);
		$statement->execute($conditionBuilder->getParameters());
		
		$postStats = array();
		while ($row = $statement->fetchArray()) {
			$postStats[$row['threadID']] = array(
				'deletedPosts' => $row['deletedPosts'],
				'disabledPosts' => $row['disabledPosts'],
				'posts' => $row['posts'],
				'visiblePosts' => $row['visiblePosts']
			);
		}
		
		return $postStats;
	}
}
