<?php
namespace wbb\data\post;
use wbb\data\modification\log\ThreadModificationLogList;
use wbb\data\thread\Thread;
use wcf\data\attachment\GroupedAttachmentList;
use wcf\system\bbcode\AttachmentBBCode;
use wcf\system\poll\PollManager;

/**
 * Represents a list of viewable posts.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	data.posts
 * @category	Burning Board
 */
class ViewablePostList extends PostList {
	/**
	 * @see	wcf\data\DatabaseObjectList::$sqlOrderBy
	 */
	public $sqlOrderBy = 'post.time';
	
	/**
	 * @see	wcf\data\DatabaseObjectList::$decoratorClassName
	 */
	public $decoratorClassName = 'wbb\data\post\ViewablePost';
	
	/**
	 * attachment object ids
	 * @var	array<integer>
	 */
	public $attachmentObjectIDs = array();
	
	/**
	 * poll ids
	 * @var	array<integer>
	 */
	public $pollIDs = array();
	
	/**
	 * attachment list
	 * @var	wcf\data\attachment\GroupedAttachmentList
	 */
	protected $attachmentList = null;
	
	/**
	 * thread modification log list
	 * @var	wbb\data\modification\log\ThreadModificationLogList
	 */
	public $logList = null;
	
	/**
	 * max post time
	 * @var	integer
	 */
	protected $maxPostTime = 0;
	
	/**
	 * thread object
	 * @var	wbb\data\thread\Thread
	 */
	public $thread = null;
	
	/**
	 * Creates a new ViewablePostList object.
	 */
	public function __construct() {
		parent::__construct();
		
		$this->sqlSelects .= "user_avatar.*, user_option_value.*, user_table.*";
		$this->sqlJoins .= " LEFT JOIN wcf".WCF_N."_user user_table ON (user_table.userID = post.userID)";
		$this->sqlJoins .= " LEFT JOIN wcf".WCF_N."_user_avatar user_avatar ON (user_avatar.avatarID = user_table.avatarID)";
		$this->sqlJoins .= " LEFT JOIN wcf".WCF_N."_user_option_value user_option_value ON (user_option_value.userID = user_table.userID)";
		
		if (MODULE_USER_RANK) {
			$this->sqlSelects .= ",user_rank.*";
			$this->sqlJoins .= " LEFT JOIN wcf".WCF_N."_user_rank user_rank ON (user_rank.rankID = user_table.rankID)";
		}
	}
	
	/**
	 * @see	wcf\data\DatabaseObjectList::readObjects()
	 */
	public function readObjects() {
		if ($this->objectIDs === null) $this->readObjectIDs();
		parent::readObjects();
		
		$postIDs = array();
		foreach ($this->objects as $post) {
			if ($post->time > $this->maxPostTime) $this->maxPostTime = $post->time;
			if ($this->thread !== null) $post->setThread($this->thread);
			
			if ($post->attachments) {
				$this->attachmentObjectIDs[] = $post->postID;
			}
			if ($post->pollID) {
				$this->pollIDs[] = $post->pollID;
			}
			
			if ($this->thread !== null && !$this->thread->isDeleted && $post->isDeleted) {
				$postIDs[] = $post->postID;
			}
		}
		unset($post);
		
		if (!empty($postIDs)) {
			$this->logList = new ThreadModificationLogList();
			$this->logList->setPostData($postIDs, 'trash');
			$this->logList->readObjects();
			
			foreach ($this->logList as $logEntry) {
				foreach ($this->objects as $post) {
					if ($post->postID == $logEntry->objectID) {
						$post->setLogEntry($logEntry);
					}
				}
			}
		}
		
		$this->readAttachments();
		$this->readPolls();
	}
	
	/**
	 * Reads the attachments of the posts in the list.
	 */
	public function readAttachments() {
		if (MODULE_ATTACHMENT && !empty($this->attachmentObjectIDs)) {
			$this->attachmentList = new GroupedAttachmentList('com.woltlab.wbb.post');
			$this->attachmentList->getConditionBuilder()->add('attachment.objectID IN (?)', array($this->attachmentObjectIDs));
			$this->attachmentList->readObjects();
			
			// set embedded attachments
			AttachmentBBCode::setAttachmentList($this->attachmentList);
		}
	}
	
	/**
	 * Reads the polls of the posts in the list.
	 */
	public function readPolls() {
		if (MODULE_POLL && !empty($this->pollIDs)) {
			$polls = PollManager::getInstance()->getPolls($this->pollIDs);
			
			foreach ($polls as $poll) {
				if (isset($this->objects[$poll->objectID])) {
					$this->objects[$poll->objectID]->setPoll($poll);
				}
			}
		}
	}
	
	/**
	 * Returns the max post time
	 * 
	 * @return	integer
	 */
	public function getMaxPostTime() {
		return $this->maxPostTime;
	}
	
	/**
	 * Returns the list of attachments
	 * 
	 * @return	wcf\data\attachment\GroupedAttachmentList
	 */
	public function getAttachmentList() {
		return $this->attachmentList;
	}
	
	/**
	 * Sets active thread.
	 * 
	 * @param	wbb\data\thread\Thread		$thread
	 */
	public function setThread(Thread $thread) {
		$this->thread = $thread;
	}
	
	/**
	 * Returns thread modification log list.
	 * 
	 * @return	wbb\data\modification\log\ThreadModificationLogList
	 */
	public function getLogList() {
		return $this->logList;
	}
}
