<?php
namespace wbb\data\thread;
use wbb\data\board\Board;
use wbb\data\board\BoardCache;
use wbb\data\post\Post;
use wbb\data\WBBDatabaseObject;
use wcf\data\object\type\ObjectTypeCache;
use wcf\data\user\object\watch\UserObjectWatch;
use wcf\data\IUserContent;
use wcf\system\breadcrumb\Breadcrumb;
use wcf\system\breadcrumb\IBreadcrumbProvider;
use wcf\system\database\util\PreparedStatementConditionBuilder;
use wcf\system\request\IRouteController;
use wcf\system\request\LinkHandler;
use wcf\system\WCF;

/**
 * Represents a thread.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	data.thread
 * @category	Burning Board
 */
class Thread extends WBBDatabaseObject implements IBreadcrumbProvider, IRouteController, IUserContent {
	/**
	 * @see	wcf\data\DatabaseObject::$databaseTableName
	 */
	protected static $databaseTableName = 'thread';
	
	/**
	 * @see	wcf\data\DatabaseObject::$databaseTableIndexName
	 */
	protected static $databaseTableIndexName = 'threadID';
	
	/**
	 * true, if the active user has subscribed this topic
	 * @var	boolean
	 */
	protected $subscribed = null;
	
	/**
	 * board object
	 * @var	wbb\data\board\Board
	 */
	public $board = null;
	
	/**
	 * original thread
	 * @var	wbb\data\thread\Thread
	 */
	public $movedThread = null;
	
	// thread types (internal use)
	const TYPE_DEFAULT = 0;
	const TYPE_STICKY = 1;
	const TYPE_ANNOUNCEMENT = 2;
	
	/**
	 * @see	wcf\system\breadcrumb\IBreadcrumbProvider::getBreadcrumb()
	 */
	public function getBreadcrumb() {
		return new Breadcrumb($this->getTitle(), $this->getLink());
	}
	
	/**
	 * Returns true if the active user has the permission to read this thread.
	 * 
	 * @return	boolean
	 */
	public function canRead() {
		$board = $this->getBoard();
		
		if (!$board->canEnter()) {
			return false;
		}
		
		if (!$board->getPermission('canReadThread') || ($this->isDeleted && !$board->getModeratorPermission('canReadDeletedThread')) || ($this->isDisabled && !$board->getModeratorPermission('canEnableThread'))) {
			return false;
		}
		
		return true;
	}
	
	/**
	 * Returns true if the active user can reply this thread.
	 * 
	 * @return	boolean
	 */
	public function canReply() {
		$board = $this->getBoard();
		
		// check permission
		$canReply = (!$board->isClosed && (($this->isClosed && $board->getModeratorPermission('canReplyClosedThread')) 
			|| (!$this->isClosed && ($board->getPermission('canReplyThread') || ($this->userID && $this->userID == WCF::getUser()->userID && $board->getPermission('canReplyOwnThread'))))));
		if (!$canReply) return false;
		
		// check double posts
		if (WCF::getSession()->getPermission('user.board.doublePostLock') != 0 && WCF::getUser()->userID && WCF::getUser()->userID == $this->lastPosterID) {
			if (WCF::getSession()->getPermission('user.board.doublePostLock') == -1 || $this->lastPostTime >= TIME_NOW - WCF::getSession()->getPermission('user.board.doublePostLock') * 60) {
				return false;
			}
		}
		
		return true;
	}
	
	/**
	 * Returns true if the active user can edit the given post.
	 * 
	 * @param	wbb\data\post\Post	$post
	 * @return	boolean
	 */
	public function canEditPost(Post $post) {
		if ($post->threadID != $this->threadID) return false;
		
		// get board
		$board = $this->getBoard();
		
		// check permissions
		$isModerator = $board->getModeratorPermission('canEditPost') || $board->getModeratorPermission('canDeletePost');
		$isAuthor = $post->userID && $post->userID == WCF::getUser()->userID;
		
		$canEditPost = $board->getModeratorPermission('canEditPost') || $isAuthor && $board->getPermission('canEditOwnPost');
		$canDeletePost = $board->getModeratorPermission('canDeletePost') || $isAuthor && $board->getPermission('canDeleteOwnPost');
		
		if ((!$canEditPost && !$canDeletePost) || (!$isModerator && ($board->isClosed || $this->isClosed || $post->isClosed))) {
			return false;
		}
		
		// check post edit timeout 
		if (!$isModerator && WCF::getSession()->getPermission('user.board.postEditTimeout') != -1 && TIME_NOW - $post->time > WCF::getSession()->getPermission('user.board.postEditTimeout') * 60) {
			return false;
		}
		
		return true;
	}
	
	/**
	 * Returns true if active user can mark this thread as done or undone.
	 * 
	 * @return	boolean
	 */
	public function canMarkAsDone() {
		if (!WBB_MODULE_THREAD_MARKING_AS_DONE) {
			return false;
		}
		
		// get board
		$board = $this->getBoard();
		
		// check configuration
		if (!$board->enableMarkingAsDone) {
			return false;
		}
		
		// check permissions
		if ($board->getModeratorPermission('canMarkAsDoneThread')) {
			return true;
		}
		
		if (($this->userID == WCF::getUser()->userID) && $board->getPermission('canMarkAsDoneOwnThread')) {
			return true;
		}
		
		return false;
	}
	
	/**
	 * Returns true if current user can edit this thread, effectively edit the first post.
	 * 
	 * @return	boolean
	 */
	public function canEdit() {
		if (!$this->threadID) return false;
		
		// get board
		$board = $this->getBoard();
		
		// check permissions
		$isModerator = $board->getModeratorPermission('canEditPost') || $board->getModeratorPermission('canDeletePost');
		$isAuthor = $this->userID && $this->userID == WCF::getUser()->userID;
		
		$canEditPost = $board->getModeratorPermission('canEditPost') || $isAuthor && $board->getPermission('canEditOwnPost');
		$canDeletePost = $board->getModeratorPermission('canDeletePost') || $isAuthor && $board->getPermission('canDeleteOwnPost');
		
		if ((!$canEditPost && !$canDeletePost) || (!$isModerator && ($board->isClosed || $this->isClosed))) {
			return false;
		}
		
		return true;
	}
	
	/**
	 * @see	wcf\system\request\IRouteController::getTitle()
	 */
	public function getTitle() {
		return $this->topic;
	}
	
	/**
	 * Returns associated board object.
	 * 
	 * @return	wbb\data\board\Board
	 */
	public function getBoard() {
		if ($this->board === null) {
			$this->board = BoardCache::getInstance()->getBoard($this->boardID);
		}
		
		return $this->board;
	}
	
	/**
	 * Sets associated board object.
	 * 
	 * @param	wbb\data\board\Board	$board
	 */
	public function setBoard(Board $board) {
		if ($board->boardID == $this->boardID) {
			$this->board = $board;
		}
	}
	
	/**
	 * Returns moved thread or null if not moved.
	 * 
	 * @return	wbb\data\thread\Thread
	 */
	public function getMovedThread() {
		if ($this->movedThreadID) {
			if ($this->movedThread === null) {
				$this->movedThread = new Thread($this->movedThreadID);
			}
			
			return $this->movedThread;
		}
		
		return null;
	}
	
	/**
	 * Returns a list of board ids for this announcement.
	 * 
	 * @return	array<integer>
	 */
	public function getAnnouncementBoardIDs() {
		$boardIDs = array();
		
		if ($this->isAnnouncement) {
			$sql = "SELECT	boardID
				FROM	wbb".WCF_N."_thread_announcement
				WHERE	threadID = ?";
			$statement = WCF::getDB()->prepareStatement($sql);
			$statement->execute(array($this->threadID));
			while ($row = $statement->fetchArray()) {
				$boardIDs[] = $row['boardID'];
			}
		}
		
		return $boardIDs;
	}
	
	/**
	 * Returns true if the active user has subscribed this topic.
	 * 
	 * @return	boolean
	 */
	public function isSubscribed() {
		if ($this->subscribed === null) {
			$objectType = ObjectTypeCache::getInstance()->getObjectTypeByName('com.woltlab.wcf.user.objectWatch', 'com.woltlab.wbb.thread');
			if (UserObjectWatch::getUserObjectWatch($objectType->objectTypeID, WCF::getUser()->userID, $this->threadID) !== null) $this->subscribed = true;
			else $this->subscribed = false;
		}
		
		return $this->subscribed;
	}
	
	/**
	 * @see	wcf\data\IUserContent::getTime()
	 */
	public function getTime() {
		return $this->time;
	}
	
	/**
	 * @see	wcf\data\IUserContent::getUserID()
	 */
	public function getUserID() {
		return $this->userID;
	}
	
	/**
	 * @see	wcf\data\IUserContent::getUsername()
	 */
	public function getUsername() {
		return $this->username;
	}
	
	/**
	 * @see	wcf\data\ILinkableObject::getLink()
	 */
	public function getLink() {
		return LinkHandler::getInstance()->getLink('Thread', array(
			'application' => 'wbb',
			'object' => $this
		));
	}
	
	/**
	 * Returns the ids of similar threads.
	 * 
	 * @param	string			$topic
	 * @param	array<integer>		$accessibleBoardIDs
	 * @param	integer			$limit
	 * @param	integer			$preferredBoardID
	 * @param	integer			$ignoreThreadID
	 * @param	integer			$languageID
	 * @return	array<integer>
	 */
	public static function getSimilarThreads($topic, array $accessibleBoardIDs, $limit = 5, $preferredBoardID = 0, $ignoreThreadID = 0, $languageID = null) {
		// get object type
		$objectType = ObjectTypeCache::getInstance()->getObjectTypeByName('com.woltlab.wcf.searchableObjectType', 'com.woltlab.wbb.post');
		
		// get post ids
		$threadIDs = array();
		$conditionBuilder = new PreparedStatementConditionBuilder();
		$conditionBuilder->add('post.isDisabled = 0 AND post.isDeleted = 0');
		if ($ignoreThreadID) $conditionBuilder->add('post.threadID <> ?', array($ignoreThreadID));
		$conditionBuilder->add('thread.boardID IN (?)', array($accessibleBoardIDs));
		$sql = "SELECT		post.threadID,
					search_index.relevance ".($preferredBoardID ? "+ IF(thread.boardID=".$preferredBoardID.",2,0)" : '')." AS relevance
			FROM		wbb".WCF_N."_post post
			INNER JOIN	(
						SELECT		objectID,
								MATCH (subject, message, metaData) AGAINST ('".escapeString($topic)."') + (5 / (1 + POW(LN(1 + (".TIME_NOW." - time) / 2592000), 2))) AS relevance
						FROM		wcf".WCF_N."_search_index
						WHERE		MATCH (subject, message, metaData) AGAINST ('".escapeString($topic)."' IN BOOLEAN MODE)
								AND objectTypeID = ".$objectType->objectTypeID."
								AND time > ".(TIME_NOW - WBB_THREAD_SIMILAR_THREADS_SEARCH_PERIOD * 86400)."
								".($languageID ? "AND languageID = ".$languageID : '')."
						ORDER BY	relevance DESC
						LIMIT		1000
					) search_index
			ON		(post.postID = search_index.objectID)
			LEFT JOIN	wbb".WCF_N."_thread thread
			ON		(thread.threadID = post.threadID)
			".$conditionBuilder."
			ORDER BY	relevance DESC";
		$statement = WCF::getDB()->prepareStatement($sql, $limit);
		$statement->execute($conditionBuilder->getParameters());
		while ($row = $statement->fetchArray()) {
			if (!in_array($row['threadID'], $threadIDs)) $threadIDs[] = $row['threadID'];
		}
		
		return $threadIDs;
	}
}
