<?php
namespace wbb\data\thread;
use wbb\data\board\BoardCache;
use wbb\data\modification\log\ViewableBoardThreadModificationLog;
use wbb\data\post\Post;
use wbb\data\post\ThreadPostList;
use wcf\data\label\Label;
use wcf\data\user\User;
use wcf\data\user\UserProfile;
use wcf\data\DatabaseObjectDecorator;
use wcf\system\language\LanguageFactory;
use wcf\system\visitTracker\VisitTracker;
use wcf\system\WCF;

/**
 * Represents a viewable thread.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	data.thread
 * @category	Burning Board
 */
class ViewableThread extends DatabaseObjectDecorator {
	/**
	 * @see	wcf\data\DatabaseObjectDecorator::$baseClass
	 */
	protected static $baseClass = 'wbb\data\thread\Thread';
	
	/**
	 * user profile object
	 * @var	wcf\data\user\UserProfile
	 */
	protected $userProfile = null;
	
	/**
	 * last poster's profile
	 * @var	wcf\data\user\UserProfile
	 */
	protected $lastPosterProfile = null;
	
	/**
	 * list of assigned labels
	 * @var	array<wcf\data\label\Label>
	 */
	protected $labels = array();
	
	/**
	 * modification log entry
	 * @var	wbb\data\modification\log\ViewableThreadModificationLog
	 */
	protected $logEntry = null;
	
	/**
	 * effective visit time
	 * @var	integer
	 */
	protected $effectiveVisitTime = null;
	
	/**
	 * Returns the user profile object.
	 * 
	 * @return	wcf\data\user\UserProfile
	 */
	public function getUserProfile() {
		if ($this->userProfile === null) {
			$this->userProfile = new UserProfile(new User(null, $this->getDecoratedObject()->data));
		}
		
		return $this->userProfile;
	}
	
	/**
	 * Returns the user profile object.
	 * 
	 * @return	wcf\data\user\UserProfile
	 */
	public function getLastPosterProfile() {
		if ($this->lastPosterProfile === null) {
			if ($this->userID && $this->userID == $this->lastPosterID) {
				$this->lastPosterProfile = $this->getUserProfile();
			}
			else {
				$this->lastPosterProfile = new UserProfile(new User(null, array(
					'userID' => $this->lastPosterID,
					'username' => $this->lastPoster,
					'avatarID' => $this->lastPosterAvatarID,
					'avatarName' => $this->lastPosterAvatarName,
					'avatarExtension' => $this->lastPosterAvatarExtension,
					'width' => $this->lastPosterAvatarWidth,
					'height' => $this->lastPosterAvatarHeight,
					'email' => $this->lastPosterEmail,
					'disableAvatar' => $this->lastPosterDisableAvatar,
					'enableGravatar' => $this->lastPosterEnableGravatar,
					'fileHash' => $this->lastPosterAvatarFileHash
				)));
			}
		}
		
		return $this->lastPosterProfile;
	}
	
	/**
	 * Returns the effective visit time.
	 * 
	 * @return	integer
	 */
	public function getVisitTime() {
		if ($this->effectiveVisitTime === null) {
			if (WCF::getUser()->userID) {
				$this->effectiveVisitTime = max($this->visitTime, $this->boardVisitTime, VisitTracker::getInstance()->getVisitTime('com.woltlab.wbb.thread'));
			}
			else {
				$this->effectiveVisitTime = max(VisitTracker::getInstance()->getObjectVisitTime('com.woltlab.wbb.thread', $this->threadID), VisitTracker::getInstance()->getObjectVisitTime('com.woltlab.wbb.board', $this->boardID), VisitTracker::getInstance()->getVisitTime('com.woltlab.wbb.thread'));
			}
			if ($this->effectiveVisitTime === null) {
				$this->effectiveVisitTime = 0;
			}
		}
		
		return $this->effectiveVisitTime;
	}
	
	/**
	 * Returns true if this thread is new for the active user.
	 * 
	 * @return	boolean
	 */
	public function isNew() {
		if (!$this->movedThreadID) {
			if ($this->lastPostTime > $this->getVisitTime()) {
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Returns true if the active user doesn't have read the given post.
	 * 
	 * @param	wbb\data\post\Post	$post
	 * @return	boolean
	 */
	public function isNewPost(Post $post) {
		return ($post->time > $this->getVisitTime());
	}
	
	/**
	 * Returns the number of pages in this thread.
	 * 
	 * @return	integer
	 */
	public function getPages() {
		if (WCF::getUser()->postsPerPage) {
			$postsPerPage = WCF::getUser()->postsPerPage;
		}
		else if (BoardCache::getInstance()->getBoard($this->boardID)->postsPerPage) {
			$postsPerPage = BoardCache::getInstance()->getBoard($this->boardID)->postsPerPage;
		}
		else {
			$postsPerPage = WBB_THREAD_POSTS_PER_PAGE;
		}
		
		return intval(ceil(($this->replies + 1) / $postsPerPage));
	}
	
	/**
	 * Returns the language of this thread.
	 * 
	 * @return	wcf\data\language\Language
	 */
	public function getLanguage() {
		if ($this->languageID) return LanguageFactory::getInstance()->getLanguage($this->languageID);
		
		return null;
	}
	
	/**
	 * Returns the flag icon for the thread language.
	 * 
	 * @return	string
	 */
	public function getLanguageIcon() {
		return '<img src="'.$this->getLanguage()->getIconPath().'" alt="" title="'.$this->getLanguage().'" class="jsTooltip iconFlag" />';
	}
	
	/**
	 * Returns the viewable thread object with the given thread id.
	 * 
	 * @param	integer		$threadID
	 * @return	wbb\data\thread\ViewableThread
	 */
	public static function getThread($threadID) {
		$list = new ViewableThreadList();
		$list->setObjectIDs(array($threadID));
		$list->readObjects();
		$objects = $list->getObjects();
		if (isset($objects[$threadID])) return $objects[$threadID];
		return null;
	}
	
	/**
	 * Adds a label.
	 * 
	 * @param	wcf\data\label\Label	$label
	 */
	public function addLabel(Label $label) {
		$this->labels[$label->labelID] = $label;
	}
	
	/**
	 * Returns a list of labels.
	 * 
	 * @return	array<wcf\data\label\Label>
	 */
	public function getLabels() {
		return $this->labels;
	}
	
	/**
	 * Returns the primary (first) label.
	 * 
	 * @return	wcf\data\label\Label
	 */
	public function getPrimaryLabel() {
		if (!$this->hasLabels()) return null;
		
		foreach ($this->labels as $label) return $label;
	}
	
	/**
	 * Returns true if one or more labels are assigned to this thread.
	 * 
	 * @return	boolean
	 */
	public function hasLabels() {
		return (count($this->labels)) ? true : false;
	}
	
	/**
	 * Sets modification log entry.
	 * 
	 * @param	wbb\data\modification\log\ViewableBoardThreadModificationLog	$logEntry
	 */
	public function setLogEntry(ViewableBoardThreadModificationLog $logEntry) {
		$this->logEntry = $logEntry;
	}
	
	/**
	 * Returns delete note if applicable.
	 * 
	 * @return	string
	 */
	public function getDeleteNote() {
		if ($this->logEntry === null || $this->logEntry->action != 'trash') {
			return '';
		}
		
		return $this->logEntry->__toString();
	}
	
	/**
	 * @see	wbb\data\thread\Thread::isSubscribed()
	 */
	public function isSubscribed() {
		return ($this->watchID ? 1 : 0);
	}
	
	/**
	 * Returns last post time for current thread. Please be aware that it might
	 * not return the actual last post time if you're not on the last page!
	 * 
	 * @param	wcf\data\post\ThreadPostList		$postList
	 * @return	integer
	 */
	public function getLastPostTime(ThreadPostList $postList) {
		$lastPostTime = $this->lastPostTime;
		foreach ($postList as $post) {
			if ($post->time > $lastPostTime) {
				$lastPostTime = $post->time;
			}
		}
		
		return $lastPostTime;
	}
}
