<?php
namespace wbb\form;
use wbb\data\board\BoardCache;
use wbb\data\post\Post;
use wbb\data\post\PostAction;
use wbb\data\post\PostEditor;
use wbb\data\post\ThreadPostList;
use wbb\data\thread\Thread;
use wbb\data\thread\ThreadAction;
use wbb\data\thread\ThreadEditor;
use wcf\data\user\object\watch\UserObjectWatchAction;
use wcf\form\MessageForm;
use wcf\system\exception\IllegalLinkException;
use wcf\system\exception\PermissionDeniedException;
use wcf\system\message\quote\MessageQuoteManager;
use wcf\system\message\QuickReplyManager;
use wcf\system\poll\PollManager;
use wcf\system\request\LinkHandler;
use wcf\system\user\notification\UserNotificationHandler;
use wcf\system\WCF;
use wcf\util\HeaderUtil;

/**
 * Shows the reply form.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	form
 * @category	Burning Board
 */
class PostAddForm extends ThreadAddForm {
	/**
	 * disable post after posting
	 * @var	boolean
	 */
	public $disablePost = false;
	
	/**
	 * mark thread as done after posting
	 * @var	boolean
	 */
	public $markAsDone = false;
	
	/**
	 * thread id
	 * @var	integer
	 */
	public $threadID = 0;
	
	/**
	 * thread object
	 * @var	wbb\data\thread\Thread
	 */
	public $thread = null;
	
	/**
	 * post list
	 * @var	wbb\data\post\ViewablePostList
	 */
	public $postList = null;
	
	/**
	 * @see	wbb\form\ThreadAddForm::$minCharLength
	 */
	public $minCharLength = WBB_POST_MIN_CHAR_LENGTH;
	
	/**
	 * @see	wbb\form\ThreadAddForm::$minWordCount
	 */
	public $minWordCount = WBB_POST_MIN_WORD_COUNT;
	
	/**
	 * @see wcf\page\IPage::readParameters()
	 */
	public function readParameters() {
		MessageForm::readParameters();
		
		if (isset($_REQUEST['id'])) $this->threadID = intval($_REQUEST['id']);
		$this->thread = new Thread($this->threadID);
		if (!$this->thread->threadID) {
			throw new IllegalLinkException();
		}
		$this->board = BoardCache::getInstance()->getBoard($this->thread->boardID);
		
		// check permission
		$this->board->checkPermission(array('canViewBoard', 'canEnterBoard'));
		if (!$this->thread->canReply()) {
			throw new PermissionDeniedException();
		}
		
		// check flood control
		Post::enforceFloodControl();
		
		// set attachment parent id
		$this->attachmentParentObjectID = $this->board->boardID;
		
		// quotes
		MessageQuoteManager::getInstance()->readParameters();
		
		// polls
		if ($this->canUsePoll()) PollManager::getInstance()->setObject('com.woltlab.wbb.post', 0);
		
		// init board style
		$this->board->initStyle();
		
		// get max text length
		$this->maxTextLength = WCF::getSession()->getPermission('user.board.maxPostLength');
	}
	
	/**
	 * @see	wcf\form\IForm::readFormParameters()
	 */
	public function readFormParameters() {
		parent::readFormParameters();
		
		// settings
		if (isset($_POST['disablePost']) && $this->board->getModeratorPermission('canEnablePost')) $this->disablePost = true;
		if (isset($_POST['markAsDone']) && $this->thread->canMarkAsDone()) $this->markAsDone = true;
		
		// quotes
		MessageQuoteManager::getInstance()->readFormParameters();
	}
	
	/**
	 * @see	wcf\form\MessageForm::validateSubject()
	 */
	protected function validateSubject() {}
	
	/**
	 * @see	wcf\form\IForm::save()
	 */
	public function save() {
		MessageForm::save();
		
		// save post
		$data = array(
			'threadID' => $this->threadID,
			'subject' => $this->subject,
			'message' => $this->text,
			'time' => TIME_NOW,
			'userID' => (WCF::getUser()->userID ?: null),
			'username' => (WCF::getUser()->userID ? WCF::getUser()->username : $this->username),
			'enableBBCodes' => $this->enableBBCodes,
			'enableHtml' => $this->enableHtml,
			'enableSmilies' => $this->enableSmilies,
			'showSignature' => $this->showSignature
		);
		if ($this->disablePost || !$this->board->getPermission('canReplyThreadWithoutModeration')) $data['isDisabled'] = 1;
		
		$this->objectAction = new PostAction(array(), 'create', array(
			'data' => $data,
			'thread' => $this->thread,
			'board' => $this->board,
			'attachmentHandler' => $this->attachmentHandler,
			'subscribeThread' => $this->subscribeThread
		));
		$resultValues = $this->objectAction->executeAction();
		
		// thread options
		if ($this->closeThread) {
			$threadAction = new ThreadAction(array($this->thread), 'close');
			$threadAction->executeAction();
		}
		if ($this->markAsDone) {
			$threadAction = new ThreadAction(array($this->thread), 'markAsDone');
			$threadAction->executeAction();
		}
		
		// quotes
		MessageQuoteManager::getInstance()->saved();
		
		// save polls
		if ($this->canUsePoll()) {
			$addedPoll = false;
			$pollID = PollManager::getInstance()->save($resultValues['returnValues']->postID);
			if ($pollID) {
				$postEditor = new PostEditor($resultValues['returnValues']);
				$postEditor->update(array(
					'pollID' => $pollID
				));
				
				$addedPoll = true;
			}
			
			if ($addedPoll) {
				$threadEditor = new ThreadEditor($this->thread);
				$threadEditor->updateCounters(array(
					'polls' => 1
				));
			}
		}
		
		// remove message from session
		QuickReplyManager::getInstance()->removeMessage('thread', $this->thread->threadID);
		$this->saved();
		
		if ($resultValues['returnValues']->isDisabled && !$this->board->getModeratorPermission('canEnablePost')) {
			HeaderUtil::delayedRedirect(LinkHandler::getInstance()->getLink('Thread', array(
				'application' => 'wbb',
				'object' => $this->thread
			)), WCF::getLanguage()->get('wbb.post.moderation.redirect'), 30);
		}
		else {
			HeaderUtil::redirect(LinkHandler::getInstance()->getLink('Thread', array(
				'application' => 'wbb',
				'object' => $this->thread,
				'postID' => $resultValues['returnValues']->postID
			)).'#post'.$resultValues['returnValues']->postID);
		}
		exit;
	}
	
	/**
	 * @see	wcf\page\IPage::readData()
	 */
	public function readData() {
		parent::readData();
		
		if (empty($_POST)) {
			// check for quick reply message
			$this->text = QuickReplyManager::getInstance()->getMessage('thread', $this->thread->threadID);
			
			if (empty($this->text)) {
				if (MessageQuoteManager::getInstance()->getQuoteMessageID()) {
					$post = new Post(MessageQuoteManager::getInstance()->getQuoteMessageID());
					if (!$post->postID) {
						throw new IllegalLinkException();
					}
					
					if ($post->threadID == $this->thread->threadID) {
						$post->setThread($this->thread);
						if ($post->canRead()) {
							$this->text = MessageQuoteManager::getInstance()->renderQuote($post, $post->message);
						}
					}
				}
				
				if (empty($this->text)) {
					// get all post ids from current thread
					$sql = "SELECT	postID
						FROM	wbb".WCF_N."_post
						WHERE	threadID = ?";
					$statement = WCF::getDB()->prepareStatement($sql);
					$statement->execute(array($this->thread->threadID));
					$postIDs = array();
					while ($row = $statement->fetchArray()) {
						$postIDs[] = $row['postID'];
					}
					
					$renderedQuotes = MessageQuoteManager::getInstance()->getQuotesByObjectIDs('com.woltlab.wbb.post', $postIDs);
					if (!empty($renderedQuotes)) {
						$this->text = implode("\n", $renderedQuotes);
					}
				}
			}
			
			if (!$this->subscribeThread && WCF::getUser()->userID && $this->thread->isSubscribed()) {
				$this->subscribeThread = true;
			}
		}
		
		// add breadcrumbs
		WCF::getBreadcrumbs()->add($this->thread->getBreadcrumb());
		
		// get post list
		$this->postList = new ThreadPostList($this->thread, false);
		$this->postList->sqlLimit = WBB_REPLY_SHOW_POSTS_MAX;
		$this->postList->sqlOrderBy = 'post.time DESC';
		$this->postList->readObjects();
	}
	
	/**
	 * @see	wcf\page\IPage::assignVariables()
	 */
	public function assignVariables() {
		parent::assignVariables();
		
		MessageQuoteManager::getInstance()->assignVariables();
		
		WCF::getTPL()->assign(array(
			'attachmentList' => $this->postList->getAttachmentList(),
			'disablePost' => $this->disablePost,
			'items' => $this->postList->countObjects(),
			'markAsDone' => $this->markAsDone,
			'messageQuoteCount' => MessageQuoteManager::getInstance()->countQuotes(),
			'posts' => $this->postList->getObjects(),
			'thread' => $this->thread,
			'threadID' => $this->threadID
		));
	}
	
	/**
	 * @see	wcf\page\ITrackablePage::getParentObjectType()
	 */
	public function getParentObjectType() {
		return 'com.woltlab.wbb.board';
	}
	
	/**
	 * @see	wcf\page\ITrackablePage::getParentObjectID()
	 */
	public function getParentObjectID() {
		return $this->thread->boardID;
	}
	
	/**
	 * @see	wcf\page\ITrackablePage::getObjectType()
	 */
	public function getObjectType() {
		return 'com.woltlab.wbb.thread';
	}
	
	/**
	 * @see	wcf\page\ITrackablePage::getObjectID()
	 */
	public function getObjectID() {
		return $this->threadID;
	}
	
	/**
	 * @see	wbb\form\ThreadAddForm::canUsePoll()
	 */
	protected function canUsePoll() {
		if (!parent::canUsePoll()) {
			return false;
		}
		
		if (!WCF::getSession()->getPermission('user.board.canStartPollInReply')) {
			return false;
		}
		
		return true;
	}
	
	/**
	 * @see	wbb\form\ThreadAddForm::validateLabelIDs()
	 */
	protected function validateLabelIDs() {
		// does nothing
	}
}
