<?php
namespace wbb\form;
use wbb\data\board\BoardCache;
use wbb\data\board\RestrictedBoardNodeList;
use wbb\data\post\Post;
use wbb\data\post\PostEditor;
use wbb\data\thread\Thread;
use wbb\data\thread\ThreadAction;
use wbb\data\thread\ThreadEditor;
use wbb\system\label\object\ThreadLabelObjectHandler;
use wbb\system\WBBCore;
use wcf\data\user\object\watch\UserObjectWatchAction;
use wcf\form\MessageForm;
use wcf\system\exception\IllegalLinkException;
use wcf\system\exception\PermissionDeniedException;
use wcf\system\exception\UserInputException;
use wcf\system\label\LabelHandler;
use wcf\system\language\LanguageFactory;
use wcf\system\message\quote\MessageQuoteManager;
use wcf\system\poll\PollManager;
use wcf\system\request\LinkHandler;
use wcf\system\user\notification\UserNotificationHandler;
use wcf\system\WCF;
use wcf\util\ArrayUtil;
use wcf\util\HeaderUtil;
use wcf\util\StringUtil;
use wcf\util\UserUtil;

/**
 * Shows the new thread form.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	form
 * @category	Burning Board
 */
class ThreadAddForm extends MessageForm {
	/**
	 * @see	wcf\page\AbstractPage::$activeMenuItem
	 */
	public $activeMenuItem = 'wbb.header.menu.board';
	
	/**
	 * @see	wcf\form\MessageForm::$attachmentObjectType
	 */
	public $attachmentObjectType = 'com.woltlab.wbb.post';
	
	/**
	 * @see	wcf\page\AbstractPage::$enableTracking
	 */
	public $enableTracking = true;
	
	/**
	 * board id
	 * @var	integer
	 */
	public $boardID = 0;
	
	/**
	 * list of board ids for announcements
	 * @var	array<integer>
	 */
	public $boardIDs = array();
	
	/**
	 * board object
	 * @var wbb\data\board\Board
	 */
	public $board = null;
	
	/**
	 * board node list
	 * @var	wbb\data\board\RestrictedBoardNodeList
	 */
	public $boardList = null;
	
	/**
	 * close thread after post
	 * @var	boolean
	 */
	public $closeThread = false;
	
	/**
	 * disable thread after post
	 * @var	boolean
	 */
	public $disableThread = false;
	
	/**
	 * subscribe thread
	 * @var	boolean
	 */
	public $subscribeThread = false;
	
	/**
	 * @see	wcf\form\MessageForm::$enableMultilingualism
	 */
	public $enableMultilingualism = true;
	
	/**
	 * label group list
	 * @var	array<wcf\data\label\group\ViewableLabelGroup>
	 */
	public $labelGroups = null;
	
	/**
	 * list of label ids
	 * @var	array<integer>
	 */
	public $labelIDs = array();
	
	/**
	 * username
	 * @var	string
	 */
	public $username = '';
	
	/**
	 * tags
	 * @var	array
	 */
	public $tags = array();
	
	/**
	 * thread type (0 = default, 1 = sticky, 2 = announcement)
	 * @var	integer
	 */
	public $type = 0;
	
	/**
	 * minimum number of characters in message text
	 * @var integer
	 */
	public $minCharLength = WBB_THREAD_MIN_CHAR_LENGTH;
	
	/**
	 * minimum number of words in message text
	 * @var integer
	 */
	public $minWordCount = WBB_THREAD_MIN_WORD_COUNT;
	
	/**
	 * @see	wcf\page\IPage::readParameters()
	 */
	public function readParameters() {
		parent::readParameters();
		
		if (isset($_REQUEST['id'])) $this->boardID = intval($_REQUEST['id']);
		$this->board = BoardCache::getInstance()->getBoard($this->boardID);
		if ($this->board === null || !$this->board->isBoard() || $this->board->isClosed) {
			throw new IllegalLinkException();
		}
		
		// check permissions
		$this->board->checkPermission(array('canViewBoard', 'canEnterBoard', 'canStartThread'));
		
		// check flood control
		Post::enforceFloodControl();
		
		// set attachment parent id
		$this->attachmentParentObjectID = $this->board->boardID;
		
		// polls
		if ($this->canUsePoll()) PollManager::getInstance()->setObject('com.woltlab.wbb.post', 0);
		
		// init board style
		$this->board->initStyle();
		
		// get max text length
		$this->maxTextLength = WCF::getSession()->getPermission('user.board.maxPostLength');
		
		// quotes
		MessageQuoteManager::getInstance()->readParameters();
		
		// labels
		ThreadLabelObjectHandler::getInstance()->setBoardID($this->board->boardID);
	}
	
	/**
	 * @see	wcf\form\IForm::readFormParameters()
	 */
	public function readFormParameters() {
		parent::readFormParameters();
		
		if (isset($_POST['boardIDs']) && is_array($_POST['boardIDs'])) $this->boardIDs = ArrayUtil::toIntegerArray($_POST['boardIDs']);
		if (isset($_POST['labelIDs']) && is_array($_POST['labelIDs'])) $this->labelIDs = $_POST['labelIDs'];
		if (isset($_POST['tags']) && is_array($_POST['tags'])) $this->tags = ArrayUtil::trim($_POST['tags']);
		if (isset($_POST['type'])) $this->type = intval($_POST['type']);
		if (isset($_POST['username'])) $this->username = StringUtil::trim($_POST['username']);
		
		// settings
		if (isset($_POST['closeThread']) && $this->board->getModeratorPermission('canCloseThread')) $this->closeThread = true;
		if (isset($_POST['disableThread']) && $this->board->getModeratorPermission('canEnableThread')) $this->disableThread = true;
		if (isset($_POST['subscribeThread']) && WCF::getUser()->userID) $this->subscribeThread = true;
		
		// polls
		if (WCF::getSession()->getPermission('user.board.canStartPoll')) PollManager::getInstance()->readFormParameters();
		
		// quotes
		MessageQuoteManager::getInstance()->readFormParameters();
	}
	
	/**
	 * @see	wcf\form\IForm::validate()
	 */
	public function validate() {
		parent::validate();
		
		$this->validateUsername();
		$this->validateLabelIDs();
		$this->validateType();
		
		// polls
		if ($this->canUsePoll())  PollManager::getInstance()->validate();
	}
	
	/**
	 * @see	wcf\form\MessageForm::validateText()
	 */
	protected function validateText() {
		parent::validateText();
		
		if ($this->minCharLength && (mb_strlen($this->text) < $this->minCharLength)) {
			throw new UserInputException('text', 'minCharLength');
		}
		
		if ($this->minWordCount && (count(explode(' ', $this->text)) < $this->minWordCount)) {
			throw new UserInputException('text', 'minWordCount');
		}
	}
	
	/**
	 * Validates the username.
	 */
	protected function validateUsername() {
		// only for guests
		if (WCF::getUser()->userID == 0) {
			if (empty($this->username)) {
				throw new UserInputException('username');
			}
			if (!UserUtil::isValidUsername($this->username)) {
				throw new UserInputException('username', 'notValid');
			}
			if (!UserUtil::isAvailableUsername($this->username)) {
				throw new UserInputException('username', 'notAvailable');
			}
			
			WCF::getSession()->register('username', $this->username);
		}
	}
	
	/**
	 * Validates label ids.
	 */
	protected function validateLabelIDs() {
		if (!ThreadLabelObjectHandler::getInstance()->validateLabelIDs($this->labelIDs, 'canSetLabel')) {
			throw new UserInputException('labelIDs');
		}
	}
	
	/**
	 * Validates thread type.
	 */
	protected function validateType() {
		switch ($this->type) {
			case Thread::TYPE_STICKY:
				// validate permissions
				if (!$this->board->getModeratorPermission('canPinThread')) {
					throw new PermissionDeniedException();
				}
				
				// reset board ids
				$this->boardIDs = array();
			break;
			
			case Thread::TYPE_ANNOUNCEMENT:
				// validate permissions
				if (!$this->board->getModeratorPermission('canStartAnnouncement')) {
					throw new PermissionDeniedException();
				}
				
				// validate board ids
				if (empty($this->boardIDs)) {
					$this->boardIDs = array($this->board->boardID);
				}
				else {
					$accessibleBoardIDs = $this->board->getAccessibleBoardIDs();
					foreach ($this->boardIDs as $boardID) {
						if (!in_array($boardID, $accessibleBoardIDs)) {
							throw new PermissionDeniedException();
						}
					}
				}
			break;
			
			default:
			case Thread::TYPE_DEFAULT:
				$this->type = 0;
				
				// reset board ids
				$this->boardIDs = array();
			break;
		}
	}
	
	/**
	 * @see	wcf\form\IForm::save()
	 */
	public function save() {
		parent::save();
		
		// save thread
		$data = array(
			'boardID' => $this->boardID,
			'languageID' => $this->languageID,
			'topic' => $this->subject,
			'time' => TIME_NOW,
			'userID' => (WCF::getUser()->userID ?: null),
			'username' => (WCF::getUser()->userID ? WCF::getUser()->username : $this->username),
			'hasLabels' => (!empty($this->labelIDs) ? 1 : 0)
		);
		if ($this->closeThread) $data['isClosed'] = 1;
		if ($this->disableThread || !$this->board->getPermission('canStartThreadWithoutModeration')) $data['isDisabled'] = 1;
		
		$threadData = array(
			'data' => $data,
			'board' => $this->board,
			'attachmentHandler' => $this->attachmentHandler,
			'postData' => array(
				'message' => $this->text,
				'enableBBCodes' => $this->enableBBCodes,
				'enableHtml' => $this->enableHtml,
				'enableSmilies' => $this->enableSmilies,
				'showSignature' => $this->showSignature
			),
			'tags' => array(),
			'subscribeThread' => $this->subscribeThread
		);
		
		// handle thread type
		switch ($this->type) {
			case Thread::TYPE_STICKY:
				$threadData['data']['isSticky'] = 1;
			break;
			
			case Thread::TYPE_ANNOUNCEMENT:
				$threadData['data']['isAnnouncement'] = 1;
				$threadData['announcementBoardIDs'] = $this->boardIDs;
			break;
		}
		
		if (MODULE_TAGGING && WBB_THREAD_ENABLE_TAGS && $this->board->getPermission('canSetTags')) {
			$threadData['tags'] = $this->tags;
		}
		$this->objectAction = new ThreadAction(array(), 'create', $threadData);
		$resultValues = $this->objectAction->executeAction();
		
		// save labels
		if (!empty($this->labelIDs)) {
			ThreadLabelObjectHandler::getInstance()->setLabels($this->labelIDs, $resultValues['returnValues']->threadID);
		}
		
		// save polls
		if ($this->canUsePoll()) {
			$thread = new Thread($resultValues['returnValues']->threadID);
			$pollID = PollManager::getInstance()->save($thread->firstPostID);
			$addedPoll = false;
			if ($pollID) {
				$postEditor = new PostEditor(new Post($thread->firstPostID));
				$postEditor->update(array(
					'pollID' => $pollID
				));
				
				$addedPoll = true;
			}
			
			if ($addedPoll) {
				$threadEditor = new ThreadEditor($thread);
				$threadEditor->updateCounters(array(
					'polls' => 1
				));
			}
		}
		
		MessageQuoteManager::getInstance()->saved();
		
		$this->saved();
		
		if ($resultValues['returnValues']->isDisabled && !$this->board->getModeratorPermission('canEnableThread')) {
			HeaderUtil::delayedRedirect(LinkHandler::getInstance()->getLink('Board', array(
				'application' => 'wbb',
				'object' => $this->board
			)), WCF::getLanguage()->get('wbb.thread.moderation.redirect'), 30);
		}
		else {
			HeaderUtil::redirect(LinkHandler::getInstance()->getLink('Thread', array(
				'application' => 'wbb',
				'object' => $resultValues['returnValues']
			)));
		}
		exit;
	}
	
	/**
	 * @see	wcf\page\IPage::readData()
	 */
	public function readData() {
		parent::readData();
		
		$this->boardList = new RestrictedBoardNodeList();
		$this->boardList->readNodeTree();
		$labelGroupIDs = BoardCache::getInstance()->getLabelGroups($this->board->boardID);
		if (!empty($labelGroupIDs)) {
			$this->labelGroups = LabelHandler::getInstance()->getLabelGroups($labelGroupIDs);
		}
		
		if (empty($_POST)) {
			// default values
			$this->username = WCF::getSession()->getVar('username');
			if (WCF::getUser()->watchThreadOnReply) $this->subscribeThread = true;
			
			// multilingualism
			if (!empty($this->availableContentLanguages)) {
				if (!$this->languageID) {
					$language = LanguageFactory::getInstance()->getUserLanguage();
					$this->languageID = $language->languageID;
				}
				
				if (!isset($this->availableContentLanguages[$this->languageID])) {
					$languageIDs = array_keys($this->availableContentLanguages);
					$this->languageID = array_shift($languageIDs);
				}
			}
		}
		
		// add breadcrumbs
		WBBCore::getInstance()->setBreadcrumbs($this->board->getParentBoards(), $this->board);
	}
	
	/**
	 * @see	wcf\page\IPage::assignVariables()
	 */
	public function assignVariables() {
		parent::assignVariables();
		
		if ($this->canUsePoll()) PollManager::getInstance()->assignVariables();
		MessageQuoteManager::getInstance()->assignVariables();
		
		WCF::getTPL()->assign(array(
			'boardID' => $this->boardID,
			'boardIDs' => $this->boardIDs,
			'board' => $this->board,
			'boardNodeList' => $this->boardList->getNodeList(),
			'closeThread' => $this->closeThread,
			'disableThread' => $this->disableThread,
			'subscribeThread' => $this->subscribeThread,
			'labelGroups' => $this->labelGroups,
			'labelIDs' => $this->labelIDs,
			'username' => $this->username,
			'tags' => $this->tags,
			'type' => $this->type,
			'minCharLength' => $this->minCharLength,
			'minWordCount' => $this->minWordCount
		));
	}
	
	/**
	 * @see	wcf\page\ITrackablePage::getObjectType()
	 */
	public function getObjectType() {
		return 'com.woltlab.wbb.board';
	}
	
	/**
	 * @see	wcf\page\ITrackablePage::getObjectID()
	 */
	public function getObjectID() {
		return $this->boardID;
	}
	
	/**
	 * Returns true if current user may use polls.
	 * 
	 * @return	boolean
	 */
	protected function canUsePoll() {
		if (!MODULE_POLL || !WCF::getSession()->getPermission('user.board.canStartPoll')) {
			return false;
		}
		
		return true;
	}
}
