<?php
namespace wbb\page;
use wcf\data\user\group\UserGroup;

use wbb\data\board\BoardCache;
use wbb\data\board\BoardEditor;
use wbb\data\board\BoardTagCloud;
use wbb\data\board\DetailedBoardNodeList;
use wbb\data\thread\BoardThreadList;
use wbb\system\WBBCore;
use wcf\data\user\online\UsersOnlineList;
use wcf\data\user\UserProfile;
use wcf\page\SortablePage;
use wcf\system\clipboard\ClipboardHandler;
use wcf\system\dashboard\DashboardHandler;
use wcf\system\exception\IllegalLinkException;
use wcf\system\exception\PermissionDeniedException;
use wcf\system\label\LabelHandler;
use wcf\system\language\LanguageFactory;
use wcf\system\user\collapsible\content\UserCollapsibleContentHandler;
use wcf\system\WCF;
use wcf\util\HeaderUtil;
use wcf\util\StringUtil;

/**
 * Shows the board page.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	page
 * @category	Burning Board
 */
class BoardPage extends SortablePage {
	/**
	 * @see	wcf\page\AbstractPage::$activeMenuItem
	 */
	public $activeMenuItem = 'wbb.header.menu.board';
	
	/**
	 * @see	wcf\page\SortablePage::$defaultSortField
	 */
	public $defaultSortField = WBB_BOARD_DEFAULT_SORT_FIELD;
	
	/**
	 * @see	wcf\page\SortablePage::$defaultSortOrder
	 */
	public $defaultSortOrder = WBB_BOARD_DEFAULT_SORT_ORDER;
	
	/**
	 * @see	wcf\page\SortablePage::$validSortFields
	 */
	public $validSortFields = array('topic', 'username', 'time', 'views', 'replies', 'lastPostTime', 'cumulativeLikes');
	
	/**
	 * @see	wcf\page\MultipleLinkPage::$itemsPerPage
	 */
	public $itemsPerPage = WBB_BOARD_THREADS_PER_PAGE;
	
	/**
	 * @see	wcf\page\AbstractPage::$enableTracking
	 */
	public $enableTracking = true;
	
	/**
	 * list of available content languages
	 * @var	array
	 */
	public $availableLanguages = array();
	
	/**
	 * board id
	 * @var	integer
	 */
	public $boardID = 0;
	
	/**
	 * board object
	 * @var	wbb\data\board\Board
	 */
	public $board = null;
	
	/**
	 * board node list
	 * @var	wbb\data\board\BoardNodeList
	 */
	public $boardNodeList = null;
	
	/**
	 * date filter
	 * @var	integer
	 */
	public $filterByDate = WBB_BOARD_DEFAULT_DAYS_PRUNE;
	
	/**
	 * status filter
	 * @var	string
	 */
	public $filterByStatus = '';
	
	/**
	 * label filter
	 * @var	array<integer>
	 */
	public $labelIDs = array();
	
	/**
	 * list of available label groups
	 * @var	array<wcf\data\label\group\ViewableLabelGroup>
	 */
	public $labelGroups = array();
	
	/**
	 * language filter
	 * @var	integer
	 */
	public $languageID = 0;
	
	/**
	 * list of moderator user profiles
	 * @var	array<wcf\data\user\UserProfile>
	 */
	public $userModerators = array();
	
	/**
	 * list of moderator groups
	 * @var	array<wcf\data\user\group\UserGroup>
	 */
	public $groupModerators = array();
	
	/**
	 * tag id
	 * @var	integer
	 */
	public $tagID = 0;
	
	/**
	 * query parameters for tags
	 * @var	string
	 */
	public $tagLinkParameters = '';
	
	/**
	 * users online list
	 * @var	wcf\data\user\online\UsersOnlineList
	 */
	public $usersOnlineList = null;
	
	/**
	 * @see	wcf\page\IPage::readParameters()
	 */
	public function readParameters() {
		parent::readParameters();
		
		if (isset($_REQUEST['id'])) $this->boardID = intval($_REQUEST['id']);
		$this->board = BoardCache::getInstance()->getBoard($this->boardID);
		if ($this->board === null) {
			throw new IllegalLinkException();
		}
		
		// check permissions
		if (!$this->board->canEnter()) {
			throw new PermissionDeniedException();
		}
		
		// redirect to external url if given
		if ($this->board->isExternalLink()) {
			if (!WCF::getSession()->spiderID) {
				$boardEditor = new BoardEditor($this->board);
				$boardEditor->updateCounters(array('clicks' => 1));
				BoardEditor::resetDataCache();
			}
			
			// do redirect
			HeaderUtil::redirect($this->board->externalURL);
			exit;
		}
		
		// init board style
		$this->board->initStyle();
		
		// threads per page
		if ($this->board->threadsPerPage) $this->itemsPerPage = $this->board->threadsPerPage;
		if (WCF::getUser()->threadsPerPage) $this->itemsPerPage = WCF::getUser()->threadsPerPage;
		
		// get sorting values
		if ($this->board->sortField) $this->defaultSortField = $this->board->sortField;
		if ($this->board->sortOrder) $this->defaultSortOrder = $this->board->sortOrder;
		
		// date filter
		if ($this->board->daysPrune) $this->filterByDate = $this->board->daysPrune;
		if (isset($_REQUEST['filterByDate'])) $this->filterByDate = intval($_REQUEST['filterByDate']);
		
		// status filter
		if (isset($_REQUEST['filterByStatus'])) $this->filterByStatus = StringUtil::trim($_REQUEST['filterByStatus']);
		
		// language filter
		if (isset($_REQUEST['languageID'])) $this->languageID = intval($_REQUEST['languageID']);
		
		// read available label groups
		$labelGroupIDs = BoardCache::getInstance()->getLabelGroups($this->boardID);
		if (!empty($labelGroupIDs)) {
			$this->labelGroups = LabelHandler::getInstance()->getLabelGroups($labelGroupIDs, true, 'canViewLabel');
			
			if (isset($_REQUEST['labelIDs']) && is_array($_REQUEST['labelIDs'])) {
				$this->labelIDs = $_REQUEST['labelIDs'];
				
				foreach ($this->labelIDs as $groupID => $labelID) {
					$isValid = false;
					
					// ignore zero-values
					if (!is_array($labelID) && $labelID) {
						if (isset($this->labelGroups[$groupID]) && $this->labelGroups[$groupID]->isValid($labelID)) {
							$isValid = true;
						}
						else if ($labelID == -1) {
							$isValid = true;
						}
					}
					
					if (!$isValid) {
						unset($this->labelIDs[$groupID]);
					}
				}
			}
		}
		
		// read available tags
		if (MODULE_TAGGING) {
			$tagCloud = new BoardTagCloud($this->board->boardID);
			$this->tags = $tagCloud->getTags();
			
			if (!empty($this->tags)) {
				if (isset($_REQUEST['tagID'])) {
					$tagID = intval($_REQUEST['tagID']);
					foreach ($this->tags as $tag) {
						if ($tag->tagID == $tagID) {
							$this->tagID = $tagID;
							break;
						}
					}
				}
				
				// build link parameters
				$this->tagLinkParameters = 'sortField='.$this->sortField.'&sortOrder='.$this->sortOrder.'&filterByDate='.$this->filterByDate.'&filterByStatus='.$this->filterByStatus.'&pageNo='.$this->pageNo;
				if (!empty($this->labelIDs)) {
					foreach ($this->labelIDs as $labelID) {
						$this->tagLinkParameters .= '&labelID[]='.$labelID;
					}
				}
			}
		}
	}
	
	/**
	 * @see	wcf\page\MultipleLinkPage::countItems()
	 */
	public function countItems() {
		if (!$this->board->isBoard()) {
			return 0;
		}
		
		return parent::countItems();
	}
	
	/**
	 * @see	wcf\page\MultipleLinkPage::calculateNumberOfPages()
	 */
	public function calculateNumberOfPages() {
		parent::calculateNumberOfPages();
		
		// check for announcements since they're excluded within countObjects()
		if ($this->board->isBoard() && $this->items == 0) {
			$this->items = $this->objectList->countAnnouncements();
		}
	}
	
	/**
	 * @see	wcf\page\MultipleLinkPage::initObjectList()
	 */
	protected function initObjectList() {
		if ($this->board->isBoard()) {
			$this->objectList = new BoardThreadList($this->board, $this->filterByDate, $this->labelIDs, $this->tagID, $this->filterByStatus, $this->languageID);
		}
	}
	
	/**
	 * @see	wcf\page\IPage::readData()
	 */
	public function readData() {
		parent::readData();
		
		// get node tree
		$this->boardNodeList = new DetailedBoardNodeList($this->board->boardID);
		$this->boardNodeList->readNodeTree();
		
		// users online
		if (MODULE_USERS_ONLINE && WBB_BOARD_ENABLE_ONLINE_LIST) {
			$this->usersOnlineList = new UsersOnlineList();
			$this->usersOnlineList->getConditionBuilder()->add('((session.objectType = ? AND session.objectID = ?) OR (session.parentObjectType = ? AND session.parentObjectID = ?))', array('com.woltlab.wbb.board', $this->boardID, 'com.woltlab.wbb.board', $this->boardID));
			$this->usersOnlineList->readStats();
			$this->usersOnlineList->getConditionBuilder()->add('session.userID IS NOT NULL');
			$this->usersOnlineList->readObjects();
		}
		
		// add breadcrumbs
		WBBCore::getInstance()->setBreadcrumbs($this->board->getParentBoards());
		
		// read available content languages
		$this->availableLanguages = LanguageFactory::getInstance()->getContentLanguages();
		
		// assigned moderators
		$userIDs = BoardCache::getInstance()->getUserModerators($this->board->boardID);
		if (!empty($userIDs)) {
			$this->userModerators = UserProfile::getUserProfiles($userIDs);
		}
		$groupIDs = BoardCache::getInstance()->getGroupModerators($this->board->boardID);
		if (!empty($groupIDs)) {
			foreach ($groupIDs as $groupID) {
				if (($group = UserGroup::getGroupByID($groupID)) !== null) {
					$this->groupModerators[] = $group;
				}
			}
		}
	}
	
	/**
	 * @see	wcf\page\IPage::assignVariables()
	 */
	public function assignVariables() {
		parent::assignVariables();
		
		DashboardHandler::getInstance()->loadBoxes('com.woltlab.wbb.BoardPage', $this);
		
		$hasMarkedItems = ClipboardHandler::getInstance()->hasMarkedItems(ClipboardHandler::getInstance()->getObjectTypeID('com.woltlab.wbb.thread'));
		if (!$hasMarkedItems) {
			// check if there are any posts marked
			$hasMarkedItems = ClipboardHandler::getInstance()->hasMarkedItems(ClipboardHandler::getInstance()->getObjectTypeID('com.woltlab.wbb.post'));
		}
		
		WCF::getTPL()->assign(array(
			'availableLanguages' => $this->availableLanguages,
			'boardID' => $this->boardID,
			'board' => $this->board,
			'boardNodeList' => $this->boardNodeList->getNodeList(),
			'filterByDate' => $this->filterByDate,
			'filterByStatus' => $this->filterByStatus,
			'hasMarkedItems' => $hasMarkedItems,
			'labelGroups' => $this->labelGroups,
			'labelIDs' => $this->labelIDs,
			'userModerators' => $this->userModerators,
			'groupModerators' => $this->groupModerators,
			'languageID' => $this->languageID,
			'sidebarCollapsed' => UserCollapsibleContentHandler::getInstance()->isCollapsed('com.woltlab.wcf.collapsibleSidebar', 'com.woltlab.wbb.BoardPage'),
			'sidebarName' => 'com.woltlab.wbb.BoardPage',
			'tagID' => $this->tagID,
			'usersOnlineList' => $this->usersOnlineList,
			'defaultFilterByDate' => ($this->board->filterByDate ?: WBB_BOARD_DEFAULT_DAYS_PRUNE),
			'allowSpidersToIndexThisPage' => true
		));
	}
	
	/**
	 * @see	wcf\page\ITrackablePage::getObjectType()
	 */
	public function getObjectType() {
		return 'com.woltlab.wbb.board';
	}
	
	/**
	 * @see	wcf\page\ITrackablePage::getObjectID()
	 */
	public function getObjectID() {
		return $this->boardID;
	}
}
