<?php
namespace wbb\page;
use wbb\data\board\Board;
use wbb\data\board\BoardCache;
use wbb\data\post\Post;
use wbb\data\post\ThreadPostList;
use wbb\data\thread\SimilarThreadList;
use wbb\data\thread\ThreadAction;
use wbb\data\thread\ThreadEditor;
use wbb\data\thread\ViewableThread;
use wbb\system\label\object\ThreadLabelObjectHandler;
use wbb\system\WBBCore;
use wcf\data\smiley\SmileyCache;
use wcf\data\user\online\UsersOnlineList;
use wcf\page\MultipleLinkPage;
use wcf\system\bbcode\BBCodeHandler;
use wcf\system\clipboard\ClipboardHandler;
use wcf\system\exception\IllegalLinkException;
use wcf\system\exception\PermissionDeniedException;
use wcf\system\language\LanguageFactory;
use wcf\system\like\LikeHandler;
use wcf\system\message\quote\MessageQuoteManager;
use wcf\system\request\LinkHandler;
use wcf\system\tagging\TagEngine;
use wcf\system\MetaTagHandler;
use wcf\system\WCF;
use wcf\util\HeaderUtil;
use wcf\util\StringUtil;

/**
 * Shows the thread page.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	page
 * @category	Burning Board
 */
class ThreadPage extends MultipleLinkPage {
	/**
	 * @see	wcf\page\AbstractPage::$activeMenuItem
	 */
	public $activeMenuItem = 'wbb.header.menu.board';
	
	/**
	 * @see	wcf\page\MultipleLinkPage::$itemsPerPage
	 */
	public $itemsPerPage = WBB_THREAD_POSTS_PER_PAGE;
	
	/**
	 * @see	wcf\page\MultipleLinkPage::$sortOrder
	 */
	public $sortOrder = 'ASC';
	
	/**
	 * @see	wcf\page\AbstractPage::$enableTracking
	 */
	public $enableTracking = true;
	
	/**
	 * thread id
	 * @var	integer
	 */
	public $threadID = 0;
	
	/**
	 * thread object
	 * @var	wbb\data\thread\Thread
	 */
	public $thread = null;
	
	/**
	 * post id
	 * @var	integer
	 */
	public $postID = 0;
	
	/**
	 * post object
	 * @var	wbb\data\post\Post
	 */
	public $post = null;
	
	/**
	 * board object
	 * @var	wbb\data\board\Board
	 */
	public $board = null;
	
	/**
	 * like data for posts
	 * @var	array<wcf\data\like\object\LikeObject>
	 */
	public $likeData = array();
	
	/**
	 * list of tags
	 * @var	array<wcf\data\tag\Tag>
	 */
	public $tags = array();
	
	/**
	 * users online list
	 * @var	wcf\data\user\online\UsersOnlineList
	 */
	public $usersOnlineList = null;
	
	/**
	 * sidebar factory object
	 * @var	wcf\system\message\sidebar\MessageSidebarFactory
	 */
	public $sidebarFactory = null;
	
	/**
	 * list of similar threads
	 * @var	wbb\data\thread\SimilarThreadList
	 */
	public $similarThreadList = null;
	
	/**
	 * @see	wcf\page\IPage::readParameters()
	 */
	public function readParameters() {
		parent::readParameters();
		
		if (isset($_REQUEST['id'])) $this->threadID = intval($_REQUEST['id']);
		if (isset($_REQUEST['postID'])) $this->postID = intval($_REQUEST['postID']);
		if ($this->postID) {
			$this->post = new Post($this->postID);
			if (!$this->post->postID) {
				throw new IllegalLinkException();
			}
			$this->threadID = $this->post->threadID;
		}
		
		$this->thread = ViewableThread::getThread($this->threadID);
		if ($this->thread === null || $this->thread->movedThreadID) {
			throw new IllegalLinkException();
		}
		if (!$this->thread->canRead()) {
			throw new PermissionDeniedException();
		}
		
		$this->board = BoardCache::getInstance()->getBoard($this->thread->boardID);
		
		// posts per page
		if ($this->board->postsPerPage) $this->itemsPerPage = $this->board->postsPerPage;
		if (WCF::getUser()->postsPerPage) $this->itemsPerPage = WCF::getUser()->postsPerPage;
		
		// init board style
		$this->board->initStyle();
	}
	
	/**
	 * @see	wcf\page\MultipleLinkPage::initObjectList()
	 */
	protected function initObjectList() {
		$this->objectList = new ThreadPostList($this->thread->getDecoratedObject());
		
		// handle jump to
		if ($this->action == 'lastPost') $this->goToLastPost();
		if ($this->action == 'firstNew') $this->goToFirstNewPost();
		if ($this->postID) $this->goToPost();
	}
	
	/**
	 * @see	wcf\page\IPage::readData()
	 */
	public function readData() {
		parent::readData();
		
		// add breadcrumbs
		WBBCore::getInstance()->setBreadcrumbs($this->board->getParentBoards(), $this->board);
		
		// update view count
		$threadEditor = new ThreadEditor($this->thread->getDecoratedObject());
		$threadEditor->updateCounters(array(
			'views' => 1
		));
		
		// update thread visit
		if ($this->thread->isNew() && $this->objectList->getMaxPostTime() > $this->thread->getVisitTime()) {
			$threadAction = new ThreadAction(array($this->thread->getDecoratedObject()), 'markAsRead', array(
				'visitTime' => $this->objectList->getMaxPostTime(),
				'viewableThread' => $this->thread
			));
			$threadAction->executeAction();
		}
		
		// set attachment permissions
		if ($this->objectList->getAttachmentList() !== null) {
			$this->objectList->getAttachmentList()->setPermissions(array(
				'canDownload' => $this->board->getPermission('canDownloadAttachment'),
				'canViewPreview' => $this->board->getPermission('canViewAttachmentPreview')
			));
		}
		
		// fetch likes
		if (MODULE_LIKE) {
			$postIDs = array();
			foreach ($this->objectList as $post) {
				$postIDs[] = $post->postID;
			}
			$objectType = LikeHandler::getInstance()->getObjectType('com.woltlab.wbb.likeablePost');
			LikeHandler::getInstance()->loadLikeObjects($objectType, $postIDs);
			$this->likeData = LikeHandler::getInstance()->getLikeObjects($objectType);
		}
		
		// fetch labels
		if ($this->thread->hasLabels) {
			$assignedLabels = ThreadLabelObjectHandler::getInstance()->getAssignedLabels(array($this->thread->threadID));
			if (isset($assignedLabels[$this->thread->threadID])) {
				foreach ($assignedLabels[$this->thread->threadID] as $label) {
					$this->thread->addLabel($label);
				}
			}
		}
		
		// get tags
		if (MODULE_TAGGING && WBB_THREAD_ENABLE_TAGS) {
			$this->tags = TagEngine::getInstance()->getObjectTags(
				'com.woltlab.wbb.thread',
				$this->thread->threadID,
				array(($this->thread->languageID === null ? LanguageFactory::getInstance()->getDefaultLanguageID() : ""))
			);
		}
		
		// users online
		if (MODULE_USERS_ONLINE && WBB_THREAD_ENABLE_ONLINE_LIST) {
			$this->usersOnlineList = new UsersOnlineList();
			$this->usersOnlineList->getConditionBuilder()->add('session.objectType = ?', array('com.woltlab.wbb.thread'));
			$this->usersOnlineList->getConditionBuilder()->add('session.objectID = ?', array($this->threadID));
			$this->usersOnlineList->readStats();
			$this->usersOnlineList->getConditionBuilder()->add('session.userID IS NOT NULL');
			$this->usersOnlineList->readObjects();
		}
		
		// init quote objects
		$postIDs = array();
		foreach ($this->objectList as $post) {
			$postIDs[] = $post->postID;
		}
		MessageQuoteManager::getInstance()->initObjects('com.woltlab.wbb.post', $postIDs);
		
		// get similar threads
		if (WBB_THREAD_ENABLE_SIMILAR_THREADS) {
			$boardIDs = Board::getAccessibleBoardIDs(array('canViewBoard', 'canEnterBoard', 'canReadThread'));
			if (!empty($boardIDs)) {
				$this->similarThreadList = new SimilarThreadList();
				$this->similarThreadList->getConditionBuilder()->add('thread_similar.threadID = ?', array($this->threadID));
				$this->similarThreadList->getConditionBuilder()->add('thread.boardID IN (?)', array($boardIDs));
				$this->similarThreadList->getConditionBuilder()->add('thread.isDisabled = 0 AND thread.isDeleted = 0');
				$this->similarThreadList->readObjects();
			}
		}
		
		MetaTagHandler::getInstance()->addTag('og:title', 'og:title', $this->thread->topic . ' - ' . WCF::getLanguage()->get(PAGE_TITLE), true);
		MetaTagHandler::getInstance()->addTag('og:url', 'og:url', LinkHandler::getInstance()->getLink('Thread', array('application' => 'wbb', 'object' => $this->thread)), true);
		MetaTagHandler::getInstance()->addTag('og:type', 'og:type', 'article', true);
		$posts = $this->objectList->getObjects();
		if (isset($posts[$this->thread->firstPostID])) {
			MetaTagHandler::getInstance()->addTag('og:description', 'og:description', StringUtil::decodeHTML(StringUtil::stripHTML($posts[$this->thread->firstPostID]->getExcerpt())), true);
		}
		else {
			$post = new Post($this->thread->firstPostID);
			MetaTagHandler::getInstance()->addTag('og:description', 'og:description', StringUtil::decodeHTML(StringUtil::stripHTML($post->getExcerpt())), true);
		}
		
		// add thread tags as keywords
		if (!empty($this->tags)) {
			$keywords = '';
			foreach ($this->tags as $tag) {
				if (!empty($keywords)) $keywords .= ', ';
				$keywords .= $tag->name;
			}
			MetaTagHandler::getInstance()->addTag('keywords', 'keywords', $keywords);
		}
	}
	
	/**
	 * @see	wcf\page\IPage::assignVariables()
	 */
	public function assignVariables() {
		parent::assignVariables();
		
		MessageQuoteManager::getInstance()->assignVariables();
		
		WCF::getTPL()->assign(array(
			'attachmentList' => $this->objectList->getAttachmentList(),
			'board' => $this->board,
			'defaultSmilies' => SmileyCache::getInstance()->getCategorySmilies(),
			'hasMarkedItems' => ClipboardHandler::getInstance()->hasMarkedItems(ClipboardHandler::getInstance()->getObjectTypeID('com.woltlab.wbb.post')),
			'isLastPage' => $this->isLastPage(),
			'likeData' => $this->likeData,
			'sidebarFactory' => $this->sidebarFactory,
			'sortOrder' => $this->sortOrder,
			'tags' => $this->tags,
			'thread' => $this->thread,
			'threadID' => $this->threadID,
			'usersOnlineList' => $this->usersOnlineList,
			'similarThreadList' => $this->similarThreadList,
			'allowSpidersToIndexThisPage' => true,
			'permissionCanUseSmilies' => 'user.message.canUseSmilies'
		));
		
		BBCodeHandler::getInstance()->setAllowedBBCodes(explode(',', WCF::getSession()->getPermission('user.message.allowedBBCodes')));
	}
	
	/**
	 * Calculates the position of a specific post in this thread.
	 */
	protected function goToPost() {
		$conditionBuilder = clone $this->objectList->getConditionBuilder();
		$conditionBuilder->add('time '.($this->sortOrder == 'ASC' ? '<=' : '>=').' ?', array($this->post->time));
		
		$sql = "SELECT	COUNT(*) AS posts
			FROM	wbb".WCF_N."_post post
			".$conditionBuilder;
		$statement = WCF::getDB()->prepareStatement($sql);
		$statement->execute($conditionBuilder->getParameters());
		$row = $statement->fetchArray();
		$this->pageNo = intval(ceil($row['posts'] / $this->itemsPerPage));
	}
	
	/**
	 * Forwards the user to the last post of the thread.
	 */
	protected function goToLastPost() {
		$sql = "SELECT		post.postID
			FROM		wbb".WCF_N."_post post
			".$this->objectList->getConditionBuilder()."
			ORDER BY	time DESC";
		$statement = WCF::getDB()->prepareStatement($sql, 1);
		$statement->execute($this->objectList->getConditionBuilder()->getParameters());
		$row = $statement->fetchArray();
		HeaderUtil::redirect(LinkHandler::getInstance()->getLink('Thread', array(
			'application' => 'wbb',
			'object' => $this->thread,
			'postID' => $row['postID']
		)).'#post'.$row['postID']);
		exit;
	}
	
	/**
	 * Forwards the user to the first new post in this thread.
	 */
	protected function goToFirstNewPost() {
		$conditionBuilder = clone $this->objectList->getConditionBuilder();
		$conditionBuilder->add('time > ?', array($this->thread->getVisitTime()));
		
		$sql = "SELECT		post.postID
			FROM		wbb".WCF_N."_post post
			".$conditionBuilder."
			ORDER BY	time ASC";
		$statement = WCF::getDB()->prepareStatement($sql, 1);
		$statement->execute($conditionBuilder->getParameters());
		$row = $statement->fetchArray();
		if ($row !== false) {
			HeaderUtil::redirect(LinkHandler::getInstance()->getLink('Thread', array(
				'application' => 'wbb',
				'object' => $this->thread,
				'postID' => $row['postID']
			)).'#post'.$row['postID']);
			exit;
		}
		else {
			$this->goToLastPost();
		}
	}
	
	/**
	 * @see	wcf\page\ITrackablePage::getParentObjectType()
	 */
	public function getParentObjectType() {
		return 'com.woltlab.wbb.board';
	}
	
	/**
	 * @see	wcf\page\ITrackablePage::getParentObjectID()
	 */
	public function getParentObjectID() {
		if ($this->thread) return $this->thread->boardID;
		
		return 0;
	}
	
	/**
	 * @see	wcf\page\ITrackablePage::getObjectType()
	 */
	public function getObjectType() {
		return 'com.woltlab.wbb.thread';
	}
	
	/**
	 * @see	wcf\page\ITrackablePage::getObjectID()
	 */
	public function getObjectID() {
		return $this->threadID;
	}
}
