<?php
namespace wbb\system\clipboard\action;
use wbb\data\board\BoardCache;
use wbb\data\board\FilteredBoardNodeList;
use wbb\data\thread\Thread;
use wbb\data\thread\ThreadList;
use wcf\data\clipboard\action\ClipboardAction;
use wcf\system\clipboard\action\AbstractClipboardAction;
use wcf\system\clipboard\ClipboardHandler;
use wcf\system\WCF;

/**
 * Prepares clipboard editor items for posts.
 * 
 * @author	Alexander Ebert
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	system.clipboard.action
 * @category	Burning Board
 */
class PostClipboardAction extends AbstractClipboardAction {
	/**
	 * @see	wcf\system\clipboard\action\AbstractClipboardAction::$actionClassActions
	 */
	protected $actionClassActions = array('delete', 'disable', 'enable', 'moveToExistingThread', 'restore', 'trash');
	
	/**
	 * list of posts
	 * @var	array<wbb\data\post\Post>
	 */
	public $posts = array();
	
	/**
	 * @see	wcf\system\clipboard\action\AbstractClipboardAction::$supportedActions
	 */
	protected $supportedActions = array('delete', 'disable', 'enable', 'merge', 'moveToExistingThread', 'moveToNewThread', 'trash', 'restore');
	
	/**
	 * @see	wcf\system\clipboard\action\IClipboardAction::execute()
	 */
	public function execute(array $objects, ClipboardAction $action) {
		if (empty($this->posts)) {
			$this->posts = $this->loadThreads($objects);
		}
		
		$item = parent::execute($objects, $action);
		if ($item === null) {
			return null;
		}
		
		// handle actions
		switch ($action->actionName) {
			case 'delete':
				$item->addInternalData('confirmMessage', WCF::getLanguage()->getDynamicVariable('wcf.clipboard.item.com.woltlab.wbb.post.deleteCompletely.confirmMessage', array(
					'count' => $item->getCount()
				)));
			break;
			
			case 'merge':
				if ($item->getCount() < 2) {
					return null;
				}
			break;
			
			case 'moveToExistingThread':
				$item->addInternalData('confirmMessage', WCF::getLanguage()->getDynamicVariable('wcf.clipboard.item.com.woltlab.wbb.post.moveToExistingThread.confirmMessage', array(
					'count' => $item->getCount()
				)));
				$item->addInternalData('parameters', array(
					'threadID' => ClipboardHandler::getInstance()->getPageObjectID()
				));
			break;
			
			case 'moveToNewThread':
				$boardNodeList = new FilteredBoardNodeList();
				$boardNodeList->readNodeTree();
				WCF::getTPL()->assign(array(
					'boardID' => ClipboardHandler::getInstance()->getPageObjectID(),
					'boardNodeList' => $boardNodeList->getNodeList()
				));
				
				$item->addParameter('template', WCF::getTPL()->fetch('postMoveToNewThread', 'wbb'));
			break;
			
			case 'trash':
				$item->addInternalData('confirmMessage', WCF::getLanguage()->getDynamicVariable('wcf.clipboard.item.com.woltlab.wbb.post.trash.confirmMessage', array(
					'count' => $item->getCount()
				)));
				$item->addInternalData('template', WCF::getTPL()->fetch('postDeleteReason', 'wbb'));
			break;
		}
		
		return $item;
	}
	
	/**
	 * @see	wcf\system\clipboard\action\IClipboardAction::getClassName()
	 */
	public function getClassName() {
		return 'wbb\data\post\PostAction';
	}
	
	/**
	 * @see	wcf\system\clipboard\action\IClipboardAction::getTypeName()
	 */
	public function getTypeName() {
		return 'com.woltlab.wbb.post';
	}
	
	/**
	 * Returns the ids of the posts which can be moved to trash bin.
	 * 
	 * @return	arrayinteger>
	 */
	protected function validateTrash() {
		// check if user may delete given posts
		$postIDs = array();
		foreach ($this->posts as $post) {
			if (!$post->isDeleted && $post->canDelete()) {
				$postIDs[] = $post->postID;
			}
		}
		
		return $postIDs;
	}
	
	/**
	 * Returns the ids of the posts which can be deleted.
	 * 
	 * @return	array<integer>
	 */
	protected function validateDelete() {
		// check if user may delete given posts
		$postIDs = array();
		foreach ($this->posts as $post) {
			if ($post->isDeleted && $post->canDeleteCompletely()) {
				$postIDs[] = $post->postID;
			}
		}
		
		return $postIDs;
	}
	
	/**
	 * Returns the ids of the posts which can be restored.
	 * 
	 * @return	array<integer>
	 */
	protected function validateRestore() {
		// check if user may delete given posts
		$postIDs = array();
		foreach ($this->posts as $post) {
			if ($post->isDeleted && $post->canRestore()) {
				$postIDs[] = $post->postID;
			}
		}
		
		return $postIDs;
	}
	
	/**
	 * Validates posts valid for enabling and returns their ids.
	 *
	 * @return	array<integer>
	 */
	public function validateEnable() {
		$postIDs = array();
	
		foreach ($this->posts as $post) {
			if ($post->isDisabled && !$post->isDeleted && $post->canEnable()) {
				$postIDs[] = $post->postID;
			}
		}
	
		return $postIDs;
	}
	
	/**
	 * Validates posts valid for disabling and returns their ids.
	 *
	 * @return	array<integer>
	 */
	public function validateDisable() {
		$postIDs = array();
	
		foreach ($this->posts as $post) {
			if (!$post->isDisabled && !$post->isDeleted && $post->canEnable()) {
				$postIDs[] = $post->postID;
			}
		}
	
		return $postIDs;
	}
	
	/**
	 * Validates posts for merging.
	 * 
	 * @return	array<integer>
	 */
	public function validateMerge() {
		$postIDs = array();
		foreach ($this->posts as $post) {
			if ($post->getThread()->canRead() && $post->getThread()->getBoard()->getModeratorPermission('canMergePost')) {
				$postIDs[] = $post->postID;
			}
		}
		
		return $postIDs;
	}
	
	/**
	 * Validates posts to move them into an existing thread.
	 * 
	 * @return	array<integer>
	 */
	public function validateMoveToExistingThread() {
		$postIDs = $this->__validateMove();
		
		if (!empty($postIDs)) {
			// validate permissions for target thread
			$thread = new Thread(ClipboardHandler::getInstance()->getPageObjectID());
			if (!$thread->threadID || !$thread->getBoard()->getModeratorPermission('canMergePost')) {
				$postIDs = array();
			}
			else {
				foreach ($postIDs as $key => $postID) {
					// cannot move into the same thread
					if ($this->posts[$postID]->threadID == $thread->threadID) {
						unset($postIDs[$key]);
					}
				}
			}
		}
		
		return $postIDs;
	}
	
	/**
	 * Validates posts to move them into a new thread.
	 * 
	 * @return	array<integer>
	 */
	public function validateMoveToNewThread() {
		$postIDs = $this->__validateMove();
		
		if (!empty($postIDs)) {
			// validate permissions for target board
			$board = BoardCache::getInstance()->getBoard(ClipboardHandler::getInstance()->getPageObjectID());
			if ($board === null || !$board->canStartThread()) {
				$postIDs = array();
			}
		}
		
		return $postIDs;
	}
	
	/**
	 * Validates posts for moving.
	 * 
	 * @return	array<integer>
	 */
	protected function __validateMove() {
		if (!ClipboardHandler::getInstance()->getPageObjectID()) {
			return array();
		}
		
		$postIDs = array();
		foreach ($this->posts as $post) {
			if ($post->getThread()->canRead() && $post->getThread()->getBoard()->getModeratorPermission('canMovePost')) {
				$postIDs[] = $post->postID;
			}
		}
		
		return $postIDs;
	}
	
	/**
	 * Loads threads for given posts.
	 * 
	 * @param	array<wbb\data\post\Post>	$objects
	 * @return	array<wbb\data\post\Post>
	 */
	protected function loadThreads(array $objects) {
		// get thread objects
		$threadIDs = array();
		foreach ($objects as $post) {
			$threadIDs[] = $post->threadID;
		}
		
		$threadList = new ThreadList();
		$threadList->getConditionBuilder()->add("thread.threadID IN (?)", array($threadIDs));
		$threadList->readObjects();
		
		foreach ($threadList as $thread) {
			foreach ($objects as $post) {
				if ($post->threadID == $thread->threadID) {
					$post->setThread($thread);
				}
			}
		}
		
		return $objects;
	}
}
