<?php
namespace wbb\system\clipboard\action;
use wbb\data\board\BoardList;
use wcf\data\clipboard\action\ClipboardAction;
use wcf\system\clipboard\action\AbstractClipboardAction;
use wcf\system\WCF;

/**
 * Prepares clipboard editor items for posts.
 * 
 * @author	Alexander Ebert
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	system.clipboard.action
 * @category	Burning Board
 */
class ThreadClipboardAction extends AbstractClipboardAction {
	/**
	 * @see	wcf\system\clipboard\action\AbstractClipboardAction::$actionClassActions
	 */
	protected $actionClassActions = array('close', 'delete', 'done', 'enable', 'disable', 'open', 'removeLink', 'restore', 'trash', 'undone');
	
	/**
	 * @see	wcf\system\clipboard\action\AbstractClipboardAction::$supportedActions
	 */
	protected $supportedActions = array('close', 'delete', 'done', 'enable', 'disable', 'merge', 'move', 'open', 'removeLink', 'restore', 'trash', 'undone');
	
	/**
	 * list of active thread objects
	 * @var	array<wbb\data\thread\Thread>
	 */
	protected $threads = array();
	
	/**
	 * @see	wcf\system\clipboard\action\IClipboardAction::execute()
	 */
	public function execute(array $objects, ClipboardAction $action) {
		if (empty($this->threads)) {
			$this->threads = $this->loadBoards($objects);
		}
		
		$item = parent::execute($objects, $action);
		if ($item === null) {
			return null;
		}
		
		// handle actions
		switch ($action->actionName) {
			case 'trash':
				$item->addInternalData('confirmMessage', WCF::getLanguage()->getDynamicVariable('wcf.clipboard.item.com.woltlab.wbb.thread.trash.confirmMessage', array(
					'count' => $item->getCount()
				)));
			break;
			
			case 'delete':
				$item->addInternalData('confirmMessage', WCF::getLanguage()->getDynamicVariable('wcf.clipboard.item.com.woltlab.wbb.thread.delete.confirmMessage', array(
					'count' => $item->getCount()
				)));
			break;
			
			case 'removeLink':
				$item->addInternalData('confirmMessage', WCF::getLanguage()->getDynamicVariable('wcf.clipboard.item.com.woltlab.wbb.thread.removeLink.confirmMessage', array(
					'count' => $item->getCount()
				)));
			break;
			
			case 'merge':
				if ($item->getCount() < 2) {
					return null;
				}
			break;
		}
		
		return $item;
	}
	
	/**
	 * @see	wcf\system\clipboard\action\IClipboardAction::getClassName()
	 */
	public function getClassName() {
		return 'wbb\data\thread\ThreadAction';
	}
	
	/**
	 * @see	wcf\system\clipboard\action\IClipboardAction::getTypeName()
	 */
	public function getTypeName() {
		return 'com.woltlab.wbb.thread';
	}
	
	/**
	 * Returns the ids of the relevant threads which can marked as done.
	 * 
	 * @return	array<integer>
	 */
	protected function validateDone() {
		$this->__validateDone(true);
	}
	
	/**
	 * Returns the ids of the relevant threads which can marked as undone.
	 * 
	 * @return	array<integer>
	 */
	protected function validateUndone() {
		$this->__validateDone(false);
	}
	
	/**
	 * Validates parameters to mark threads as done/undone.
	 * 
	 * @param	boolean		$markAsDone
	 * @return	array<integer>
	 */
	protected function __validateDone($markAsDone = true) {
		$threadIDs = array();
		
		if (WBB_MODULE_THREAD_MARKING_AS_DONE) {
			foreach ($this->threads as $thread) {
				if (($markAsDone && $thread->isDone) || (!$markAsDone && !$thread->isDone)) {
					continue;
				}
				
				if ($thread->getBoard()->getModeratorPermission('canMarkAsDoneThread')) {
					$threadIDs[] = $thread->threadID;
				}
				else if ($thread->getBoard()->getPermission('canMarkAsDoneOwnThread') && ($thread->userID == WCF::getUser()->userID)) {
					$threadIDs[] = $thread->threadID;
				}
			}
		}
		
		return $threadIDs;
	}
	
	/**
	 * Validates threads valid for moving and returns their ids.
	 * 
	 * @return	array<integer>
	 */
	protected function validateMove() {
		$threadIDs = array();
		
		foreach ($this->threads as $thread) {
			if (!$thread->movedThreadID && $thread->getBoard()->getModeratorPermission('canMoveThread')) {
				$threadIDs[] = $thread->threadID;
			}
		}
		
		return $threadIDs;
	}
	
	/**
	 * Validates threads valid for closing and returns their ids.
	 * 
	 * @return	array<integer>
	 */
	public function validateClose() {
		$threadIDs = array();
		
		foreach ($this->threads as $thread) {
			if (!$thread->movedThreadID && !$thread->isClosed && $thread->getBoard()->getModeratorPermission('canCloseThread')) {
				$threadIDs[] = $thread->threadID;
			}
		}
		
		return $threadIDs;
	}
	
	/**
	 * Validates threads valid for opening and returns their ids.
	 *
	 * @return	array<integer>
	 */
	public function validateOpen() {
		$threadIDs = array();
		
		foreach ($this->threads as $thread) {
			if (!$thread->movedThreadID && $thread->isClosed && $thread->getBoard()->getModeratorPermission('canCloseThread')) {
				$threadIDs[] = $thread->threadID;
			}
		}
		
		return $threadIDs;
	}
	
	/**
	 * Validates threads valid for trashing and returns their ids.
	 * 
	 * @return	array<integer>
	 */
	public function validateTrash() {
		$threadIDs = array();
		
		foreach ($this->threads as $thread) {
			if (!$thread->movedThreadID && !$thread->isDeleted && $thread->getBoard()->getModeratorPermission('canDeleteThread')) {
				$threadIDs[] = $thread->threadID;
			}
		}
		
		return $threadIDs;
	}
	
	/**
	 * Validates threads valid for deleting and returns their ids.
	 * 
	 * @return	array<integer>
	 */
	public function validateDelete() {
		$threadIDs = array();
		
		foreach ($this->threads as $thread) {
			if (!$thread->movedThreadID && $thread->isDeleted && $thread->getBoard()->getModeratorPermission('canDeleteThreadCompletely')) {
				$threadIDs[] = $thread->threadID;
			}
		}
		
		return $threadIDs;
	}
	
	/**
	 * Validates threads valid for restoring and returns their ids.
	 * 
	 * @return	array<integer>
	 */
	public function validateRestore() {
		$threadIDs = array();
		
		foreach ($this->threads as $thread) {
			if (!$thread->movedThreadID && $thread->isDeleted && $thread->getBoard()->getModeratorPermission('canRestoreThread')) {
				$threadIDs[] = $thread->threadID;
			}
		}
		
		return $threadIDs;
	}
	
	/**
	 * Validates threads valid for enabling and returns their ids.
	 * 
	 * @return	array<integer>
	 */
	public function validateEnable() {
		$threadIDs = array();
		
		foreach ($this->threads as $thread) {
			if (!$thread->movedThreadID && $thread->isDisabled && !$thread->isDeleted && $thread->getBoard()->getModeratorPermission('canEnableThread')) {
				$threadIDs[] = $thread->threadID;
			}
		}
		
		return $threadIDs;
	}
	
	/**
	 * Validates threads valid for disabling and returns their ids.
	 *
	 * @return	array<integer>
	 */
	public function validateDisable() {
		$threadIDs = array();
	
		foreach ($this->threads as $thread) {
			if (!$thread->movedThreadID && !$thread->isDisabled && !$thread->isDeleted && $thread->getBoard()->getModeratorPermission('canEnableThread')) {
				$threadIDs[] = $thread->threadID;
			}
		}
	
		return $threadIDs;
	}
	
	/**
	 * Validates threads representing a link to a moved thread and returns their ids.
	 * 
	 * @return	array<integer>
	 */
	public function validateRemoveLink() {
		$threadIDs = array();
		
		foreach ($this->threads as $thread) {
			if ($thread->movedThreadID && $thread->getBoard()->getModeratorPermission('canMoveThread')) {
				$threadIDs[] = $thread->threadID;
			}
		}
		
		return $threadIDs;
	}
	
	/**
	 * Validates threads for merging.
	 * 
	 * @return	array<integer>
	 */
	public function validateMerge() {
		$threadIDs = array();
		
		foreach ($this->threads as $thread) {
			if ($thread->canRead() && $thread->getBoard()->getModeratorPermission('canMergeThread')) {
				$threadIDs[] = $thread->threadID;
			}
		}
		
		return $threadIDs;
	}
	
	/**
	 * Loads associated boards for given threads.
	 * 
	 * @param	array<wbb\data\thread\Thread>	$threads
	 * @return	array<wbb\data\thread\Thread>
	 */
	protected function loadBoards(array $threads) {
		// get board objects
		$boardIDs = array();
		foreach ($threads as $thread) {
			$boardIDs[] = $thread->boardID;
		}
		
		if (empty($boardIDs)) {
			return array();
		}
		
		$boardList = new BoardList();
		$boardList->getConditionBuilder()->add("board.boardID IN (?)", array($boardIDs));
		$boardList->readObjects();
		
		foreach ($boardList as $board) {
			foreach ($threads as $thread) {
				if ($board->boardID == $thread->boardID) {
					$thread->setBoard($board);
				}
			}
		}
		
		return $threads;
	}
}
