<?php
namespace wbb\system\moderation\queue;
use wbb\data\board\BoardCache;
use wbb\data\post\Post;
use wbb\data\post\PostAction;
use wbb\data\post\PostList;
use wbb\data\thread\ThreadList;
use wcf\data\moderation\queue\ModerationQueue;
use wcf\system\database\util\PreparedStatementConditionBuilder;
use wcf\system\moderation\queue\AbstractModerationQueueHandler;
use wcf\system\moderation\queue\ModerationQueueManager;
use wcf\system\WCF;

/**
 * An abstract implementation of IModerationQueueHandler for forum posts.
 * 
 * @author	Alexander Ebert
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	system.moderation.queue
 * @category	Burning Board
 */
abstract class AbstractPostModerationQueueHandler extends AbstractModerationQueueHandler {
	/**
	 * list of post objects
	 * @var	array<wbb\data\post\Post>
	 */
	protected static $posts = array();
	
	/**
	 * @see	wcf\system\moderation\queue\IModerationQueueHandler::assignQueues()
	 */
	public function assignQueues(array $queues) {
		$assignments = $orphanedQueueIDs = $postIDs = array();
		foreach ($queues as $queue) {
			$postIDs[] = $queue->objectID;
		}
		
		// fetch board id for each post
		$conditions = new PreparedStatementConditionBuilder();
		$conditions->add("post.postID IN (?)", array($postIDs));
		$sql = "SELECT		post.postID, thread.boardID
			FROM		wbb".WCF_N."_post post
			LEFT JOIN	wbb".WCF_N."_thread thread
			ON		(thread.threadID = post.threadID)
			".$conditions;
		$statement = WCF::getDB()->prepareStatement($sql);
		$statement->execute($conditions->getParameters());
		$boardIDs = array();
		while ($row = $statement->fetchArray()) {
			$boardIDs[$row['postID']] = $row['boardID'];
		}
		
		foreach ($queues as $queue) {
			$assignUser = false;
			if (!isset($boardIDs[$queue->objectID])) {
				$orphanedQueueIDs[] = $queue->queueID;
				continue;
			}
			
			if (WCF::getSession()->getPermission('mod.board.isSuperMod')) {
				$assignUser = true;
			}
			else if (in_array(WCF::getUser()->userID, BoardCache::getInstance()->getUserModerators($boardIDs[$queue->objectID]))) {
				$assignUser = true;
			}
			else if (count(array_intersect(WCF::getUser()->getGroupIDs(), BoardCache::getInstance()->getGroupModerators($boardIDs[$queue->objectID])))) {
				$assignUser = true;
			}
				
			$assignments[$queue->queueID] = $assignUser;
		}
		
		ModerationQueueManager::getInstance()->removeOrphans($orphanedQueueIDs);
		ModerationQueueManager::getInstance()->setAssignment($assignments);
	}
	
	/**
	 * @see	wcf\system\moderation\queue\IModerationQueueHandler::getContainerID()
	 */
	public function getContainerID($objectID) {
		$sql = "SELECT	boardID
			FROM	wbb".WCF_N."_thread
			WHERE	threadID = ?";
		$statement = WCF::getDB()->prepareStatement($sql);
		$statement->execute(array($this->getPost($objectID)->threadID));
		$row = $statement->fetchArray();
		
		return ($row === false ? 0 : $row['boardID']);
	}
	
	/**
	 * @see	wcf\system\moderation\queue\IModerationQueueHandler::isValid()
	 */
	public function isValid($objectID) {
		if ($this->getPost($objectID) === null) {
			return false;
		}
		
		return true;
	}
	
	/**
	 * Returns a post object by post id or null if post id is invalid.
	 * 
	 * @param	integer		$objectID
	 * @return	wbb\data\post\Post
	 */
	protected function getPost($objectID) {
		if (!array_key_exists($objectID, self::$posts)) {
			self::$posts[$objectID] = new Post($objectID);
			if (!self::$posts[$objectID]->postID) {
				self::$posts[$objectID] = null;
			}
		}
		
		return self::$posts[$objectID];
	}
	
	/**
	 * @see	wcf\system\moderation\queue\IModerationQueueHandler::populate()
	 */
	public function populate(array $queues) {
		$threads = $objectIDs = array();
		foreach ($queues as $object) {
			$objectIDs[] = $object->objectID;
		}
		
		// fetch posts
		$postList = new PostList();
		$postList->sqlSelects .= "user_avatar.*, user_table.*";
		$postList->sqlJoins .= " LEFT JOIN wcf".WCF_N."_user user_table ON (user_table.userID = post.userID)";
		$postList->sqlJoins .= " LEFT JOIN wcf".WCF_N."_user_avatar user_avatar ON (user_avatar.avatarID = user_table.avatarID)";
		$postList->getConditionBuilder()->add("post.postID IN (?)", array($objectIDs));
		$postList->readObjects();
		$posts = $postList->getObjects();
		
		// fetch threads
		if (!empty($posts)) {
			$threadIDs = array();
			foreach ($posts as $post) {
				$threadIDs[] = $post->threadID;
			}
			
			$threadList = new ThreadList();
			$threadList->getConditionBuilder()->add("thread.threadID IN (?)", array($threadIDs));
			$threadList->readObjects();
			$threads = $threadList->getObjects();
		}
		
		foreach ($queues as $object) {
			if (isset($posts[$object->objectID])) {
				$post = $posts[$object->objectID];
				$post->setThread($threads[$post->threadID]);
				
				$object->setAffectedObject($post);
			}
			else {
				$object->setIsOrphaned();
			}
		}
	}
	
	/**
	 * @see	wcf\system\moderation\queue\IModerationQueueHandler::removeContent()
	 */
	public function removeContent(ModerationQueue $queue, $message) {
		if ($this->isValid($queue->objectID) && !$this->getPost($queue->objectID)->isDeleted) {
			$postAction = new PostAction(array($this->getPost($queue->objectID)), 'trash', array('reason' => $message));
			$postAction->executeAction();
		}
	}
}
