<?php
namespace wbb\system\search;
use wbb\data\board\BoardCache;
use wbb\data\board\SearchableBoardNodeList;
use wbb\data\post\SearchResultPostList;
use wbb\data\thread\SearchResultThreadList;
use wbb\system\event\listener\SearchListener;
use wcf\form\IForm;
use wcf\system\database\util\PreparedStatementConditionBuilder;
use wcf\system\exception\PermissionDeniedException;
use wcf\system\exception\UserInputException;
use wcf\system\language\LanguageFactory;
use wcf\system\search\AbstractSearchableObjectType;
use wcf\system\search\SearchEngine;
use wcf\system\WCF;
use wcf\util\ArrayUtil;

/**
 * An implementation of ISearchableObjectType for searching in forum posts.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	system.search
 * @category	Burning Board
 */
class PostSearch extends AbstractSearchableObjectType {
	/**
	 * message data cache
	 * @var	array
	 */
	public $messageCache = array();
	
	/**
	 * board ids
	 * @var	array<integer>
	 */
	public $boardIDs = array();
	
	/**
	 * enables search for attachments
	 * @var	boolean
	 */
	public $findAttachments = 0;
	
	/**
	 * enables search for polls
	 * @var	boolean
	 */
	public $findPolls = 0;
	
	/**
	 * shows results as threads
	 * @var	boolean
	 */
	public $findThreads = WBB_SEARCH_FIND_THREADS;
	
	/**
	 * enables search for user threads
	 * @var	integer
	 */
	public $findUserThreads = 0;
	
	/**
	 * list of all boards
	 * @var	array
	 */
	public $boards = array();
	
	/**
	 * list of selected boards
	 * @var	array
	 */
	public $selectedBoards = array();
	
	/**
	 * @see	wcf\system\search\ISearchableObjectType::cacheObjects()
	 */
	public function cacheObjects(array $objectIDs, array $additionalData = null) {
		if ($additionalData !== null && !empty($additionalData['findThreads'])) {
			SearchListener::$findThreads = true;
			$threadList = new SearchResultThreadList();
			$threadList->getConditionBuilder()->add('thread.threadID IN (?)', array($objectIDs));
			$threadList->readObjects();
			foreach ($threadList->getObjects() as $thread) {
				$this->messageCache[$thread->threadID] = $thread;
			}
		}
		else {
			$postList = new SearchResultPostList();
			$postList->getConditionBuilder()->add('post.postID IN (?)', array($objectIDs));
			$postList->readObjects();
			foreach ($postList->getObjects() as $post) {
				$this->messageCache[$post->postID] = $post;
			}
		}
	}
	
	/**
	 * @see	wcf\system\search\AbstractSearchableObjectType::getApplication()
	 */
	public function getApplication() {
		return 'wbb';
	}
	
	/**
	 * @see	wcf\system\search\ISearchableObjectType::getObject()
	 */
	public function getObject($objectID) {
		if (isset($this->messageCache[$objectID])) return $this->messageCache[$objectID];
		return null;
	}
	
	/**
	 * @see	wcf\system\search\ISearchableObjectType::getFormTemplateName()
	 */
	public function getFormTemplateName() {
		return 'searchPost';
	}
	
	/**
	 * @see	wcf\system\search\ISearchableObjectType::getJoins()
	 */
	public function getJoins() {
		return "LEFT JOIN wbb".WCF_N."_thread thread ON (thread.threadID = ".$this->getTableName().".threadID)";
	}
	
	/**
	 * @see	wcf\system\search\ISearchableObjectType::getTableName()
	 */
	public function getTableName() {
		return 'wbb'.WCF_N.'_post';
	}
	
	/**
	 * @see	wcf\system\search\ISearchableObjectType::getIDFieldName()
	 */
	public function getIDFieldName() {
		return $this->getTableName().'.postID';
	}
	
	/**
	 * @see	wcf\system\search\ISearchableObjectType::getAdditionalData()
	 */
	public function getAdditionalData() {
		return array(
			'findThreads' => $this->findThreads,
			'findAttachments' => $this->findAttachments,
			'findPolls' => $this->findPolls,
			'findUserThreads' => $this->findUserThreads,
			'boardIDs' => $this->boardIDs
		);
	}
	
	/**
	 * @see	wcf\system\search\ISearchableObjectType::getConditions()
	 */
	public function getConditions(IForm $form = null) {
		$conditionBuilder = new PreparedStatementConditionBuilder();
		$this->readFormParameters($form);
		
		$boardIDs = $this->boardIDs;
		if (count($boardIDs) && $boardIDs[0] == '*') $boardIDs = array();
		
		// remove empty elements
		foreach ($boardIDs as $key => $boardID) {
			if ($boardID == '-') unset($boardIDs[$key]);
		}
		
		// get all boards from cache
		$this->boards = BoardCache::getInstance()->getBoards();
		$this->selectedBoards = array();
		
		// check whether the selected board does exist
		foreach ($boardIDs as $boardID) {
			if (!isset($this->boards[$boardID]) || !$this->boards[$boardID]->searchable) {
				throw new UserInputException('boardIDs', 'notValid');
			}
			
			if (!isset($this->selectedBoards[$boardID])) {
				$this->selectedBoards[$boardID] = $this->boards[$boardID];
				
				// include children
				$this->includeSubBoards($boardID);
			}
		}
		if (count($this->selectedBoards) == 0) $this->selectedBoards = $this->boards;
		
		// check permission of the active user
		foreach ($this->selectedBoards as $board) {
			if ($board->isIgnored() || !$board->getPermission() || !$board->getPermission('canEnterBoard') || !$board->getPermission('canReadThread') || !$board->searchable) {
				unset($this->selectedBoards[$board->boardID]);
			}
		}
		if (count($this->selectedBoards) == 0) {
			throw new PermissionDeniedException();
		}
		
		// get board ids 
		$boardIDs = array();
		if (count($this->selectedBoards) != count($this->boards)) {
			foreach ($this->selectedBoards as $board) {
				$boardIDs[] = $board->boardID;
			}
		}
		
		// board ids
		if (count($boardIDs)) {
			$conditionBuilder->add('thread.boardID IN (?)', array($boardIDs));
		}
		
		// find user threads
		if ($this->findUserThreads && $form !== null && ($userIDs = $form->getUserIDs())) {
			$conditionBuilder->add('thread.userID IN (?)', array($userIDs));
		}
		
		if (!$this->findThreads) {
			// default conditions
			$conditionBuilder->add($this->getTableName().'.isDeleted = 0');
			$conditionBuilder->add($this->getTableName().'.isDisabled = 0');
			
			// find attachments
			if ($this->findAttachments) $conditionBuilder->add($this->getTableName().'.attachments > 0');
			// find polls
			if ($this->findPolls) $conditionBuilder->add($this->getTableName().'.pollID IS NOT NULL');
		}
		
		// language
		if (LanguageFactory::getInstance()->multilingualismEnabled() && count(WCF::getUser()->getLanguageIDs())) {
			$conditionBuilder->add('(thread.languageID IN (?) OR thread.languageID IS NULL)', array(WCF::getUser()->getLanguageIDs()));
		}
		
		return $conditionBuilder;
	}
	
	/**
	 * @see	wcf\system\search\ISearchableObjectType::show()
	 */
	public function show(IForm $form = null) {
		// get searchable boards
		$boardNodeList = new SearchableBoardNodeList();
		$boardNodeList->readNodeTree();
		
		// get existing values
		if ($form !== null && isset($form->searchData['additionalData']['post'])) {
			$this->boardIDs = $form->searchData['additionalData']['post']['boardIDs'];
			$this->findAttachments = $form->searchData['additionalData']['post']['findAttachments'];
			$this->findPolls = $form->searchData['additionalData']['post']['findPolls'];
		}
		
		WCF::getTPL()->assign(array(
			'boardNodeList' => $boardNodeList->getNodeList(),
			'boardIDs' => $this->boardIDs,
			'selectAllBoards' => count($this->boardIDs) == 0 || $this->boardIDs[0] == '*',
			'findAttachments' => $this->findAttachments,
			'findPolls' => $this->findPolls,
			'findThreads' => $this->findThreads,
			'findUserThreads' => $this->findUserThreads
		));
	}
	
	/**
	 * @see	wcf\system\search\ISearchableObjectType::getSpecialSQLQuery()
	 */
	public function getSpecialSQLQuery(PreparedStatementConditionBuilder &$fulltextCondition = null, PreparedStatementConditionBuilder &$searchIndexConditions = null, PreparedStatementConditionBuilder &$additionalConditions = null, $orderBy = 'time DESC') {
		if (!$this->findThreads) return '';
		
		$objectType = SearchEngine::getInstance()->getObjectType('com.woltlab.wbb.post');
		
		if ($fulltextCondition === null) {
			return "SELECT		thread.threadID AS objectID, thread.topic AS subject, thread.time, thread.username,
						'com.woltlab.wbb.post' AS objectType
				FROM		wbb".WCF_N."_thread thread
				INNER JOIN	(
							SELECT		DISTINCT post.threadID
							FROM		wbb".WCF_N."_post post
							WHERE		".$searchIndexConditions."
									AND post.isDeleted = 0
									AND post.isDisabled = 0
									".($this->findAttachments ? "AND post.attachments > 0" : '')."
									".($this->findPolls ? "AND post.pollID IS NOT NULL" : '')."
						) AS subselect
				ON		(thread.threadID = subselect.threadID)
				".($additionalConditions !== null ? $additionalConditions : '');
		}
		else {
			return "SELECT		thread.threadID AS objectID, thread.topic AS subject, thread.time, thread.username,
						'com.woltlab.wbb.post' AS objectType
				FROM		wbb".WCF_N."_thread thread
				INNER JOIN	(
							SELECT		DISTINCT post.threadID
							FROM		wbb".WCF_N."_post post
							INNER JOIN	(
										SELECT		objectID
										FROM		wcf".WCF_N."_search_index
										WHERE		".$fulltextCondition."
												".(($searchIndexConditions !== null && $searchIndexConditions->__toString()) ? ("AND ".$searchIndexConditions) : '')."
												AND objectTypeID = ".$objectType->objectTypeID."
										LIMIT		1000
									) search_index
							ON		(post.postID = search_index.objectID)
							WHERE		post.isDeleted = 0
									AND post.isDisabled = 0
									".($this->findAttachments ? "AND post.attachments > 0" : '')."
									".($this->findPolls ? "AND post.pollID IS NOT NULL" : '')."
						) AS subselect
				ON		(thread.threadID = subselect.threadID)
				".($additionalConditions !== null ? $additionalConditions : '');
		}
	}
	
	/**
	 * Reads the given form parameters.
	 * 
	 * @param	wcf\form\IForm		$form
	 */
	protected function readFormParameters(IForm $form = null) {
		$this->findThreads = 0;
		
		// get existing values
		if ($form !== null && isset($form->searchData['additionalData']['post'])) {
			$this->boardIDs = $form->searchData['additionalData']['post']['boardIDs'];
			$this->findAttachments = $form->searchData['additionalData']['post']['findAttachments'];
			$this->findPolls = $form->searchData['additionalData']['post']['findPolls'];
			//$this->findThreads = $form->searchData['additionalData']['post']['findThreads'];
			$this->findUserThreads = $form->searchData['additionalData']['post']['findUserThreads'];
		}
		
		// get new values
		if (isset($_POST['boardIDs']) && is_array($_POST['boardIDs'])) {
			$this->boardIDs = ArrayUtil::toIntegerArray($_POST['boardIDs']);
		}
		
		if (isset($_POST['findAttachments'])) {
			$this->findAttachments = intval($_POST['findAttachments']);
		}
		
		if (isset($_POST['findPolls'])) {
			$this->findPolls = intval($_POST['findPolls']);
		}
		
		if (isset($_POST['findThreads'])) {
			$this->findThreads = intval($_POST['findThreads']);
		}
		
		if (isset($_REQUEST['findUserThreads'])) {
			$this->findUserThreads = intval($_REQUEST['findUserThreads']);
			if ($this->findUserThreads) $this->findThreads = 1;
		}
	}
	
	private function includeSubBoards($boardID) {
		foreach (BoardCache::getInstance()->getChildIDs($boardID) as $childBoardID) {
			if (!isset($this->selectedBoards[$childBoardID])) {
				$this->selectedBoards[$childBoardID] = $this->boards[$childBoardID];
				
				// include children
				$this->includeSubBoards($childBoardID);
			}
		}
	}
}
