<?php
namespace wbb\system\thread;
use wbb\data\thread\Thread;
use wbb\system\thread\editor\DefaultThreadEditor;
use wbb\system\thread\editor\LabelThreadEditor;
use wcf\system\event\EventHandler;
use wcf\system\exception\IllegalLinkException;
use wcf\system\exception\PermissionDeniedException;
use wcf\system\SingletonFactory;

/**
 * Handles thread-related actions.
 * 
 * @author	Alexander Ebert
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	system.thread
 * @category	Burning Board
 */
class ThreadHandler extends SingletonFactory {
	/**
	 * list of thread editors
	 * @var	array<array>
	 */
	protected $editors = array(
		'type' => array(),
		'object' => array()
	);
	
	/**
	 * @see	wcf\system\SingletonFactory::init()
	 */
	protected function init() {
		$this->editors = array(
			'type' => array(
				1 => 'label',
				0 => 'default'
			),
			'object' => array(
				'label' => new LabelThreadEditor(),
				'default' => new DefaultThreadEditor()
			)
		);
		
		EventHandler::getInstance()->fireAction($this, 'didInit');
	}
	
	/**
	 * Returns template for thread editor.
	 * 
	 * @param	integer		$threadID
	 * @return	string
	 */
	public function beginEdit($threadID) {
		$this->setThread($threadID);
		$template = '';
		
		foreach ($this->editors['type'] as $typeName) {
			$template .= $this->editors['object'][$typeName]->beginEdit();
		}
		
		return $template;
	}
	
	/**
	 * Saves editor data.
	 * 
	 * @param	integer		$threadID
	 * @param	array<array>	$parameters
	 * @return	array
	 */
	public function saveEdit($threadID, array $parameters) {
		$this->setThread($threadID);
		
		// validate parameters prior to updating data
		foreach ($this->editors['type'] as $typeName) {
			if (isset($parameters[$typeName])) {
				$this->editors['object'][$typeName]->validate($parameters[$typeName]);
			}
		}
		
		// update data
		$returnValues = array();
		foreach ($this->editors['type'] as $typeName) {
			if (isset($parameters[$typeName])) {
				$returnValues[$typeName] = $this->editors['object'][$typeName]->saveEdit($parameters[$typeName]);
			}
		}
		
		$forceReload = false;
		foreach ($returnValues as $data) {
			if (isset($data['forceReload']) && $data['forceReload']) {
				$forceReload = true;
				break;
			}
		}
		
		$returnValues['forceReload'] = $forceReload;
		return $returnValues;
	}
	
	/**
	 * Loads and assigns a thread to loaded editors.
	 * 
	 * @param	integer		$threadID
	 */
	protected function setThread($threadID) {
		$thread = new Thread($threadID);
		if (!$thread->threadID) {
			throw new IllegalLinkException();
		}
		else if (!$thread->canEdit()) {
			throw new PermissionDeniedException();
		}
		
		foreach ($this->editors['object'] as $editor) {
			$editor->setThread($thread);
		}
	}
}
