<?php
namespace wbb\system\thread\editor;
use wbb\data\board\BoardCache;
use wbb\data\thread\ThreadEditor;
use wbb\system\label\object\ThreadLabelObjectHandler;
use wbb\system\log\modification\ThreadModificationLogHandler;
use wcf\data\label\LabelList;
use wcf\system\exception\UserInputException;
use wcf\system\label\LabelHandler;
use wcf\system\request\LinkHandler;
use wcf\system\WCF;

/**
 * Provides an editor for thread labels.
 * 
 * @author	Alexander Ebert
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	system.thread.editor
 * @category	Burning Board
 */
class LabelThreadEditor extends AbstractThreadEditor {
	/**
	 * list of label ids
	 * @var	array<integer>
	 */
	public $labelIDs = array();
	
	/**
	 * @see wbb\system\thread\editor\IThreadEditor::beginEdit()
	 */
	public function beginEdit() {
		$labelGroupIDs = BoardCache::getInstance()->getLabelGroups($this->thread->boardID);
		if (empty($labelGroupIDs)) {
			// nothing to do here :)
			return '';
		}
		
		$assignedLabels = ThreadLabelObjectHandler::getInstance()->getAssignedLabels(array($this->thread->threadID));
		if (isset($assignedLabels[$this->thread->threadID])) {
			$assignedLabels = $assignedLabels[$this->thread->threadID];
		}
		
		WCF::getTPL()->assign(array(
			'assignedLabels' => $assignedLabels,
			'labelGroups' => LabelHandler::getInstance()->getLabelGroups($labelGroupIDs)
		));
		
		return WCF::getTPL()->fetch('threadEditorLabel', 'wbb');
	}
	
	/**
	 * @see	wbb\system\thread\editor\IThreadEditor::validate()
	 */
	public function validate(array &$data) {
		// get already assigned label ids
		$labelIDs = ThreadLabelObjectHandler::getInstance()->getAssignedLabels(array($this->thread->threadID));
		if (isset($labelIDs[$this->thread->threadID])) {
			foreach ($labelIDs[$this->thread->threadID] as $label) {
				$this->labelIDs[$label->groupID] = $label->labelID;
			}
		}
		
		$labelGroupIDs = BoardCache::getInstance()->getLabelGroups($this->thread->boardID);
		$labelGroups = LabelHandler::getInstance()->getLabelGroups($labelGroupIDs);
		if (!empty($labelGroupIDs)) {
			foreach ($data as $groupID => $labelID) {
				if (is_array($labelID)) {
					throw new UserInputException('labelIDs');
				}
				
				if (!$labelID) {
					unset($this->labelIDs[$groupID]);
					continue;
				}
				
				if (!isset($labelGroups[$groupID]) || !$labelGroups[$groupID]->isValid($labelID)) {
					throw new UserInputException('labelIDs');
				}
				
				$this->labelIDs[$groupID] = $labelID;
			}
		}
		
		// check if required label groups were set
		$optionID = LabelHandler::getInstance()->getOptionID('canSetLabel');
		foreach ($labelGroups as $labelGroup) {
			if ($labelGroup->forceSelection && !isset($this->labelIDs[$labelGroup->groupID])) {
				// check if group wasn't set, but is not accessible for this user anyway
				if (!$labelGroup->getPermission($optionID)) {
					continue;
				}
		
				throw new UserInputException('labelIDs');
			}
		}
	}
	
	/**
	 * @see	wbb\system\thread\editor\IThreadEditor::saveEdit()
	 */
	public function saveEdit(array $data) {
		// log changes
		$newLabelIDs = array();
		$assignedLabels = ThreadLabelObjectHandler::getInstance()->getAssignedLabels(array($this->thread->threadID));
		foreach ($this->labelIDs as $labelID) {
			if (!isset($assignedLabels[$this->thread->threadID][$labelID])) {
				$newLabelIDs[] = $labelID;
			}
		}
		if (!empty($newLabelIDs)) {
			$labelList = new LabelList();
			$labelList->setObjectIDs($newLabelIDs);
			$labelList->readObjects();
			foreach ($labelList as $label) {
				ThreadModificationLogHandler::getInstance()->setLabel($this->thread, $label);
			}
		}
		
		// save changes
		ThreadLabelObjectHandler::getInstance()->setLabels($this->labelIDs, $this->thread->threadID);
		$assignedLabels = ThreadLabelObjectHandler::getInstance()->getAssignedLabels(array($this->thread->threadID), false);
		$editor = new ThreadEditor($this->thread);
		$editor->update(array(
			'hasLabels' => (isset($assignedLabels[$this->thread->threadID]) && !empty($assignedLabels[$this->thread->threadID])) ? 1 : 0
		));
		
		$labels = array();
		if (!empty($this->labelIDs)) {
			// get label data
			$labelList = new LabelList();
			$labelList->getConditionBuilder()->add("label.labelID IN (?)", array($this->labelIDs));
			$labelList->readObjects();
			
			foreach ($labelList as $label) {
				$labels[$label->labelID] = array(
					'cssClassName' => $label->cssClassName,
					'label' => $label->getTitle(),
					'link' => LinkHandler::getInstance()->getLink('Board', array('application' => 'wbb', 'object' => $this->thread->getBoard()), 'labelIDs['.$label->groupID.']='.$label->labelID)
				);
			}
		}
		
		return array(
			'labels' => $labels
		);
	}
}
