<?php
namespace wbb\system\worker;
use wbb\data\post\PostEditor;
use wbb\data\thread\ThreadEditor;
use wbb\data\thread\ThreadList;
use wbb\system\cache\builder\StatsCacheBuilder;
use wcf\data\object\type\ObjectTypeCache;
use wcf\data\option\OptionEditor;
use wcf\system\database\util\PreparedStatementConditionBuilder;
use wcf\system\search\SearchIndexManager;
use wcf\system\user\activity\point\UserActivityPointHandler;
use wcf\system\worker\AbstractRebuildDataWorker;
use wcf\system\WCF;

/**
 * Worker implementation for updating posts.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb
 * @subpackage	system.worker
 * @category	Burning Board
 */
class PostRebuildDataWorker extends AbstractRebuildDataWorker {
	/**
	 * @see	wcf\system\worker\AbstractRebuildDataWorker::$objectListClassName
	 */
	protected $objectListClassName = 'wbb\data\post\PostList';
	
	/**
	 * @see	wcf\system\worker\AbstractWorker::$limit
	 */
	protected $limit = 500;
	
	/**
	 * @see	wcf\system\worker\AbstractRebuildDataWorker::initObjectList
	 */
	protected function initObjectList() {
		parent::initObjectList();
		
		$this->objectList->sqlOrderBy = 'post.postID';
	}
	
	/**
	 * @see	wcf\system\worker\IWorker::execute()
	 */
	public function execute() {
		parent::execute();
		
		if (!$this->loopCount) {
			// reset activity points
			UserActivityPointHandler::getInstance()->reset('com.woltlab.wbb.activityPointEvent.post');
			
			// reset search index
			SearchIndexManager::getInstance()->reset('com.woltlab.wbb.post');
			
			// update install date
			$sql = "SELECT	MIN(time)
				FROM	wbb".WCF_N."_post";
			$statement = WCF::getDB()->prepareStatement($sql);
			$statement->execute();
			$time = $statement->fetchColumn();
				
			if ($time < WBB_INSTALL_DATE) {
				// set install date
				$sql = "UPDATE	wcf".WCF_N."_option
					SET	optionValue = ?
					WHERE	optionName = ?";
				$statement = WCF::getDB()->prepareStatement($sql);
				$statement->execute(array($time, 'wbb_install_date'));
				
				OptionEditor::resetCache();
				StatsCacheBuilder::getInstance()->reset();
			}
		}
		
		if (!count($this->objectList)) {
			return;
		}
		
		// fetch cumulative likes
		$conditions = new PreparedStatementConditionBuilder();
		$conditions->add("objectTypeID = ?", array(ObjectTypeCache::getInstance()->getObjectTypeIDByName('com.woltlab.wcf.like.likeableObject', 'com.woltlab.wbb.likeablePost')));
		$conditions->add("objectID IN (?)", array($this->objectList->getObjectIDs()));
		
		$sql = "SELECT	objectID, cumulativeLikes
			FROM	wcf".WCF_N."_like_object
			".$conditions;
		$statement = WCF::getDB()->prepareStatement($sql);
		$statement->execute($conditions->getParameters());
		$cumulativeLikes = array();
		while ($row = $statement->fetchArray()) {
			$cumulativeLikes[$row['objectID']] = $row['cumulativeLikes'];
		}
		
		// prepare statements
		$attachmentObjectType = ObjectTypeCache::getInstance()->getObjectTypeByName('com.woltlab.wcf.attachment.objectType', 'com.woltlab.wbb.post');
		$sql = "SELECT		COUNT(*) AS attachments
			FROM		wcf".WCF_N."_attachment
			WHERE		objectTypeID = ?
					AND objectID = ?";
		$attachmentStatement = WCF::getDB()->prepareStatement($sql);
		
		// read associated threads
		$threadIDs = array();
		foreach ($this->objectList as $post) {
			$threadIDs[] = $post->threadID;
		}
		
		$threadList = new ThreadList();
		$threadList->getConditionBuilder()->add("thread.threadID IN (?)", array($threadIDs));
		$threadList->readObjects();
		$threads = $threadList->getObjects();
		
		$itemsToUser = array();
		foreach ($this->objectList as $post) {
			$post->setThread($threads[$post->threadID]);
			
			$editor = new PostEditor($post);
			$data = array();
			
			// count attachments
			$attachmentStatement->execute(array($attachmentObjectType->objectTypeID, $post->postID));
			$row = $attachmentStatement->fetchArray();
			$data['attachments'] = $row['attachments'];
			
			// update cumulative likes
			$data['cumulativeLikes'] = (isset($cumulativeLikes[$post->postID])) ? $cumulativeLikes[$post->postID] : 0;
			
			$editor->update($data);
			
			if ($post->userID && $post->getThread()->getBoard()->countUserPosts) {
				if (!isset($itemsToUser[$post->userID])) {
					$itemsToUser[$post->userID] = 0;
				}
				$itemsToUser[$post->userID]++;
			}
			
			// update search index
			if ($post->getThread()->getBoard()->searchable) {
				SearchIndexManager::getInstance()->add('com.woltlab.wbb.post', $post->postID, $post->message, $post->subject, $post->time, $post->userID, $post->username, $post->getThread()->languageID);
			}
		}
		
		// update activity points
		UserActivityPointHandler::getInstance()->fireEvents('com.woltlab.wbb.activityPointEvent.post', $itemsToUser, false);
	}
}
